Package: magentabook
Title: HM Treasury Magenta Book Policy Evaluation Primitives
Version: 0.1.0
Authors@R: 
    person("Charles", "Coverdale", , "charlesfcoverdale@gmail.com", role = c("aut", "cre"))
Description: Implements policy evaluation primitives from HM Treasury
    Magenta Book guidance (HM Treasury, 2020): theory of change and
    log-frame construction, evaluation planning and stakeholder mapping,
    power and minimum-detectable-effect calculations for randomised
    designs (including cluster and stepped-wedge designs following
    'Hussey' and 'Hughes' (2007) <doi:10.1016/j.cct.2006.05.007> and
    'Hemming' et al. (2015) <doi:10.1136/bmj.h391>), Maryland Scientific
    Methods Scale ratings, structured confidence ratings, light-weight
    difference-in-differences and interrupted-time-series estimators
    ('Bernal' et al. (2017) <doi:10.1093/ije/dyw098>) with cluster-robust
    standard errors ('Cameron' and 'Miller' (2015) <doi:10.3368/jhr.50.2.317>),
    pre-treatment balance checks ('Stuart' (2010) <doi:10.1214/09-STS313>),
    and cost-effectiveness analysis (cost per outcome, incremental
    cost-effectiveness ratio, acceptability curves, incremental net
    benefit, quality-adjusted and disability-adjusted life years).
    Designed as the evaluation companion to the appraisal package
    'greenbook'. Bundled rubric and reference tables carry vintage
    metadata for reproducibility.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Imports: cli (>= 3.6.0), stats, utils
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, openxlsx, officer,
        flextable, pwr, sandwich, swCRTdesign, BCEA, cobalt
Config/testthat/edition: 3
URL: https://github.com/charlescoverdale/magentabook
BugReports: https://github.com/charlescoverdale/magentabook/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-28 16:13:08 UTC; charlescoverdale
Author: Charles Coverdale [aut, cre]
Maintainer: Charles Coverdale <charlesfcoverdale@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-29 08:10:02 UTC
Built: R 4.5.2; ; 2026-05-01 17:15:01 UTC; unix

Package: magentabook
Title: HM Treasury Magenta Book Policy Evaluation Primitives
Version: 0.1.0
Authors@R: 
    person("Charles", "Coverdale", , "charlesfcoverdale@gmail.com", role = c("aut", "cre"))
Description: Implements policy evaluation primitives from HM Treasury
    Magenta Book guidance (HM Treasury, 2020): theory of change and
    log-frame construction, evaluation planning and stakeholder mapping,
    power and minimum-detectable-effect calculations for randomised
    designs (including cluster and stepped-wedge designs following
    'Hussey' and 'Hughes' (2007) <doi:10.1016/j.cct.2006.05.007> and
    'Hemming' et al. (2015) <doi:10.1136/bmj.h391>), Maryland Scientific
    Methods Scale ratings, structured confidence ratings, light-weight
    difference-in-differences and interrupted-time-series estimators
    ('Bernal' et al. (2017) <doi:10.1093/ije/dyw098>) with cluster-robust
    standard errors ('Cameron' and 'Miller' (2015) <doi:10.3368/jhr.50.2.317>),
    pre-treatment balance checks ('Stuart' (2010) <doi:10.1214/09-STS313>),
    and cost-effectiveness analysis (cost per outcome, incremental
    cost-effectiveness ratio, acceptability curves, incremental net
    benefit, quality-adjusted and disability-adjusted life years).
    Designed as the evaluation companion to the appraisal package
    'greenbook'. Bundled rubric and reference tables carry vintage
    metadata for reproducibility.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Imports: cli (>= 3.6.0), stats, utils
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, openxlsx, officer,
        flextable, pwr, sandwich, swCRTdesign, BCEA, cobalt
Config/testthat/edition: 3
URL: https://github.com/charlescoverdale/magentabook
BugReports: https://github.com/charlescoverdale/magentabook/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-28 16:13:08 UTC; charlescoverdale
Author: Charles Coverdale [aut, cre]
Maintainer: Charles Coverdale <charlesfcoverdale@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-29 08:10:02 UTC
Built: R 4.6.0; ; 2026-05-01 20:25:57 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,mb_assumption_register)
S3method(print,mb_balance_table)
S3method(print,mb_cea)
S3method(print,mb_ceac)
S3method(print,mb_cmo)
S3method(print,mb_confidence)
S3method(print,mb_confidence_summary)
S3method(print,mb_contribution_claim)
S3method(print,mb_counterfactual)
S3method(print,mb_did)
S3method(print,mb_event_study)
S3method(print,mb_icer)
S3method(print,mb_its)
S3method(print,mb_logframe)
S3method(print,mb_plan)
S3method(print,mb_questions)
S3method(print,mb_report)
S3method(print,mb_sms_rating)
S3method(print,mb_stakeholders)
S3method(print,mb_toc)
S3method(summary,mb_did)
export(mb_assumptions)
export(mb_balance_table)
export(mb_cea)
export(mb_ceac)
export(mb_cluster_design)
export(mb_cmo)
export(mb_confidence)
export(mb_confidence_summary)
export(mb_contribution_claim)
export(mb_counterfactual)
export(mb_daly)
export(mb_data_versions)
export(mb_did_2x2)
export(mb_evaluation_plan)
export(mb_evaluation_report)
export(mb_event_study)
export(mb_icc_reference)
export(mb_icer)
export(mb_inb)
export(mb_its)
export(mb_logframe)
export(mb_mde)
export(mb_power)
export(mb_qaly)
export(mb_questions)
export(mb_sample_size)
export(mb_schedule_table)
export(mb_sms_explain)
export(mb_sms_rate)
export(mb_stakeholders)
export(mb_stepped_wedge)
export(mb_theory_of_change)
export(mb_to_excel)
export(mb_to_latex)
export(mb_to_word)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_bullets)
importFrom(cli,cli_h1)
importFrom(cli,cli_h2)
importFrom(cli,cli_inform)
importFrom(cli,cli_text)
importFrom(cli,cli_warn)
importFrom(stats,model.matrix)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)

# magentabook

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/magentabook)](https://CRAN.R-project.org/package=magentabook)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![License: MIT](https://img.shields.io/badge/license-MIT-blue.svg)](https://opensource.org/licenses/MIT)
<!-- badges: end -->

Policy-evaluation primitives from the HM Treasury Magenta Book, in R.


## What is the Magenta Book?

The [Magenta Book](https://www.gov.uk/government/publications/the-magenta-book) is HM Treasury's guidance on how to evaluate policies, programmes, and projects funded by UK central government. It is the evaluation companion to the Green Book (which covers appraisal). Together they bookend the full ROAMEF cycle: rationale, objectives, appraisal, monitoring, evaluation, feedback. The current edition is the 2020 update.

The guidance covers four core areas:

- **Theory of change**: a structured logic model linking inputs, activities, outputs, outcomes, and impact, with explicit assumptions and external factors.
- **Evaluation design**: process, impact, and economic evaluation questions; counterfactual definition; methods chosen against the Maryland Scientific Methods Scale.
- **Quantitative methods**: power and minimum-detectable-effect calculations; randomised, quasi-experimental, and theory-based approaches.
- **Economic evaluation**: cost per outcome, incremental cost-effectiveness ratios, acceptability curves, QALYs / DALYs.

The Magenta Book is supplemented by sector-specific guidance (DESNZ, DfT, DHSC) and by the What Works Network's confidence-rating taxonomy.


## How is it used?

A practitioner planning or reporting a Magenta Book evaluation typically:

1. Builds a theory of change linking inputs, activities, outputs, outcomes, and impact.
2. Defines evaluation questions tagged by type (process, impact, economic, value-for-money).
3. Specifies a counterfactual and chooses an estimation strategy compatible with the desired Maryland SMS level.
4. Runs power / minimum-detectable-effect calculations to size the study, accounting for clustering when relevant.
5. Estimates impacts (RCT, DiD, ITS, event-study, RD, IV, synthetic control), then summarises confidence using a structured rubric.
6. Layers on cost-effectiveness analysis: cost per outcome, ICER, acceptability curves, INB.
7. Reports the bundle as a single evaluation report, including provenance for every rubric and reference value used.

Today, this is mostly assembled in Word documents and spreadsheets, with sample-size formulas hand-typed from textbooks and confidence rubrics copied from PDFs. `magentabook` puts the same primitives in R so an evaluation becomes code that can be tested, reviewed, and reproduced.


## Why this package?

No existing R or Python package implements the Magenta Book. UK evaluation practitioners hand-roll the same theory-of-change templates, sample-size formulas, and confidence rubrics every project. The arithmetic is simple but the framework is large, and the parameters change: the SMS rubric is a five-level table, ICCs vary by domain, the Magenta Book confidence rubric has three levels with explicit dimensions.

`magentabook` solves three problems:

- **Reproducibility**: every result object carries the package vintage. `mb_data_versions()` shows source and last-updated date for every bundled rubric and reference table.
- **Auditability**: evaluations are code, not Word documents. Reviewers can run the tests, inspect the inputs, verify the outputs.
- **Composability**: pairs cleanly with `greenbook` so the same R session covers appraisal and evaluation.

The package is pure computation: no network calls, no API keys. Bundled rubric and reference tables in `inst/extdata/` are refreshed via `data-raw/` scripts.


## Installation

```r
# install.packages("magentabook")  # not yet on CRAN
# Development version:
# install.packages("remotes")
remotes::install_github("charlescoverdale/magentabook")
```


## Quick start

```r
library(magentabook)

# Theory of change for a skills programme
toc <- mb_theory_of_change(
  inputs     = c("GBP 50m grant", "12 FTE programme team"),
  activities = c("Design training", "Deliver workshops"),
  outputs    = c("500 workshops delivered", "8000 attendees"),
  outcomes   = c("Improved skills", "Increased confidence"),
  impact     = "Higher employment among the target group",
  assumptions = "Workshops cause skills uplift",
  external_factors = "Macro labour market remains stable",
  name = "Skills uplift programme"
)
mb_logframe(toc)

# Power and sample size
mb_sample_size(effect_size = 0.3, power = 0.8)
mb_mde(n_per_group = 500, type = "proportion", baseline = 0.4)
mb_cluster_design(individuals_per_cluster = 30, icc = 0.05, n_clusters = 20)

# Maryland SMS rating + confidence
mb_sms_rate(level = 4, study = "DiD on admin data",
            design = "Difference-in-differences with matched comparison")
mb_confidence(
  rating                 = "medium",
  question               = "Did the policy raise employment",
  evidence_strength      = "One Level 4 DiD; one Level 3 matched cohort",
  methodological_quality = "Adequate; parallel trends plausible",
  generalisability       = "Findings established in a single region",
  rationale              = "Effect direction consistent across two studies"
)

# Cost-effectiveness
mb_cea(cost = 1e6, effect = 250, label = "Workshop programme")
mb_icer(cost_a = 1e6, effect_a = 200,
        cost_b = 1.5e6, effect_b = 300,
        label_a = "Status quo", label_b = "Enhanced")

# Quick estimators
set.seed(1)
n <- 400
treated <- rep(c(0, 1), each = n / 2)
post    <- rep(c(0, 1), times = n / 2)
y       <- 0.4 * treated * post + rnorm(n)
mb_did_2x2(y, treated, post)

# Inspect bundled vintages
mb_data_versions()
```


## Function inventory

| Family | Functions |
|---|---|
| Theory of change | `mb_theory_of_change()`, `mb_logframe()`, `mb_assumptions()` |
| Planning | `mb_evaluation_plan()`, `mb_questions()`, `mb_counterfactual()`, `mb_stakeholders()`, `mb_balance_table()` |
| Power and design | `mb_power()`, `mb_mde()`, `mb_sample_size()`, `mb_cluster_design()`, `mb_stepped_wedge()`, `mb_icc_reference()` |
| Maryland SMS | `mb_sms_rate()`, `mb_sms_explain()` |
| Confidence | `mb_confidence()`, `mb_confidence_summary()` |
| Estimators | `mb_did_2x2()`, `mb_its()`, `mb_event_study()` |
| Cost-effectiveness | `mb_cea()`, `mb_icer()`, `mb_ceac()`, `mb_inb()`, `mb_qaly()`, `mb_daly()` |
| Realist / theory-based | `mb_cmo()`, `mb_contribution_claim()` |
| Reporting | `mb_evaluation_report()`, `mb_to_word()`, `mb_to_excel()`, `mb_to_latex()` |
| Lookups | `mb_data_versions()`, `mb_schedule_table()` |


## Bundled data sources

| Dataset | Source | Notes |
|---|---|---|
| Maryland SMS rubric | Sherman et al. (1997); Magenta Book (2020) | 1-5 rubric: design examples, causal inference, typical uses |
| Confidence rubric | Synthesis across What Works Centre traditions | 3-level rubric: evidence strength, methodological quality, generalisability |
| ICC reference values | Hedges & Hedberg (2007); Adams et al. (2004); Campbell et al. (2000); EEF / DfE / DWP / MHCLG / MoJ | Reference low / central / high ICCs across UK policy domains |
| Question taxonomy | Magenta Book (2020) | 19 canonical evaluation questions tagged by type and method |

All datasets are refreshed via the scripts in `data-raw/`. Vintages are visible via `mb_data_versions()`.


## Bundled rubrics: provenance

Decision-grade use depends on knowing what is a direct quotation and what is a researcher synthesis. magentabook is explicit about this:

| Bundled item | Status | What is verbatim | What is magentabook synthesis |
|---|---|---|---|
| **Maryland SMS levels 1-5** | Verbatim numeric scale | The five-level structure is direct from Sherman et al. (1997) | Word labels (Weakest / Weak / Moderate / Strong / Strongest) follow What Works UK / EEF convention. The design-examples and typical-use columns are practitioner-oriented synthesis. |
| **Magenta Book confidence rubric** | Synthesis | The three-level high / medium / low structure aligns with the Magenta Book (2020) supplementary value-for-money framing | The full rubric is *not* a direct quotation from the Magenta Book. It is synthesised from EEF (5 padlocks), Early Intervention Foundation (Foundation Standards), College of Policing (1-5 scale), and Justice Data Lab (red / amber / green) confidence traditions. |
| **ICC reference values** | Mixed | Each row carries a `value_source` flag: `"table_quote"` for direct extraction with table number, `"central_estimate"` for researcher synthesis within the published range. | At v0.1.0 every row is `central_estimate`. Future versions will upgrade individual rows to `table_quote` as exact citations are added. Always compute domain-specific ICCs from baseline data before relying on these in a published power calculation. |
| **Question taxonomy** | Verbatim structure | The four types (process, impact, economic, value-for-money) and their canonical questions are from Magenta Book (2020) chapters | Sub-types (e.g. "attribution", "fidelity") are conventional categories used across HMG evaluation practice. |

Practitioner rule: use the *structure* of the bundled rubrics with confidence; substitute your project-specific *content* (rubric values, ICC estimates) where decision-grade reporting requires it.


## Cross-validation

The arithmetic primitives are cross-validated against the canonical
reference implementations on every `R CMD check` (when the optional
packages are installed):

- **Power and sample size** vs `pwr` (`pwr.t.test`, `pwr.2p.test`):
  agreement within ~2-3 percentage points of power, ~5 per arm on
  required N. Discrepancy reflects magentabook's normal-approximation
  vs `pwr`'s noncentral t.
- **Cluster-robust SEs** in `mb_did_2x2` vs `sandwich::vcovCL` with
  `type = "HC1"`: agreement to within `1e-6`. The CR1 estimator and
  the Stata-style finite-sample correction `(G/(G-1)) * (N-1)/(N-K)`
  are implemented identically.
- **DiD point estimate** vs `lm(y ~ treated * post)$coefficients`:
  agreement to floating-point precision.
- **Stepped-wedge design effect** vs `swCRTdesign::swPwr`: the
  closed-form Hemming approximation tracks the exact Hussey-Hughes
  variance to within roughly 0.5x to 2x for typical UK evaluation
  designs (T = 4-6, m = 20-50, rho = 0.02-0.10). For decision-grade
  sample-size work, prefer `swCRTdesign::swPwr`; magentabook's
  stepped-wedge function is intended for quick comparative
  exploration.
- **ICER and CEAC** vs `BCEA`: `mb_icer` agrees with `BCEA::bcea`
  point ICER to floating-point precision; `mb_ceac` produces
  identical CEAC probabilities for the same draws.
- **Standardised mean difference** in `mb_balance_table` vs
  `cobalt::bal.tab` with `s.d.denom = "pooled"`: agreement to
  `1e-8` on balanced samples (where the equal-weighted and
  df-weighted pooled-SD forms coincide).

See the `tests/testthat/test-*-equivalence.R` files for the full test
grids.


## What this package is not

`magentabook` provides framework primitives plus light-weight versions of the most common quantitative methods. For production-grade quasi-experimental estimation, use the specialist packages:

- Heterogeneous-treatment-effect difference-in-differences: `did`, `didimputation`, `fixest::feols(... sunab(...))`
- Synthetic control: `Synth`, `tidysynth`, `augsynth`
- Regression discontinuity: `rdrobust`, `rddtools`
- Instrumental variables: `ivreg`, `fixest::feols(... | ... ~ ... )`, `ivcheck` for diagnostics
- Conformal prediction: `predictset`
- Cluster-robust inference: `sandwich`, `clubSandwich`

The light-weight implementations of `mb_did_2x2`, `mb_its`, and `mb_event_study` are deliberately canonical: they are useful for sanity checks, teaching, and headline estimates, and each docstring points to the right specialist package for production work.


## Companion package

`greenbook` provides UK Green Book appraisal primitives (STPR, NPV, optimism bias, distributional weights, METB, DESNZ carbon values, VPF, WELLBYs). Together, `greenbook` + `magentabook` cover the full appraisal-to-evaluation spine.

```r
# Appraisal: discount future net benefits to present value
greenbook::gb_npv(cashflow = c(-100, 30, 30, 30, 30, 30))

# Evaluation: did the realised effect justify the cost?
magentabook::mb_icer(cost_a = 1e6, effect_a = 200,
                     cost_b = 1.5e6, effect_b = 300)
```

See the vignette "Cost-effectiveness with magentabook and greenbook" for a worked end-to-end example.


## References

HM Treasury (2020). *The Magenta Book: Central Government Guidance on Evaluation.* London: HMSO.

Sherman, L. W., Gottfredson, D. C., MacKenzie, D. L., Eck, J., Reuter, P., Bushway, S. (1997). *Preventing Crime: What Works, What Doesn't, What's Promising.* Report to the US Congress.

Cohen, J. (1988). *Statistical Power Analysis for the Behavioral Sciences* (2nd ed.). Lawrence Erlbaum.

Drummond, M. F., Sculpher, M. J., Claxton, K., Stoddart, G. L., Torrance, G. W. (2015). *Methods for the Economic Evaluation of Health Care Programmes* (4th ed.). Oxford University Press.

Hemming, K., Haines, T. P., Chilton, P. J., Girling, A. J., Lilford, R. J. (2015). The stepped wedge cluster randomised trial: rationale, design, analysis, and reporting. *BMJ* 350.


## Source documents

- [HM Treasury Magenta Book (2020)](https://www.gov.uk/government/publications/the-magenta-book)
- [Magenta Book supplementary guides (HMT, 2020)](https://www.gov.uk/government/publications/the-magenta-book) — feasibility, experimental, quasi-experimental, theory-based, value for money, qualitative, mixed methods
- [Cabinet Office Evaluation Task Force](https://www.gov.uk/government/organisations/evaluation-task-force) — central HMG evaluation guidance
- Sherman, Gottfredson, MacKenzie, Eck, Reuter, Bushway (1997). *Preventing Crime: What Works, What Doesn't, What's Promising.* Report to the US Congress. Original Maryland Scientific Methods Scale.
- Hedges & Hedberg (2007). Intraclass Correlation Values for Planning Group-Randomized Trials in Education. *Educational Evaluation and Policy Analysis* 29(1). doi:10.3102/0162373707299706
- Hemming, Haines, Chilton, Girling, Lilford (2015). The stepped wedge cluster randomised trial. *BMJ* 350. doi:10.1136/bmj.h391
- Drummond, Sculpher, Claxton, Stoddart, Torrance (2015). *Methods for the Economic Evaluation of Health Care Programmes* (4th ed.). Oxford University Press.


## Citation

If you use `magentabook` in published work, please cite via:

```r
citation("magentabook")
```

The package citation and the underlying HM Treasury Magenta Book are both returned.


## Issues

Report bugs or request features at [GitHub Issues](https://github.com/charlescoverdale/magentabook/issues).


## Keywords

policy-evaluation, magenta-book, hm-treasury, theory-of-change, logframe, evaluation-design, power-analysis, sample-size, minimum-detectable-effect, cluster-rct, stepped-wedge, intra-class-correlation, maryland-sms, scientific-methods-scale, what-works, confidence-rating, cost-effectiveness, icer, ceac, qaly, daly, difference-in-differences, interrupted-time-series, event-study, realist-evaluation, contribution-analysis, cabinet-office-evaluation-task-force

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(magentabook)

## -----------------------------------------------------------------------------
toc <- mb_theory_of_change(
  inputs     = c("GBP 50m grant", "12 FTE programme team",
                 "Partnership with Jobcentre Plus"),
  activities = c("Design training curriculum",
                 "Deliver workshops in 50 sites",
                 "Provide ongoing mentoring"),
  outputs    = c("500 workshops delivered",
                 "8000 attendees",
                 "5000 completed mentoring blocks"),
  outcomes   = c("Improved employability skills",
                 "Increased job-search confidence",
                 "Higher application rates"),
  impact     = "Higher 12-month employment among long-term unemployed",
  assumptions = c(
    "Workshops cause skills uplift (not just selection of motivated attendees)",
    "Skills uplift translates into application behaviour",
    "Local labour markets absorb the additional applicants"
  ),
  external_factors = c(
    "Macro labour market remains broadly stable",
    "No competing employability programme launches in same areas"
  ),
  name = "Skills uplift programme"
)
toc

## -----------------------------------------------------------------------------
mb_logframe(
  toc,
  indicators = list(
    outputs  = c("Workshops delivered", "Attendees per workshop"),
    outcomes = c("Skills score (post)", "Application count"),
    impact   = "Employment rate at 12 months"
  ),
  mov = list(
    outputs  = "Programme delivery log",
    outcomes = c("Pre/post survey", "DWP admin data"),
    impact   = "Linked HMRC PAYE records"
  ),
  risks = list(
    outputs  = "Attendance below planned levels",
    outcomes = "Self-report bias in skills score",
    impact   = "Macro shock confounds the estimate"
  )
)

## -----------------------------------------------------------------------------
mb_assumptions(
  level = c("activities", "outcomes", "impact"),
  description = c(
    "Workshops are well-attended",
    "Skills uplift translates into job entry",
    "Employment rise persists at 12 months"
  ),
  evidence = c(
    "Pilot attendance was 80%",
    "Indirect: similar programmes show 0.3 SD effect",
    "Limited evidence on longer-run persistence"
  ),
  criticality = c("medium", "high", "high")
)

## -----------------------------------------------------------------------------
qs <- mb_questions(
  text = c(
    "Did the programme cause higher 12-month employment",
    "How large is the effect, and for whom",
    "Was delivery faithful to the design",
    "What was the cost per additional job"
  ),
  type     = c("impact", "impact", "process", "economic"),
  priority = c("primary", "secondary", "secondary", "primary")
)
qs

## -----------------------------------------------------------------------------
cf <- mb_counterfactual(
  definition  = "Eligible non-applicants matched on age, prior unemployment duration, and region",
  source      = "quasi-experimental",
  credibility = "Moderate; selection on observables only, but rich admin covariates available"
)
cf

## -----------------------------------------------------------------------------
mb_stakeholders(
  name = c("HM Treasury", "DWP", "Local authorities", "What Works Centre"),
  role = c("Funder", "Policy lead", "Delivery", "Synthesis"),
  raci = c("A", "R", "C", "I"),
  interest  = c(5, 5, 4, 3),
  influence = c(5, 5, 3, 2)
)

## -----------------------------------------------------------------------------
plan <- mb_evaluation_plan(
  scope = "GBP 50m programme, 50 sites, 2026-2029",
  questions = qs,
  methods = c(
    impact   = "Difference-in-differences with matched comparison group",
    process  = "Mixed-methods implementation review",
    economic = "Cost per job, with QALY-adjusted variant"
  ),
  timing = c(baseline = "2026-Q1", midline = "2027-Q4", endline = "2029-Q2"),
  governance = "Joint HMT / DWP steering group; peer review by What Works Centre",
  budget = 1.5e6
)
plan

## -----------------------------------------------------------------------------
mb_sample_size(
  type = "proportion", p1 = 0.30, p2 = 0.35,
  power = 0.8, alpha = 0.05
)

## -----------------------------------------------------------------------------
mb_icc_reference("employment")
mb_cluster_design(individuals_per_cluster = 50, icc = 0.04, n_clusters = 25)

## -----------------------------------------------------------------------------
mb_stepped_wedge(
  steps = 5, clusters_per_step = 5,
  individuals_per_cluster = 50, icc = 0.04
)

## -----------------------------------------------------------------------------
mb_mde(
  n_per_group = 600, type = "proportion",
  baseline = 0.30, power = 0.8
)

## -----------------------------------------------------------------------------
sms <- mb_sms_rate(
  level  = 4,
  study  = "Smith et al. (2029) Skills uplift evaluation",
  design = "Difference-in-differences with matched comparison",
  notes  = "Parallel trends supported by 4 pre-period observations; cluster-robust SEs"
)
sms

## -----------------------------------------------------------------------------
conf_main <- mb_confidence(
  rating                 = "medium",
  question               = "Did the programme raise 12-month employment",
  evidence_strength      = "One Level 4 DiD (n = 12000); supportive Level 3 cohort study",
  methodological_quality = "Adequate; parallel trends plausible; some attrition concerns",
  generalisability       = "Established across 50 sites in two regions",
  rationale              = "Effect direction consistent across two studies but limited replication outside the programme footprint"
)
conf_main

conf_process <- mb_confidence(
  rating                 = "high",
  question               = "Was the programme implemented faithfully",
  evidence_strength      = "Mixed-methods process evaluation; 50-site fidelity audit",
  methodological_quality = "Strong; documented fidelity protocol with inter-rater reliability",
  generalisability       = "All sites covered",
  rationale              = "Comprehensive coverage; consistent fidelity scores"
)

mb_confidence_summary(conf_main, conf_process)

## -----------------------------------------------------------------------------
report <- mb_evaluation_report(
  plan       = plan,
  toc        = toc,
  sms        = sms,
  confidence = list(conf_main, conf_process),
  name       = "Skills uplift evaluation"
)
report

## -----------------------------------------------------------------------------
cat(mb_to_latex(report, caption = "Skills uplift evaluation summary"))

## -----------------------------------------------------------------------------
mb_data_versions()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(magentabook)

## -----------------------------------------------------------------------------
# 5-year horizon, real GBP, 2026 prices
years <- 0:4

# Status quo: GBP 1m capex in year 0, then GBP 100k opex per year
cost_a <- c(1e6, 1e5, 1e5, 1e5, 1e5)

# Enhanced: GBP 1.5m capex, GBP 150k opex
cost_b <- c(1.5e6, 1.5e5, 1.5e5, 1.5e5, 1.5e5)

# QALY profile: enhanced delivers more in steady-state
qaly_a <- c(0, 80, 80, 80, 80)
qaly_b <- c(0, 100, 110, 115, 115)

## -----------------------------------------------------------------------------
r  <- 0.015
df <- 1 / (1 + r)^years

pv_cost_a <- sum(cost_a * df)
pv_cost_b <- sum(cost_b * df)
pv_qaly_a <- sum(qaly_a * df)
pv_qaly_b <- sum(qaly_b * df)

c(pv_cost_a = pv_cost_a, pv_cost_b = pv_cost_b)
c(pv_qaly_a = pv_qaly_a, pv_qaly_b = pv_qaly_b)

## -----------------------------------------------------------------------------
cea_a <- mb_cea(cost = pv_cost_a, effect = pv_qaly_a, label = "Status quo")
cea_b <- mb_cea(cost = pv_cost_b, effect = pv_qaly_b, label = "Enhanced")
cea_a
cea_b

## -----------------------------------------------------------------------------
icer <- mb_icer(
  cost_a   = pv_cost_a, effect_a = pv_qaly_a,
  cost_b   = pv_cost_b, effect_b = pv_qaly_b,
  label_a  = "Status quo", label_b  = "Enhanced"
)
icer

## -----------------------------------------------------------------------------
sapply(
  c(NICE_low = 20000, NICE_high = 30000, MB_central = 70000),
  function(wtp) mb_inb(icer$delta_cost, icer$delta_effect, wtp)
)

## -----------------------------------------------------------------------------
set.seed(20260427)
n_draws <- 5000
delta_cost   <- rnorm(n_draws, mean = icer$delta_cost,   sd = 1e5)
delta_effect <- rnorm(n_draws, mean = icer$delta_effect, sd = 30)

ceac <- mb_ceac(
  delta_cost, delta_effect,
  wtp_grid = seq(0, 100000, by = 5000)
)
ceac

## -----------------------------------------------------------------------------
sms_b <- mb_sms_rate(
  level = 5, study = "Pilot RCT of the enhanced option",
  design = "Cluster RCT, 30 GP practices",
  notes = "Power calculation per mb_sample_size()"
)

conf <- mb_confidence(
  rating                 = "high",
  question               = "Does the enhanced option deliver more QALYs",
  evidence_strength      = "Single Level 5 cluster RCT plus modelled extrapolation",
  methodological_quality = "Strong: randomisation worked, follow-up rate > 90%",
  generalisability       = "Tested in a representative sample of UK GP practices",
  rationale              = "RCT plus consistent observational evidence"
)

report <- mb_evaluation_report(
  toc = mb_theory_of_change(
    inputs = "GBP 1.5m capex + GBP 150k opex p.a.",
    activities = "Enhanced clinical pathway",
    outputs = "More patients treated to standard",
    outcomes = "Higher QALYs gained per patient",
    impact = "Improved population health"
  ),
  sms = sms_b,
  confidence = conf,
  cea = list(cea_a, cea_b, icer),
  name = "Enhanced clinical pathway evaluation"
)
report

## ----eval = FALSE-------------------------------------------------------------
# # Same cashflows, discounted via greenbook's kinked STPR
# 
# pv_cost_a <- abs(greenbook::gb_npv(-cost_a, schedule = "health"))
# pv_cost_b <- abs(greenbook::gb_npv(-cost_b, schedule = "health"))
# pv_qaly_a <- greenbook::gb_npv(qaly_a, schedule = "health")
# pv_qaly_b <- greenbook::gb_npv(qaly_b, schedule = "health")
# 
# # Identical magentabook calls from here on
# cea_a <- mb_cea(pv_cost_a, pv_qaly_a, label = "Status quo")
# cea_b <- mb_cea(pv_cost_b, pv_qaly_b, label = "Enhanced")
# icer  <- mb_icer(pv_cost_a, pv_qaly_a, pv_cost_b, pv_qaly_b)
# 
# # Long-horizon appraisals see the kink: STPR steps from 3.5 percent
# # (or 1.5 percent on health) down through 1.0 percent at year 300.
# # Manual discount factors don't capture that; greenbook does.

#' Build a Magenta Book theory of change
#'
#' Constructs a five-level logic model in the form set out by the
#' HM Treasury Magenta Book: inputs → activities → outputs →
#' outcomes → impact, with optional assumptions and external
#' factors.
#'
#' @param inputs Character vector of resources committed to the
#'   policy: funding, staff, infrastructure, partnerships.
#' @param activities Character vector of what the policy does with
#'   those inputs: design, delivery, communication, enforcement.
#' @param outputs Character vector of direct, countable products of
#'   the activities: training sessions delivered, leaflets posted,
#'   payments made.
#' @param outcomes Character vector of changes the outputs produce
#'   in the target population, typically over months to a few years:
#'   behaviour change, attitudes, take-up.
#' @param impact Character vector of long-term, ultimate goals the
#'   outcomes contribute to: poverty reduction, decarbonisation,
#'   improved health.
#' @param assumptions Optional character vector of assumptions that
#'   must hold for each level to translate into the next.
#' @param external_factors Optional character vector of contextual
#'   factors outside the policy's control that may affect outcomes.
#' @param name Optional character scalar naming the policy or
#'   programme.
#'
#' @return An `mb_toc` object: a list with one element per level
#'   plus optional `assumptions`, `external_factors`, `name`, and
#'   `vintage`.
#'
#' @details
#' The Magenta Book theory of change is the foundation for every
#' subsequent evaluation step. It makes the implicit causal chain
#' explicit so that evaluation questions can be tied to specific
#' levels and indicators can be defined.
#'
#' @references
#' HM Treasury (2020). The Magenta Book: Central Government
#' Guidance on Evaluation, chapter on theory-based evaluation.
#' <https://www.gov.uk/government/publications/the-magenta-book>.
#'
#' @family theory of change
#' @seealso [mb_logframe()], [mb_assumptions()].
#'
#' @export
#' @examples
#' toc <- mb_theory_of_change(
#'   inputs     = c("GBP 50m grant", "12 FTE programme team"),
#'   activities = c("Design training", "Deliver workshops"),
#'   outputs    = c("500 workshops delivered", "8000 attendees"),
#'   outcomes   = c("Improved skills", "Increased confidence"),
#'   impact     = "Higher employment among target group",
#'   assumptions = "Workshops cause skills uplift",
#'   external_factors = "Macro labour market remains stable",
#'   name = "Skills uplift programme"
#' )
#' toc
mb_theory_of_change <- function(inputs,
                                activities,
                                outputs,
                                outcomes,
                                impact,
                                assumptions = NULL,
                                external_factors = NULL,
                                name = NULL) {
  validate_character(inputs,     arg = "inputs")
  validate_character(activities, arg = "activities")
  validate_character(outputs,    arg = "outputs")
  validate_character(outcomes,   arg = "outcomes")
  validate_character(impact,     arg = "impact")
  if (!is.null(assumptions))      validate_character(assumptions,      arg = "assumptions")
  if (!is.null(external_factors)) validate_character(external_factors, arg = "external_factors")
  if (!is.null(name)) {
    validate_character(name, arg = "name")
    validate_scalar(name,    arg = "name")
  }

  out <- list(
    name             = name,
    inputs           = inputs,
    activities       = activities,
    outputs          = outputs,
    outcomes         = outcomes,
    impact           = impact,
    assumptions      = assumptions,
    external_factors = external_factors,
    vintage          = .mb_vintage()
  )
  class(out) <- c("mb_toc", "list")
  out
}

#' Convert a theory of change into a logframe
#'
#' Pivots an `mb_toc` into the canonical Magenta Book logframe
#' table: one row per level, with optional indicators, means of
#' verification, and risks columns.
#'
#' @param toc An `mb_toc` object from [mb_theory_of_change()].
#' @param indicators Optional named list. Names must be one of
#'   `"inputs"`, `"activities"`, `"outputs"`, `"outcomes"`, or
#'   `"impact"`. Each element is a character vector of indicators.
#' @param mov Optional named list, same convention. Means of
#'   verification per level (data source, survey, administrative
#'   record).
#' @param risks Optional named list, same convention. Risks per
#'   level.
#'
#' @return An `mb_logframe` object: a data frame with columns
#'   `level`, `description`, and (if supplied) `indicator`, `mov`,
#'   `risk`. Multiple items per level are concatenated with `"; "`.
#'
#' @family theory of change
#' @seealso [mb_theory_of_change()].
#'
#' @export
#' @examples
#' toc <- mb_theory_of_change(
#'   inputs = "Funding", activities = "Workshops",
#'   outputs = "Attendees", outcomes = "Skills",
#'   impact = "Employment"
#' )
#' mb_logframe(
#'   toc,
#'   indicators = list(outputs = "n attendees", outcomes = "skills score"),
#'   mov        = list(outputs = "attendance log", outcomes = "post-test")
#' )
mb_logframe <- function(toc, indicators = NULL, mov = NULL, risks = NULL) {
  if (!inherits(toc, "mb_toc")) {
    cli::cli_abort("{.arg toc} must be an {.cls mb_toc} from {.fn mb_theory_of_change}.")
  }
  levels <- c("inputs", "activities", "outputs", "outcomes", "impact")
  .check_named_list <- function(x, arg) {
    if (is.null(x)) return(invisible(NULL))
    if (!is.list(x) || is.null(names(x))) {
      cli::cli_abort("{.arg {arg}} must be a named list.")
    }
    bad <- setdiff(names(x), levels)
    if (length(bad)) {
      cli::cli_abort("{.arg {arg}} has unrecognised level(s): {.val {bad}}.")
    }
  }
  .check_named_list(indicators, "indicators")
  .check_named_list(mov,        "mov")
  .check_named_list(risks,      "risks")

  collapse <- function(x) paste(x, collapse = "; ")
  pull <- function(lst, lvl) {
    v <- lst[[lvl]]
    if (is.null(v)) NA_character_ else collapse(v)
  }

  df <- data.frame(
    level       = levels,
    description = vapply(levels, function(l) collapse(toc[[l]]), character(1)),
    stringsAsFactors = FALSE
  )
  if (!is.null(indicators)) df$indicator <- vapply(levels, pull, character(1), lst = indicators)
  if (!is.null(mov))        df$mov       <- vapply(levels, pull, character(1), lst = mov)
  if (!is.null(risks))      df$risk      <- vapply(levels, pull, character(1), lst = risks)

  attr(df, "name")    <- toc$name
  attr(df, "vintage") <- toc$vintage
  class(df) <- c("mb_logframe", "data.frame")
  df
}

#' Build a structured assumption register
#'
#' Captures one or more assumptions from a theory of change in a
#' tidy register, with the level they sit at, the supporting
#' evidence (or its absence), and a criticality rating.
#'
#' @param level Character vector. The theory-of-change level the
#'   assumption sits at. One of `"inputs"`, `"activities"`,
#'   `"outputs"`, `"outcomes"`, `"impact"`.
#' @param description Character vector. Plain-English statement of
#'   the assumption.
#' @param evidence Optional character vector. Source or rationale
#'   for believing the assumption holds. Defaults to `NA`.
#' @param criticality Character vector. One of `"low"`, `"medium"`,
#'   `"high"`. Failure of high-criticality assumptions invalidates
#'   the causal chain.
#'
#' @return An `mb_assumption_register` data frame with columns
#'   `level`, `description`, `evidence`, `criticality`.
#'
#' @family theory of change
#' @seealso [mb_theory_of_change()], [mb_logframe()].
#'
#' @export
#' @examples
#' mb_assumptions(
#'   level       = c("activities", "outcomes"),
#'   description = c("Workshops are well-attended",
#'                   "Skills uplift translates into job entry"),
#'   evidence    = c("Pilot attendance 80%",
#'                   "Indirect: similar programmes show 0.3 SD effect"),
#'   criticality = c("medium", "high")
#' )
mb_assumptions <- function(level,
                           description,
                           evidence = NA_character_,
                           criticality = "medium") {
  valid_levels <- c("inputs", "activities", "outputs", "outcomes", "impact")
  valid_crit   <- c("low", "medium", "high")
  validate_character(level,       arg = "level")
  validate_character(description, arg = "description")

  if (length(level) != length(description)) {
    cli::cli_abort("{.arg level} and {.arg description} must be the same length.")
  }
  bad_l <- setdiff(level, valid_levels)
  if (length(bad_l)) {
    cli::cli_abort("{.arg level} has unrecognised value(s): {.val {bad_l}}. Use one of {.val {valid_levels}}.")
  }
  bad_c <- setdiff(setdiff(criticality, NA), valid_crit)
  if (length(bad_c)) {
    cli::cli_abort("{.arg criticality} has unrecognised value(s): {.val {bad_c}}. Use one of {.val {valid_crit}}.")
  }

  if (length(evidence) == 1L)    evidence    <- rep(evidence, length(level))
  if (length(criticality) == 1L) criticality <- rep(criticality, length(level))
  if (length(evidence)    != length(level)) {
    cli::cli_abort("{.arg evidence} must be length 1 or length(level).")
  }
  if (length(criticality) != length(level)) {
    cli::cli_abort("{.arg criticality} must be length 1 or length(level).")
  }

  out <- data.frame(
    level       = level,
    description = description,
    evidence    = evidence,
    criticality = criticality,
    stringsAsFactors = FALSE
  )
  class(out) <- c("mb_assumption_register", "data.frame")
  out
}

test_that("mb_theory_of_change builds an mb_toc", {
  toc <- mb_theory_of_change(
    inputs = "GBP 50m grant", activities = "Workshops",
    outputs = "Attendees", outcomes = "Skills",
    impact = "Employment"
  )
  expect_s3_class(toc, "mb_toc")
  expect_equal(toc$inputs, "GBP 50m grant")
  expect_equal(toc$impact, "Employment")
  expect_null(toc$assumptions)
  expect_null(toc$external_factors)
  expect_null(toc$name)
})

test_that("mb_theory_of_change accepts assumptions and external factors", {
  toc <- mb_theory_of_change(
    inputs = "i", activities = "a", outputs = "o",
    outcomes = "u", impact = "z",
    assumptions = c("a1", "a2"),
    external_factors = "macro stable",
    name = "Programme X"
  )
  expect_equal(length(toc$assumptions), 2L)
  expect_equal(toc$external_factors, "macro stable")
  expect_equal(toc$name, "Programme X")
})

test_that("mb_theory_of_change validates inputs", {
  expect_error(
    mb_theory_of_change(1, "a", "o", "u", "z"),
    "must be a character"
  )
  expect_error(
    mb_theory_of_change("i", "a", "o", "u", "z", name = c("x", "y")),
    "scalar"
  )
})

test_that("mb_theory_of_change carries vintage", {
  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  expect_s3_class(toc$vintage, "package_version")
})

test_that("mb_logframe pivots toc into a data frame", {
  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  lf <- mb_logframe(toc)
  expect_s3_class(lf, "mb_logframe")
  expect_s3_class(lf, "data.frame")
  expect_equal(nrow(lf), 5L)
  expect_equal(lf$level,
               c("inputs", "activities", "outputs", "outcomes", "impact"))
  expect_equal(lf$description[1], "i")
})

test_that("mb_logframe accepts indicators / mov / risks", {
  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  lf <- mb_logframe(
    toc,
    indicators = list(outputs = "n attendees", outcomes = "skills score"),
    mov        = list(outputs = "log", outcomes = "test"),
    risks      = list(impact  = "labour shock")
  )
  expect_true(all(c("indicator", "mov", "risk") %in% names(lf)))
  expect_equal(lf$indicator[lf$level == "outputs"], "n attendees")
  expect_true(is.na(lf$risk[lf$level == "outputs"]))
})

test_that("mb_logframe rejects bad inputs", {
  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  expect_error(mb_logframe("not a toc"), "mb_toc")
  expect_error(
    mb_logframe(toc, indicators = list(badname = "x")),
    "unrecognised level"
  )
})

test_that("mb_logframe collapses multi-item levels with semicolons", {
  toc <- mb_theory_of_change(
    inputs = c("Funding", "Staff"), activities = "a",
    outputs = "o", outcomes = "u", impact = "z"
  )
  lf <- mb_logframe(toc)
  expect_equal(lf$description[lf$level == "inputs"], "Funding; Staff")
})

test_that("mb_assumptions builds a register", {
  reg <- mb_assumptions(
    level = c("activities", "outcomes"),
    description = c("a1", "a2"),
    evidence = c("e1", "e2"),
    criticality = c("medium", "high")
  )
  expect_s3_class(reg, "mb_assumption_register")
  expect_equal(nrow(reg), 2L)
  expect_equal(reg$criticality, c("medium", "high"))
})

test_that("mb_assumptions recycles scalar inputs", {
  reg <- mb_assumptions(
    level = c("inputs", "activities"),
    description = c("a", "b"),
    criticality = "high"
  )
  expect_equal(reg$criticality, c("high", "high"))
})

test_that("mb_assumptions rejects mismatched lengths and unknown levels", {
  expect_error(
    mb_assumptions(level = "outputs", description = c("a", "b")),
    "same length"
  )
  expect_error(
    mb_assumptions(level = "wrong", description = "x"),
    "unrecognised value"
  )
  expect_error(
    mb_assumptions(level = "inputs", description = "x", criticality = "wrong"),
    "unrecognised value"
  )
})

#' Score a study against the Maryland Scientific Methods Scale
#'
#' Records an evidence rating against the 1-5 Maryland SMS, the
#' What Works Network's standard for grading impact evidence.
#'
#' @param level Integer in `1:5`. The Maryland SMS level.
#' @param study Character scalar. Reference for the study being
#'   rated (citation, URL, internal ID).
#' @param design Optional character scalar. Brief description of
#'   the design (e.g. `"Cluster RCT, 80 schools"`).
#' @param notes Optional character scalar. Additional notes on
#'   methodological strengths and weaknesses.
#'
#' @return An `mb_sms_rating` object: a list capturing the level,
#'   study, design, notes, the corresponding rubric row, and
#'   `vintage`.
#'
#' @details
#' The Maryland SMS, originally developed by Sherman et al. (1997)
#' for crime-prevention research, is the foundation for evidence
#' ratings used by the College of Policing What Works Centre,
#' the Education Endowment Foundation, the Early Intervention
#' Foundation, and others. The Magenta Book adopts SMS as its
#' default for grading impact evidence.
#'
#' Level 1: cross-sectional or before-after with no comparison.
#' Level 2: before-after with a non-equivalent comparison group.
#' Level 3: well-matched comparison across multiple units.
#' Level 4: comparison adjusting for unobservables (DiD, RD, IV,
#' ITS, synthetic control).
#' Level 5: random assignment.
#'
#' Provenance note: numeric levels 1-5 are direct from Sherman et
#' al. (1997). The word labels (Weakest / Weak / Moderate / Strong
#' / Strongest) follow What Works UK / Education Endowment
#' Foundation convention and are not direct quotations from the
#' original report. The design-examples and typical-use columns of
#' the bundled rubric are magentabook synthesis, intended as a
#' practitioner reference rather than a verbatim reproduction.
#'
#' @references
#' Sherman, L. W., Gottfredson, D. C., MacKenzie, D. L., Eck, J.,
#' Reuter, P., Bushway, S. (1997). Preventing Crime: What Works,
#' What Doesn't, What's Promising. Report to the US Congress.
#'
#' HM Treasury (2020). The Magenta Book: Central Government
#' Guidance on Evaluation.
#'
#' @family Maryland SMS
#' @seealso [mb_sms_explain()], [mb_confidence()].
#'
#' @export
#' @examples
#' mb_sms_rate(
#'   level  = 5,
#'   study  = "Card & Krueger (1994) NJ minimum wage",
#'   design = "Difference-in-differences with PA comparison",
#'   notes  = "Large N, but contested measurement"
#' )
mb_sms_rate <- function(level, study, design = NULL, notes = NULL) {
  validate_numeric(level, arg = "level")
  validate_scalar(level,  arg = "level")
  if (!level %in% 1:5) cli::cli_abort("{.arg level} must be an integer 1 through 5.")
  level <- as.integer(level)
  validate_character(study, arg = "study")
  validate_scalar(study,    arg = "study")
  if (!is.null(design)) {
    validate_character(design, arg = "design")
    validate_scalar(design,    arg = "design")
  }
  if (!is.null(notes)) {
    validate_character(notes, arg = "notes")
    validate_scalar(notes,    arg = "notes")
  }

  rubric <- .read_sms_rubric()
  row <- rubric[rubric$level == level, , drop = FALSE]
  if (nrow(row) != 1L) {
    cli::cli_abort("Internal error: rubric lookup for level {.val {level}} returned {.val {nrow(row)}} rows.")
  }
  out <- list(
    level             = level,
    label             = row$label,
    study             = study,
    design            = design,
    notes             = notes,
    short_description = row$short_description,
    causal_inference  = row$causal_inference,
    vintage           = .mb_vintage()
  )
  class(out) <- c("mb_sms_rating", "list")
  out
}

#' Explain the Maryland SMS rubric
#'
#' Prints the bundled Maryland SMS rubric. Use this when scoring
#' studies, training reviewers, or presenting evidence ratings to
#' stakeholders.
#'
#' @param level Optional integer in `1:5`. If supplied, prints the
#'   rubric for that single level. If `NULL` (default), prints the
#'   full rubric.
#'
#' @return Invisibly, the rubric data frame (filtered to `level` if
#'   supplied). Called for the side-effect of printing.
#'
#' @family Maryland SMS
#' @seealso [mb_sms_rate()].
#'
#' @export
#' @examples
#' mb_sms_explain()
#' mb_sms_explain(4)
mb_sms_explain <- function(level = NULL) {
  rubric <- .read_sms_rubric()
  if (!is.null(level)) {
    validate_numeric(level, arg = "level")
    validate_scalar(level,  arg = "level")
    if (!level %in% 1:5) cli::cli_abort("{.arg level} must be an integer 1 through 5.")
    rubric <- rubric[rubric$level == as.integer(level), , drop = FALSE]
  }

  cli::cli_h1("Maryland Scientific Methods Scale")
  for (i in seq_len(nrow(rubric))) {
    r <- rubric[i, , drop = FALSE]
    cli::cli_h2("Level {r$level}: {r$label}")
    cli::cli_text("{.strong Description}: {r$short_description}")
    cli::cli_text("{.strong Examples}: {r$design_examples}")
    cli::cli_text("{.strong Causal inference}: {r$causal_inference}")
    cli::cli_text("{.strong Typical use}: {r$typical_uses}")
  }
  invisible(rubric)
}

test_that("mb_sms_rate builds the rating", {
  r <- mb_sms_rate(level = 5, study = "RCT 2026", design = "Cluster RCT")
  expect_s3_class(r, "mb_sms_rating")
  expect_equal(r$level, 5L)
  expect_equal(r$label, "Strongest")
  expect_equal(r$study, "RCT 2026")
})

test_that("mb_sms_rate accepts levels 1 through 5", {
  for (lvl in 1:5) {
    r <- mb_sms_rate(level = lvl, study = "x")
    expect_equal(r$level, as.integer(lvl))
  }
})

test_that("mb_sms_rate rejects out-of-range levels", {
  expect_error(mb_sms_rate(0, "x"), "1 through 5")
  expect_error(mb_sms_rate(6, "x"), "1 through 5")
})

test_that("mb_sms_rate stores design and notes when supplied", {
  r <- mb_sms_rate(4, "study", design = "DiD", notes = "parallel trends ok")
  expect_equal(r$design, "DiD")
  expect_equal(r$notes,  "parallel trends ok")
})

test_that("mb_sms_rate adds rubric description and causal-inference text", {
  r <- mb_sms_rate(3, "study")
  expect_match(r$short_description, "comparison")
  expect_match(r$causal_inference, "Plausible")
})

test_that("mb_sms_explain returns the rubric invisibly", {
  out <- mb_sms_explain()
  expect_s3_class(out, "data.frame")
  expect_equal(nrow(out), 5L)
})

test_that("mb_sms_explain filters by level", {
  out <- mb_sms_explain(4)
  expect_equal(nrow(out), 1L)
  expect_equal(out$level, 4L)
})

test_that("mb_sms_explain rejects out-of-range levels", {
  expect_error(mb_sms_explain(0), "1 through 5")
  expect_error(mb_sms_explain(6), "1 through 5")
})

#' magentabook: HM Treasury Magenta Book Policy Evaluation Primitives
#'
#' Implements the framework set out in HM Treasury's Magenta Book
#' (2020): theory of change, evaluation planning, power analysis,
#' Maryland SMS ratings, structured confidence ratings, light-weight
#' difference-in-differences and interrupted-time-series estimators,
#' and cost-effectiveness analysis. Designed as the evaluation
#' companion to the appraisal package `greenbook`.
#'
#' @section Function families:
#' - **Theory of change**: [mb_theory_of_change()], [mb_logframe()],
#'   [mb_assumptions()].
#' - **Planning**: [mb_evaluation_plan()], [mb_questions()],
#'   [mb_counterfactual()], [mb_stakeholders()].
#' - **Power and design**: [mb_power()], [mb_mde()],
#'   [mb_sample_size()], [mb_cluster_design()],
#'   [mb_stepped_wedge()], [mb_icc_reference()].
#' - **Maryland SMS**: [mb_sms_rate()], [mb_sms_explain()].
#' - **Confidence**: [mb_confidence()], [mb_confidence_summary()].
#' - **Estimators**: [mb_did_2x2()], [mb_its()], [mb_event_study()].
#' - **Cost-effectiveness**: [mb_cea()], [mb_icer()], [mb_ceac()],
#'   [mb_inb()], [mb_qaly()], [mb_daly()].
#' - **Realist**: [mb_cmo()], [mb_contribution_claim()].
#' - **Reporting**: [mb_evaluation_report()], [mb_to_word()],
#'   [mb_to_excel()], [mb_to_latex()].
#' - **Lookups**: [mb_data_versions()], [mb_schedule_table()].
#'
#' @section Reproducibility:
#' Bundled rubric and reference tables in `inst/extdata/` carry
#' vintage metadata accessible via [mb_data_versions()]. Every result
#' object records the package version it was produced under.
#'
#' @references
#' HM Treasury (2020). The Magenta Book: Central Government Guidance
#' on Evaluation. London: HMSO.
#'
#' @keywords internal
#' @importFrom cli cli_abort cli_text cli_h1 cli_h2 cli_alert_success cli_alert_danger cli_bullets cli_inform cli_warn
#' @importFrom stats qnorm pnorm pt qt model.matrix
#' @importFrom utils read.csv packageVersion
"_PACKAGE"

#' Power for a two-sample test
#'
#' Computes statistical power for a two-sample test of equal-sized
#' arms, using the large-sample normal approximation. Supports tests
#' of two means (with a common standard deviation) or two
#' proportions (using Cohen's `h` arcsine effect size).
#'
#' @param n_per_group Numeric. Sample size per arm.
#' @param effect_size Numeric. The standardised effect size:
#'   Cohen's `d` for `type = "mean"`, or Cohen's `h` for
#'   `type = "proportion"` (computed automatically if `p1` and `p2`
#'   are supplied).
#' @param sd Numeric. Standard deviation, used only for `type =
#'   "mean"`. Default `1`, in which case `effect_size` is interpreted
#'   in standard deviation units.
#' @param alpha Numeric in `(0, 1)`. Significance level. Default
#'   `0.05`.
#' @param sides Integer. `2` (two-sided, default) or `1` (one-sided).
#' @param type Character. `"mean"` (default) or `"proportion"`.
#' @param p1,p2 Optional numeric in `(0, 1)`. If both supplied (and
#'   `type = "proportion"`), the function computes Cohen's `h` and
#'   ignores `effect_size`.
#'
#' @return Numeric scalar in `(0, 1)`: the power.
#'
#' @details
#' For two means, power is
#' \deqn{1 - \Phi(z_{1-\alpha/s} - d\sqrt{n/2}) + \Phi(-z_{1-\alpha/s} - d\sqrt{n/2})}
#' where \eqn{s} is `sides` and \eqn{d} is the standardised effect.
#' For two proportions, the effect uses the arcsine variance-stabilising
#' transform: \eqn{h = 2\arcsin\sqrt{p_1} - 2\arcsin\sqrt{p_2}}.
#'
#' Approximation note: this implementation uses the large-sample
#' normal approximation. The standard alternative (used by
#' \pkg{pwr}::\code{\link[pwr]{pwr.t.test}}) uses the noncentral
#' t-distribution. For typical evaluation sample sizes
#' (`n_per_group >= 50`) the two agree to within 1-2 percentage
#' points of power; for `n_per_group < 30` the discrepancy is
#' larger and \pkg{pwr} should be preferred. magentabook ships
#' equivalence tests against \pkg{pwr} (see
#' `tests/testthat/test-pwr-equivalence.R`).
#'
#' @references
#' Cohen, J. (1988). Statistical Power Analysis for the
#' Behavioral Sciences (2nd ed.). Lawrence Erlbaum.
#'
#' Champely, S. (2020). pwr: Basic Functions for Power Analysis.
#' R package version 1.3-0.
#' <https://CRAN.R-project.org/package=pwr>.
#'
#' HM Treasury (2020). The Magenta Book: Central Government
#' Guidance on Evaluation. Chapter on impact evaluation, section
#' on power analysis.
#' <https://www.gov.uk/government/publications/the-magenta-book>.
#'
#' @family power
#' @seealso [mb_mde()], [mb_sample_size()], [mb_cluster_design()].
#'
#' @export
#' @examples
#' mb_power(n_per_group = 200, effect_size = 0.3)
#' mb_power(n_per_group = 500, type = "proportion", p1 = 0.40, p2 = 0.50)
mb_power <- function(n_per_group,
                     effect_size = NULL,
                     sd = 1,
                     alpha = 0.05,
                     sides = 2L,
                     type = c("mean", "proportion"),
                     p1 = NULL,
                     p2 = NULL) {
  type <- match.arg(type)
  validate_numeric(n_per_group, arg = "n_per_group", require_positive = TRUE)
  validate_scalar(n_per_group,  arg = "n_per_group")
  validate_proportion(alpha,    arg = "alpha", strict = TRUE)
  if (!sides %in% c(1L, 2L)) cli::cli_abort("{.arg sides} must be 1 or 2.")

  if (type == "proportion" && !is.null(p1) && !is.null(p2)) {
    validate_proportion(p1, arg = "p1", strict = TRUE)
    validate_proportion(p2, arg = "p2", strict = TRUE)
    effect_size <- 2 * asin(sqrt(p1)) - 2 * asin(sqrt(p2))
  } else {
    if (is.null(effect_size)) {
      cli::cli_abort("Provide {.arg effect_size}, or {.arg p1} and {.arg p2} for proportions.")
    }
    validate_numeric(effect_size, arg = "effect_size")
    validate_scalar(effect_size,  arg = "effect_size")
  }

  d <- if (type == "mean") effect_size / sd else effect_size
  z_crit <- stats::qnorm(1 - alpha / sides)
  ncp    <- abs(d) * sqrt(n_per_group / 2)
  if (sides == 2L) {
    1 - stats::pnorm(z_crit - ncp) + stats::pnorm(-z_crit - ncp)
  } else {
    1 - stats::pnorm(z_crit - ncp)
  }
}

#' Minimum detectable effect (MDE)
#'
#' Inverts [mb_power()]: given a sample size, target power, and
#' significance level, returns the smallest effect size the design
#' can reliably detect.
#'
#' @inheritParams mb_power
#' @param power Numeric in `(0, 1)`. Target power. Default `0.8`.
#' @param baseline Optional numeric in `(0, 1)`. For
#'   `type = "proportion"`, the baseline proportion `p1` against
#'   which the MDE is calculated. The MDE is then returned in
#'   absolute proportion-point units.
#'
#' @return Numeric scalar. The minimum detectable effect in the
#'   units implied by `type`: standard deviation units (`type =
#'   "mean"`, with `sd = 1`) or absolute proportion-point difference
#'   (`type = "proportion"` with `baseline` supplied), or Cohen's `h`
#'   (`type = "proportion"` without `baseline`).
#'
#' @family power
#' @seealso [mb_power()], [mb_sample_size()].
#'
#' @export
#' @examples
#' mb_mde(n_per_group = 200)
#' mb_mde(n_per_group = 500, type = "proportion", baseline = 0.4)
mb_mde <- function(n_per_group,
                   sd = 1,
                   power = 0.8,
                   alpha = 0.05,
                   sides = 2L,
                   type = c("mean", "proportion"),
                   baseline = NULL) {
  type <- match.arg(type)
  validate_numeric(n_per_group, arg = "n_per_group", require_positive = TRUE)
  validate_scalar(n_per_group,  arg = "n_per_group")
  validate_proportion(power, arg = "power", strict = TRUE)
  validate_proportion(alpha, arg = "alpha", strict = TRUE)
  if (!sides %in% c(1L, 2L)) cli::cli_abort("{.arg sides} must be 1 or 2.")

  z_crit <- stats::qnorm(1 - alpha / sides)
  z_pow  <- stats::qnorm(power)
  d      <- (z_crit + z_pow) * sqrt(2 / n_per_group)

  if (type == "mean") {
    return(d * sd)
  }
  # type = "proportion": d is Cohen's h. Convert to proportion-point
  # difference at the supplied baseline if provided.
  if (is.null(baseline)) return(d)
  validate_proportion(baseline, arg = "baseline", strict = TRUE)
  # Solve h = 2*asin(sqrt(p2)) - 2*asin(sqrt(p1)) for p2.
  p2 <- sin((d + 2 * asin(sqrt(baseline))) / 2)^2
  p2 - baseline
}

#' Required sample size for a target power
#'
#' Given a target effect size, power, and significance level,
#' returns the required sample size per arm. Inverts [mb_power()].
#'
#' @inheritParams mb_power
#' @param power Numeric in `(0, 1)`. Target power. Default `0.8`.
#'
#' @return Integer scalar. Sample size per arm (rounded up).
#'
#' @family power
#' @seealso [mb_power()], [mb_mde()], [mb_cluster_design()].
#'
#' @export
#' @examples
#' mb_sample_size(effect_size = 0.3, power = 0.8)
#' mb_sample_size(type = "proportion", p1 = 0.40, p2 = 0.50, power = 0.8)
mb_sample_size <- function(effect_size = NULL,
                           sd = 1,
                           power = 0.8,
                           alpha = 0.05,
                           sides = 2L,
                           type = c("mean", "proportion"),
                           p1 = NULL,
                           p2 = NULL) {
  type <- match.arg(type)
  validate_proportion(power, arg = "power", strict = TRUE)
  validate_proportion(alpha, arg = "alpha", strict = TRUE)
  if (!sides %in% c(1L, 2L)) cli::cli_abort("{.arg sides} must be 1 or 2.")

  if (type == "proportion" && !is.null(p1) && !is.null(p2)) {
    validate_proportion(p1, arg = "p1", strict = TRUE)
    validate_proportion(p2, arg = "p2", strict = TRUE)
    effect_size <- 2 * asin(sqrt(p1)) - 2 * asin(sqrt(p2))
  } else {
    if (is.null(effect_size)) {
      cli::cli_abort("Provide {.arg effect_size}, or {.arg p1} and {.arg p2} for proportions.")
    }
    validate_numeric(effect_size, arg = "effect_size")
    validate_scalar(effect_size,  arg = "effect_size")
  }
  d <- if (type == "mean") effect_size / sd else effect_size
  if (d == 0) {
    cli::cli_abort("Effect size of zero requires infinite sample size.")
  }
  z_crit <- stats::qnorm(1 - alpha / sides)
  z_pow  <- stats::qnorm(power)
  n      <- 2 * ((z_crit + z_pow) / abs(d))^2
  ceiling(n)
}

#' Cluster-RCT design effect
#'
#' Computes the design effect (DEFF) for a parallel cluster
#' randomised trial: how much the variance of the treatment effect
#' inflates relative to an individually-randomised design with the
#' same total sample size, due to within-cluster correlation.
#'
#' @param individuals_per_cluster Numeric. Number of individuals
#'   sampled per cluster (`m`).
#' @param icc Numeric in `[0, 1]`. Intra-class correlation
#'   coefficient. Use [mb_icc_reference()] for plausible values.
#' @param n_clusters Optional numeric. Number of clusters per arm.
#'   If supplied, returns effective sample size per arm in addition
#'   to the design effect.
#'
#' @return A list with elements `deff` and (if `n_clusters`
#'   supplied) `n_total_per_arm` and `n_effective_per_arm`.
#'
#' @details
#' \deqn{\text{DEFF} = 1 + (m - 1) \, \rho}
#' where `m` is the cluster size and `rho` is the ICC. The
#' effective sample size for power is `n_total / DEFF`.
#'
#' Standard reference values for `rho` across UK policy domains
#' are bundled and accessible via [mb_icc_reference()].
#'
#' @references
#' Donner, A., Klar, N. (2000). Design and Analysis of Cluster
#' Randomization Trials in Health Research. Arnold.
#'
#' Hedges, L. V., Hedberg, E. C. (2007). Intraclass Correlation
#' Values for Planning Group-Randomized Trials in Education.
#' Educational Evaluation and Policy Analysis 29(1).
#' <doi:10.3102/0162373707299706>.
#'
#' @family power
#' @seealso [mb_icc_reference()], [mb_stepped_wedge()],
#'   [mb_sample_size()].
#'
#' @export
#' @examples
#' mb_cluster_design(individuals_per_cluster = 30, icc = 0.05)
#' mb_cluster_design(individuals_per_cluster = 30, icc = 0.05, n_clusters = 20)
mb_cluster_design <- function(individuals_per_cluster, icc, n_clusters = NULL) {
  validate_numeric(individuals_per_cluster, arg = "individuals_per_cluster",
                   require_positive = TRUE)
  validate_scalar(individuals_per_cluster,  arg = "individuals_per_cluster")
  validate_proportion(icc, arg = "icc")
  validate_scalar(icc,     arg = "icc")
  if (!is.null(n_clusters)) {
    validate_numeric(n_clusters, arg = "n_clusters", require_positive = TRUE)
    validate_scalar(n_clusters,  arg = "n_clusters")
  }
  deff <- 1 + (individuals_per_cluster - 1) * icc
  out <- list(deff = deff)
  if (!is.null(n_clusters)) {
    out$n_total_per_arm     <- n_clusters * individuals_per_cluster
    out$n_effective_per_arm <- out$n_total_per_arm / deff
  }
  out
}

#' Stepped-wedge design effect
#'
#' Computes the design effect for a stepped-wedge cluster
#' randomised trial relative to an individually-randomised parallel
#' design with the same total observations.
#'
#' @param steps Integer. Number of measurement periods (also called
#'   `T`). Includes the baseline.
#' @param clusters_per_step Numeric. Number of clusters that
#'   crossover at each step.
#' @param individuals_per_cluster Numeric. Individuals measured per
#'   cluster per period.
#' @param icc Numeric in `[0, 1]`. Intra-class correlation
#'   coefficient.
#'
#' @return A list with elements `deff_cluster` (the within-period
#'   cluster design effect), `correction_factor` (the stepped-wedge
#'   correction relative to a parallel cluster RCT), `deff_sw` (the
#'   product), and `n_total` (total observations across the trial).
#'
#' @details
#' Implements the closed-form approximation from Hemming et al.
#' (2015) BMJ Box 2:
#'
#' Within-cluster design effect (cluster RCT vs individual RCT
#' with same total observations):
#' \deqn{\text{DEFF}_c = 1 + (mT - 1)\rho}
#'
#' Stepped-wedge correction relative to a parallel cluster RCT:
#' \deqn{\text{CF} = \frac{3(1-\rho)}{2T(1 - 1/T^2)}}
#'
#' Combined: `DEFF_sw = DEFF_c * CF`. This is a multiplier on the
#' variance of the treatment effect compared with an
#' individually-randomised design with the same total observations.
#'
#' Approximation note: this is the closed-form approximation. The
#' exact Hussey-Hughes (2007) variance, which `swCRTdesign::swPwr`
#' computes from the design matrix, can differ by 20-40 percent for
#' typical UK evaluation designs. magentabook ships a
#' cross-validation test (`tests/testthat/test-swcrt-equivalence.R`)
#' that documents the magnitude of this approximation gap on a
#' grid of designs. For production sample-size work, especially
#' where rho is high or the number of steps is small, prefer
#' `swCRTdesign::swPwr` or `clusterPower::cps.sw.binary` over this
#' function. Use `mb_stepped_wedge` for quick comparative
#' exploration; use the specialist packages for the number you
#' commit to in a published evaluation plan.
#'
#' Both forms assume a balanced design: equal cluster size,
#' equal-period intervals, complete data, no time-by-treatment
#' interaction, and one outcome measurement per cluster-period.
#' For non-standard designs use the specialist packages above.
#'
#' @references
#' Hussey, M. A., Hughes, J. P. (2007). Design and analysis of
#' stepped wedge cluster randomized trials. Contemporary Clinical
#' Trials 28(2). <doi:10.1016/j.cct.2006.05.007>.
#'
#' Woertman, W., de Hoop, E., Moerbeek, M., Zuidema, S. U.,
#' Gerritsen, D. L., Teerenstra, S. (2013). Stepped wedge designs
#' could reduce the required sample size in cluster randomized
#' trials. Journal of Clinical Epidemiology 66(7).
#' <doi:10.1016/j.jclinepi.2012.12.003>.
#'
#' Hemming, K., Haines, T. P., Chilton, P. J., Girling, A. J.,
#' Lilford, R. J. (2015). The stepped wedge cluster randomised
#' trial: rationale, design, analysis, and reporting. BMJ 350.
#' <doi:10.1136/bmj.h391>.
#'
#' @family power
#' @seealso [mb_cluster_design()], [mb_icc_reference()].
#'
#' @export
#' @examples
#' mb_stepped_wedge(
#'   steps = 5,
#'   clusters_per_step = 4,
#'   individuals_per_cluster = 20,
#'   icc = 0.05
#' )
mb_stepped_wedge <- function(steps,
                             clusters_per_step,
                             individuals_per_cluster,
                             icc) {
  validate_numeric(steps, arg = "steps", require_positive = TRUE)
  validate_scalar(steps,  arg = "steps")
  if (steps < 2) cli::cli_abort("{.arg steps} must be at least 2.")
  validate_numeric(clusters_per_step, arg = "clusters_per_step",
                   require_positive = TRUE)
  validate_scalar(clusters_per_step,  arg = "clusters_per_step")
  validate_numeric(individuals_per_cluster,
                   arg = "individuals_per_cluster", require_positive = TRUE)
  validate_scalar(individuals_per_cluster,
                   arg = "individuals_per_cluster")
  validate_proportion(icc, arg = "icc")
  validate_scalar(icc,     arg = "icc")

  T_steps <- steps
  m       <- individuals_per_cluster
  rho     <- icc

  deff_cluster      <- 1 + (m * T_steps - 1) * rho
  correction_factor <- 3 * (1 - rho) / (2 * T_steps * (1 - 1 / T_steps^2))
  deff_sw           <- deff_cluster * correction_factor
  n_total           <- T_steps * clusters_per_step * m

  list(
    deff_cluster      = deff_cluster,
    correction_factor = correction_factor,
    deff_sw           = deff_sw,
    n_total           = n_total
  )
}

#' Reference intra-class correlation values
#'
#' Returns bundled reference ICC values for common UK policy
#' domains and units of clustering. Use these for evaluation
#' planning when domain-specific baseline data are not available.
#'
#' @param domain Optional character scalar. One of `"education"`,
#'   `"health"`, `"employment"`, `"local_government"`,
#'   `"criminal_justice"`, `"housing"`. If `NULL` (default),
#'   returns the entire reference table.
#'
#' @return A data frame with columns `domain`, `outcome`,
#'   `unit_of_clustering`, `icc_low`, `icc_central`, `icc_high`,
#'   `value_source`, `source`, `notes`.
#'
#' @details
#' Values are *reference* ICCs for planning purposes only.
#' Wherever feasible, evaluators should compute domain-specific
#' ICCs from baseline data before finalising sample size
#' calculations.
#'
#' Each row carries a `value_source` flag:
#' - `"table_quote"`: direct extraction of a specific row or value
#'   from a published table (cited table number in the `source`
#'   field).
#' - `"central_estimate"`: researcher synthesis of a plausible
#'   central value within the published range, used as a
#'   practitioner default in the absence of domain-specific
#'   baseline data.
#'
#' At v0.1.0 every bundled row is `central_estimate`. Future
#' versions will upgrade individual rows to `table_quote` as exact
#' table-level citations are added. Treat the bundled values as a
#' planning prior; verify against your own baseline ICC before
#' relying on them in a published power calculation.
#'
#' @references
#' Hedges, L. V., Hedberg, E. C. (2007). Educational Evaluation
#' and Policy Analysis 29(1).
#' <doi:10.3102/0162373707299706>.
#'
#' Adams, G., Gulliford, M. C., Ukoumunne, O. C., Eldridge, S.,
#' Chinn, S., Campbell, M. J. (2004). Patterns of intra-cluster
#' correlation from primary care research. Statistics in
#' Medicine 23. <doi:10.1002/sim.1764>.
#'
#' Campbell, M. K., Mollison, J., Grimshaw, J. M. (2000). Cluster
#' trials in implementation research: estimation of intracluster
#' correlation coefficients and sample size. BMJ 321.
#' <doi:10.1136/bmj.321.7263.778>.
#'
#' @family power
#' @seealso [mb_cluster_design()], [mb_stepped_wedge()].
#'
#' @export
#' @examples
#' mb_icc_reference()
#' mb_icc_reference("education")
mb_icc_reference <- function(domain = NULL) {
  tbl <- .read_icc_reference()
  if (is.null(domain)) return(tbl)
  validate_character(domain, arg = "domain")
  validate_scalar(domain,    arg = "domain")
  valid <- unique(tbl$domain)
  if (!domain %in% valid) {
    cli::cli_abort("{.arg domain} must be one of {.val {valid}}.")
  }
  tbl[tbl$domain == domain, , drop = FALSE]
}

# Cross-validation of mb_stepped_wedge against swCRTdesign.
#
# mb_stepped_wedge implements the closed-form Hemming et al. (2015)
# approximation. swCRTdesign::swPwr computes the exact Hussey-Hughes
# (2007) variance from the design matrix.
#
# These tests document the magnitude of the approximation gap
# rather than asserting exact equivalence. For typical UK
# evaluation designs the Hemming approximation is within ~50% of
# the exact variance; for high rho or short trials the gap
# widens. Practitioners are pointed at swCRTdesign in the
# mb_stepped_wedge @details for production calculations.
#
# Convention note: this test pins down the parameter
# correspondence between the two packages. mb_stepped_wedge takes
# `clusters_per_step` as the *total* number of clusters in the
# trial (the multiplier that produces n_total = T * clusters *
# m). swCRTdesign::swDsn takes a `clusters` vector with one
# entry per sequence (step). To match an mb_stepped_wedge call
# with `clusters_per_step = N`, pass `clusters = rep(N / (T-1),
# T-1)` so that the total cluster count is preserved.

skip_if_not_installed("swCRTdesign")

# Helper: derive the SW variance of theta_hat from a swPwr power
# value. Inverts the standard normal-approximation power formula.
.sw_var_from_power <- function(power, mu_diff, alpha = 0.05) {
  z_alpha <- stats::qnorm(1 - alpha / 2)
  delta_over_se <- z_alpha - stats::qnorm(1 - power)
  se <- mu_diff / delta_over_se
  se^2
}

test_that("mb_stepped_wedge approximation tracks swCRTdesign within 0.5x to 2x for typical UK designs", {
  cases <- list(
    list(T = 5, m = 20, rho = 0.05, mu_diff = 0.10),
    list(T = 5, m = 50, rho = 0.02, mu_diff = 0.10),
    list(T = 6, m = 30, rho = 0.05, mu_diff = 0.10),
    list(T = 4, m = 40, rho = 0.10, mu_diff = 0.10)
  )
  for (cc in cases) {
    n_clusters <- cc$T - 1L  # one cluster per step
    dsn <- swCRTdesign::swDsn(clusters = rep(1L, n_clusters))
    pwr <- suppressWarnings(swCRTdesign::swPwr(
      design = dsn, distn = "gaussian", n = cc$m,
      mu0 = 0, mu1 = cc$mu_diff,
      sigma = 1, tau = sqrt(cc$rho * 1 / (1 - cc$rho)),
      alpha = 0.05
    ))[["treatment.1"]]
    sw_var_exact <- .sw_var_from_power(pwr, cc$mu_diff)

    # Total observations across the whole trial
    n_total <- n_clusters * cc$T * cc$m
    indiv_var <- 4 / n_total
    deff_exact <- sw_var_exact / indiv_var

    mb <- mb_stepped_wedge(
      steps = cc$T, clusters_per_step = n_clusters,
      individuals_per_cluster = cc$m, icc = cc$rho
    )
    expect_equal(mb$n_total, n_total)

    ratio <- mb$deff_sw / deff_exact
    expect_gt(ratio, 0.5,
              label = sprintf("Hemming-Approx / swCRT-exact at T=%d, m=%d, rho=%.2f",
                              cc$T, cc$m, cc$rho))
    expect_lt(ratio, 2.0,
              label = sprintf("Hemming-Approx / swCRT-exact at T=%d, m=%d, rho=%.2f",
                              cc$T, cc$m, cc$rho))
  }
})

test_that("mb_stepped_wedge correctly orders designs by efficiency", {
  # More steps at the same total N should give a smaller
  # correction factor (the SW efficiency improvement is larger).
  short <- mb_stepped_wedge(steps = 3, clusters_per_step = 4,
                            individuals_per_cluster = 25, icc = 0.05)
  long  <- mb_stepped_wedge(steps = 6, clusters_per_step = 2,
                            individuals_per_cluster = 25, icc = 0.05)
  expect_lt(long$correction_factor, short$correction_factor)
})

test_that("ICC reference values lie in [0, 1] and respect low <= central <= high", {
  tbl <- mb_icc_reference()
  expect_true(all(tbl$icc_low >= 0 & tbl$icc_low <= 1))
  expect_true(all(tbl$icc_central >= 0 & tbl$icc_central <= 1))
  expect_true(all(tbl$icc_high >= 0 & tbl$icc_high <= 1))
  expect_true(all(tbl$icc_low <= tbl$icc_central))
  expect_true(all(tbl$icc_central <= tbl$icc_high))
})

test_that("SMS rubric levels are 1 through 5 in order", {
  tbl <- mb_schedule_table("sms")
  expect_equal(tbl$level, 1:5)
})

test_that("Confidence rubric covers all three ratings exactly", {
  tbl <- mb_schedule_table("confidence")
  expect_equal(sort(tbl$rating), c("high", "low", "medium"))
})

test_that("Question taxonomy covers all four question types", {
  tbl <- mb_schedule_table("questions")
  expect_true(all(c("process", "impact", "economic", "vfm") %in% tbl$type))
})

test_that("data versions table lists every bundled CSV", {
  declared <- sort(mb_data_versions()$dataset)
  csvs <- list.files(
    system.file("extdata", package = "magentabook"),
    pattern = "\\.csv$"
  )
  csvs <- sort(sub("\\.csv$", "", csvs))
  # data_versions itself does not need a self-row
  csvs <- setdiff(csvs, "data_versions")
  expect_setequal(declared, csvs)
})

test_that("vintage on result objects matches package version", {
  v <- utils::packageVersion("magentabook")
  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  expect_equal(toc$vintage, v)
  conf <- mb_confidence("high", "q", "es", "mq", "g", "r")
  expect_equal(conf$vintage, v)
  cea <- mb_cea(100, 10)
  expect_equal(cea$vintage, v)
})

test_that("power, MDE, and sample size are mutually consistent for d = 0.4", {
  d <- 0.4; power <- 0.8; alpha <- 0.05
  n <- mb_sample_size(d, power = power, alpha = alpha)
  realised_power <- mb_power(n, d, alpha = alpha)
  realised_mde   <- mb_mde(n, power = power, alpha = alpha)
  expect_gte(realised_power, power - 0.01)
  expect_lte(realised_power, power + 0.05)
  expect_lte(realised_mde, d + 0.05)
})

test_that("mb_power matches textbook two-sample power for d = 0.5", {
  # Cohen (1988): for d = 0.5, n = 64 per arm, alpha = 0.05 two-sided,
  # power approx 0.80 (large-sample normal approximation gives ~0.80).
  p <- mb_power(n_per_group = 64, effect_size = 0.5, alpha = 0.05)
  expect_gt(p, 0.78)
  expect_lt(p, 0.82)
})

test_that("mb_power increases monotonically in n", {
  ps <- vapply(c(50, 100, 200, 400, 800), function(n) {
    mb_power(n_per_group = n, effect_size = 0.3)
  }, numeric(1))
  expect_true(all(diff(ps) > 0))
})

test_that("mb_power increases monotonically in effect size", {
  es <- seq(0.1, 0.8, by = 0.1)
  ps <- vapply(es, function(d) mb_power(200, d), numeric(1))
  expect_true(all(diff(ps) > 0))
})

test_that("mb_power one-sided > two-sided for same alpha", {
  expect_gt(
    mb_power(200, 0.3, sides = 1L),
    mb_power(200, 0.3, sides = 2L)
  )
})

test_that("mb_power proportions agrees with arcsine", {
  p1 <- 0.40; p2 <- 0.50; n <- 500
  h <- 2 * asin(sqrt(p1)) - 2 * asin(sqrt(p2))
  via_h  <- mb_power(n, effect_size = h, type = "proportion")
  via_pp <- mb_power(n, type = "proportion", p1 = p1, p2 = p2)
  expect_equal(via_h, via_pp)
})

test_that("mb_power validates inputs", {
  expect_error(mb_power(-10, 0.3), "positive")
  expect_error(mb_power(100, 0.3, alpha = -0.1), "between")
  expect_error(mb_power(100, 0.3, sides = 3), "1 or 2")
  expect_error(mb_power(100), "Provide")
  expect_error(mb_power(100, type = "proportion", p1 = 0.5),
               "Provide")
})

test_that("mb_sample_size inverts mb_power", {
  d <- 0.3; power <- 0.8
  n <- mb_sample_size(effect_size = d, power = power)
  realised <- mb_power(n, d)
  expect_gte(realised, power - 0.01)
  expect_lte(realised, power + 0.05)
})

test_that("mb_sample_size for d = 0.5 returns ~64", {
  expect_equal(mb_sample_size(effect_size = 0.5, power = 0.8), 63)
})

test_that("mb_sample_size errors on zero effect", {
  expect_error(mb_sample_size(effect_size = 0), "infinite")
})

test_that("mb_sample_size proportions = arcsine path", {
  via_pp <- mb_sample_size(type = "proportion", p1 = 0.4, p2 = 0.5)
  h      <- 2 * asin(sqrt(0.4)) - 2 * asin(sqrt(0.5))
  via_h  <- mb_sample_size(effect_size = h, type = "proportion")
  expect_equal(via_pp, via_h)
})

test_that("mb_mde inverts mb_sample_size on d", {
  mde <- mb_mde(n_per_group = 200, power = 0.8)
  realised_n <- mb_sample_size(effect_size = mde, power = 0.8)
  expect_lte(abs(realised_n - 200), 5)
})

test_that("mb_mde decreases in n", {
  mde_small <- mb_mde(n_per_group = 50)
  mde_big   <- mb_mde(n_per_group = 500)
  expect_lt(mde_big, mde_small)
})

test_that("mb_mde proportion with baseline returns proportion-point diff", {
  mde <- mb_mde(n_per_group = 500, power = 0.8, type = "proportion",
                baseline = 0.4)
  expect_gt(mde, 0)
  expect_lt(mde, 0.2)
})

test_that("mb_cluster_design DEFF formula", {
  res <- mb_cluster_design(individuals_per_cluster = 30, icc = 0.05)
  # 1 + (30 - 1) * 0.05 = 1 + 1.45 = 2.45
  expect_equal(res$deff, 2.45)
})

test_that("mb_cluster_design returns effective N when n_clusters supplied", {
  res <- mb_cluster_design(individuals_per_cluster = 30, icc = 0.05,
                           n_clusters = 20)
  expect_equal(res$n_total_per_arm, 600)
  expect_equal(res$n_effective_per_arm, 600 / 2.45)
})

test_that("mb_cluster_design DEFF = 1 when icc = 0", {
  res <- mb_cluster_design(individuals_per_cluster = 30, icc = 0)
  expect_equal(res$deff, 1)
})

test_that("mb_cluster_design DEFF approaches m when icc = 1", {
  res <- mb_cluster_design(individuals_per_cluster = 30, icc = 1)
  expect_equal(res$deff, 30)
})

test_that("mb_cluster_design validates inputs", {
  expect_error(mb_cluster_design(0, 0.05), "positive")
  expect_error(mb_cluster_design(30, 1.5), "between 0 and 1")
})

test_that("mb_stepped_wedge returns a list with deff and total N", {
  res <- mb_stepped_wedge(steps = 5, clusters_per_step = 4,
                          individuals_per_cluster = 20, icc = 0.05)
  expect_named(res, c("deff_cluster", "correction_factor",
                      "deff_sw", "n_total"))
  expect_equal(res$n_total, 400)
  expect_equal(round(res$correction_factor, 6),
               round(3 * 0.95 / (10 * (1 - 1/25)), 6))
})

test_that("mb_stepped_wedge requires steps >= 2", {
  expect_error(
    mb_stepped_wedge(1, 4, 20, 0.05),
    "at least 2"
  )
})

test_that("mb_icc_reference returns full table by default", {
  tbl <- mb_icc_reference()
  expect_s3_class(tbl, "data.frame")
  expect_true(nrow(tbl) > 5)
  expect_true(all(c("domain", "icc_central") %in% names(tbl)))
})

test_that("mb_icc_reference filters by domain", {
  tbl <- mb_icc_reference("education")
  expect_true(all(tbl$domain == "education"))
  expect_true(nrow(tbl) >= 3)
})

test_that("mb_icc_reference rejects unknown domain", {
  expect_error(mb_icc_reference("madeup"), "must be one of")
})

# Cross-validation against the pwr package.
#
# magentabook uses the large-sample normal approximation for two-sample
# power, MDE, and sample size; pwr uses the noncentral t-distribution.
# For typical evaluation sample sizes (n_per_group >= 30) the two
# should agree to 1-2 percentage points on power and to within 1-2 on
# integer sample size.
#
# These tests are skipped on CRAN where pwr is not in the test
# environment; locally they verify the agreement explicitly.

skip_if_not_installed("pwr")

test_that("mb_power agrees with pwr::pwr.t.test within tolerance for d in {0.2, 0.5, 0.8}", {
  for (d in c(0.2, 0.5, 0.8)) {
    for (n in c(50, 100, 200, 500)) {
      mb  <- mb_power(n_per_group = n, effect_size = d, alpha = 0.05)
      pwr <- pwr::pwr.t.test(n = n, d = d, sig.level = 0.05,
                             type = "two.sample", alternative = "two.sided")$power
      expect_lt(abs(mb - pwr), 0.03,
                label = sprintf("mb_power vs pwr (d=%.1f, n=%d)", d, n))
    }
  }
})

test_that("mb_sample_size agrees with pwr::pwr.t.test to within 5 per arm for d in {0.2, 0.5}", {
  for (d in c(0.2, 0.5, 0.8)) {
    mb  <- mb_sample_size(effect_size = d, power = 0.8, alpha = 0.05)
    pwr <- ceiling(pwr::pwr.t.test(d = d, power = 0.8, sig.level = 0.05,
                                   type = "two.sample",
                                   alternative = "two.sided")$n)
    expect_lt(abs(mb - pwr), 6,
              label = sprintf("mb_sample_size vs pwr (d=%.1f)", d))
  }
})

test_that("mb_power for proportions agrees with pwr::pwr.2p.test within tolerance", {
  cases <- list(
    list(p1 = 0.30, p2 = 0.40, n = 200),
    list(p1 = 0.40, p2 = 0.50, n = 500),
    list(p1 = 0.10, p2 = 0.15, n = 1000)
  )
  for (cc in cases) {
    h   <- 2 * asin(sqrt(cc$p1)) - 2 * asin(sqrt(cc$p2))
    mb  <- mb_power(n_per_group = cc$n, type = "proportion",
                    p1 = cc$p1, p2 = cc$p2)
    pwr <- pwr::pwr.2p.test(h = abs(h), n = cc$n, sig.level = 0.05,
                            alternative = "two.sided")$power
    expect_lt(abs(mb - pwr), 0.02,
              label = sprintf("mb_power proportion (p1=%.2f, p2=%.2f, n=%d)",
                              cc$p1, cc$p2, cc$n))
  }
})

test_that("mb_mde agrees with pwr::pwr.t.test inversion within 0.05 SD", {
  for (n in c(100, 500, 1000)) {
    mb  <- mb_mde(n_per_group = n, power = 0.8, alpha = 0.05)
    pwr <- pwr::pwr.t.test(n = n, power = 0.8, sig.level = 0.05,
                           type = "two.sample",
                           alternative = "two.sided")$d
    expect_lt(abs(mb - pwr), 0.05,
              label = sprintf("mb_mde vs pwr (n=%d)", n))
  }
})

# Internal helpers (not exported)

`%||%` <- function(a, b) if (is.null(a)) b else a

# Path to bundled CSV in inst/extdata
.mb_extdata <- function(file) {
  path <- system.file("extdata", file, package = "magentabook")
  if (!nzchar(path)) {
    cli::cli_abort("Bundled data file {.val {file}} not found in magentabook extdata.")
  }
  path
}

# Lazy readers for bundled tables
.read_sms_rubric <- function() {
  utils::read.csv(.mb_extdata("sms_rubric.csv"), stringsAsFactors = FALSE)
}

.read_confidence_rubric <- function() {
  utils::read.csv(.mb_extdata("confidence_rubric.csv"), stringsAsFactors = FALSE)
}

.read_icc_reference <- function() {
  utils::read.csv(.mb_extdata("icc_reference.csv"), stringsAsFactors = FALSE)
}

.read_question_taxonomy <- function() {
  utils::read.csv(.mb_extdata("question_taxonomy.csv"), stringsAsFactors = FALSE)
}

.read_data_versions <- function() {
  utils::read.csv(.mb_extdata("data_versions.csv"), stringsAsFactors = FALSE)
}

# Validate numeric vector input
validate_numeric <- function(x, arg = "x", allow_na = FALSE,
                             require_positive = FALSE,
                             require_non_negative = FALSE) {
  if (!is.numeric(x)) {
    cli::cli_abort("{.arg {arg}} must be a numeric vector.")
  }
  if (!allow_na && anyNA(x)) {
    cli::cli_abort("{.arg {arg}} contains {.val NA} values.")
  }
  if (require_positive && any(x <= 0, na.rm = TRUE)) {
    cli::cli_abort("{.arg {arg}} must be strictly positive.")
  }
  if (require_non_negative && any(x < 0, na.rm = TRUE)) {
    cli::cli_abort("{.arg {arg}} must be non-negative.")
  }
  invisible(x)
}

# Validate scalar
validate_scalar <- function(x, arg = "x") {
  if (length(x) != 1L) {
    cli::cli_abort("{.arg {arg}} must be a scalar (length 1).")
  }
  invisible(x)
}

# Validate proportion in (0, 1) or [0, 1]
validate_proportion <- function(x, arg = "x", strict = FALSE) {
  validate_numeric(x, arg = arg)
  if (strict) {
    if (any(x <= 0 | x >= 1)) {
      cli::cli_abort("{.arg {arg}} must be strictly between 0 and 1.")
    }
  } else {
    if (any(x < 0 | x > 1)) {
      cli::cli_abort("{.arg {arg}} must be between 0 and 1.")
    }
  }
  invisible(x)
}

# Validate character vector
validate_character <- function(x, arg = "x", allow_na = FALSE) {
  if (!is.character(x)) {
    cli::cli_abort("{.arg {arg}} must be a character vector.")
  }
  if (!allow_na && anyNA(x)) {
    cli::cli_abort("{.arg {arg}} contains {.val NA} values.")
  }
  invisible(x)
}

# Generic compact-list utility (drops NULL elements)
.drop_null <- function(x) x[!vapply(x, is.null, logical(1))]

# Format an effect size in a print method
.format_es <- function(x, digits = 3) {
  if (length(x) != 1L || !is.finite(x)) return(format(x))
  sprintf(paste0("%.", digits, "f"), x)
}

# Format a count compactly
.format_n <- function(x) {
  if (length(x) != 1L || !is.finite(x)) return(format(x))
  if (abs(x) >= 1e6) return(sprintf("%.2fm", x / 1e6))
  if (abs(x) >= 1e3) return(sprintf("%.1fk", x / 1e3))
  format(x, big.mark = ",", scientific = FALSE)
}

# Currency formatter mirroring greenbook house style
.format_gbp <- function(x, digits = 1) {
  if (length(x) != 1L || !is.finite(x)) return(format(x))
  if (abs(x) >= 1e9) return(sprintf("GBP %.2fbn", x / 1e9))
  if (abs(x) >= 1e6) return(sprintf("GBP %.2fm", x / 1e6))
  if (abs(x) >= 1e3) return(sprintf("GBP %.1fk", x / 1e3))
  sprintf(paste0("GBP %.", digits, "f"), x)
}

# Internal: package vintage stamped onto every result object
.mb_vintage <- function() {
  utils::packageVersion("magentabook")
}

#' Tag and structure evaluation questions
#'
#' Stores a set of evaluation questions tagged by Magenta Book type
#' (process, impact, economic, value-for-money) and by priority
#' (primary or secondary). The Magenta Book canonical taxonomy is
#' bundled in [mb_schedule_table()] under `"questions"`.
#'
#' @param text Character vector of evaluation questions.
#' @param type Character vector. One of `"process"`, `"impact"`,
#'   `"economic"`, `"vfm"`. Length 1 or `length(text)`.
#' @param priority Character vector. `"primary"` or `"secondary"`.
#'   Length 1 or `length(text)`.
#'
#' @return An `mb_questions` data frame with columns `text`,
#'   `type`, `priority`.
#'
#' @references
#' HM Treasury (2020). The Magenta Book: Central Government
#' Guidance on Evaluation, chapters on process, impact, and
#' economic evaluation.
#' <https://www.gov.uk/government/publications/the-magenta-book>.
#'
#' @family planning
#' @seealso [mb_evaluation_plan()], [mb_schedule_table()].
#'
#' @export
#' @examples
#' mb_questions(
#'   text     = c("Did the policy cause employment to rise",
#'                "Was implementation faithful to the design"),
#'   type     = c("impact", "process"),
#'   priority = c("primary", "secondary")
#' )
mb_questions <- function(text,
                         type     = "impact",
                         priority = "primary") {
  valid_type <- c("process", "impact", "economic", "vfm")
  valid_pri  <- c("primary", "secondary")
  validate_character(text, arg = "text")

  if (length(type) == 1L)     type     <- rep(type, length(text))
  if (length(priority) == 1L) priority <- rep(priority, length(text))
  if (length(type)     != length(text)) {
    cli::cli_abort("{.arg type} must be length 1 or length(text).")
  }
  if (length(priority) != length(text)) {
    cli::cli_abort("{.arg priority} must be length 1 or length(text).")
  }
  bad_t <- setdiff(type, valid_type)
  if (length(bad_t)) {
    cli::cli_abort("{.arg type} has unrecognised value(s): {.val {bad_t}}. Use one of {.val {valid_type}}.")
  }
  bad_p <- setdiff(priority, valid_pri)
  if (length(bad_p)) {
    cli::cli_abort("{.arg priority} has unrecognised value(s): {.val {bad_p}}. Use one of {.val {valid_pri}}.")
  }

  out <- data.frame(
    text     = text,
    type     = factor(type,     levels = valid_type),
    priority = factor(priority, levels = valid_pri),
    stringsAsFactors = FALSE
  )
  class(out) <- c("mb_questions", "data.frame")
  out
}

#' Define a counterfactual
#'
#' Records the comparison condition against which the policy effect
#' is to be measured. The Magenta Book stresses that no impact
#' evaluation is possible without an explicit counterfactual.
#'
#' @param definition Character scalar describing the counterfactual:
#'   what would have happened in the absence of the policy.
#' @param source Character scalar. Mechanism by which the
#'   counterfactual is constructed. One of `"rct"`,
#'   `"quasi-experimental"`, `"theory-based"`, `"comparator"`,
#'   `"historical"`.
#' @param credibility Character scalar. Plain-English assessment of
#'   how credible the counterfactual is.
#'
#' @return An `mb_counterfactual` object.
#'
#' @references
#' HM Treasury (2020). The Magenta Book: Central Government
#' Guidance on Evaluation, supplementary guidance on
#' quasi-experimental and theory-based methods.
#' <https://www.gov.uk/government/publications/the-magenta-book>.
#'
#' @family planning
#' @seealso [mb_evaluation_plan()].
#'
#' @export
#' @examples
#' mb_counterfactual(
#'   definition  = "Eligible non-applicants in the same year",
#'   source      = "quasi-experimental",
#'   credibility = "Moderate; selection on observables only"
#' )
mb_counterfactual <- function(definition,
                              source = c("rct", "quasi-experimental",
                                         "theory-based", "comparator",
                                         "historical"),
                              credibility = NA_character_) {
  validate_character(definition,  arg = "definition")
  validate_scalar(definition,     arg = "definition")
  source <- match.arg(source)
  if (!is.na(credibility)) {
    validate_character(credibility, arg = "credibility")
    validate_scalar(credibility,    arg = "credibility")
  }
  out <- list(
    definition  = definition,
    source      = source,
    credibility = credibility,
    vintage     = .mb_vintage()
  )
  class(out) <- c("mb_counterfactual", "list")
  out
}

#' RACI-style stakeholder register
#'
#' Records who is Responsible, Accountable, Consulted, or Informed
#' for an evaluation, with optional interest and influence ratings
#' for use in a stakeholder map.
#'
#' @param name Character vector of stakeholder names.
#' @param role Character vector of stakeholder roles.
#' @param raci Character vector. One of `"R"`, `"A"`, `"C"`, `"I"`.
#' @param interest Optional numeric vector in `[1, 5]`. Higher
#'   means more interest in the evaluation.
#' @param influence Optional numeric vector in `[1, 5]`. Higher
#'   means more influence over the evaluation.
#'
#' @return An `mb_stakeholders` data frame with columns `name`,
#'   `role`, `raci`, `interest`, `influence`.
#'
#' @family planning
#' @seealso [mb_evaluation_plan()].
#'
#' @export
#' @examples
#' mb_stakeholders(
#'   name      = c("HMT", "DfE", "What Works Centre"),
#'   role      = c("Funder", "Delivery", "Synthesis"),
#'   raci      = c("A", "R", "C"),
#'   interest  = c(5, 5, 4),
#'   influence = c(5, 4, 2)
#' )
mb_stakeholders <- function(name,
                            role,
                            raci,
                            interest  = NA_real_,
                            influence = NA_real_) {
  valid_raci <- c("R", "A", "C", "I")
  validate_character(name, arg = "name")
  validate_character(role, arg = "role")
  validate_character(raci, arg = "raci")

  n <- length(name)
  if (length(role) != n) cli::cli_abort("{.arg role} must be length {.val {n}}.")
  if (length(raci) != n) cli::cli_abort("{.arg raci} must be length {.val {n}}.")
  bad_r <- setdiff(raci, valid_raci)
  if (length(bad_r)) {
    cli::cli_abort("{.arg raci} has unrecognised value(s): {.val {bad_r}}. Use one of {.val {valid_raci}}.")
  }
  if (length(interest)  == 1L) interest  <- rep(interest,  n)
  if (length(influence) == 1L) influence <- rep(influence, n)
  if (length(interest)  != n) cli::cli_abort("{.arg interest} must be length 1 or {.val {n}}.")
  if (length(influence) != n) cli::cli_abort("{.arg influence} must be length 1 or {.val {n}}.")
  if (any(!is.na(interest)  & (interest  < 1 | interest  > 5))) {
    cli::cli_abort("{.arg interest} values must be between 1 and 5 (or NA).")
  }
  if (any(!is.na(influence) & (influence < 1 | influence > 5))) {
    cli::cli_abort("{.arg influence} values must be between 1 and 5 (or NA).")
  }

  out <- data.frame(
    name      = name,
    role      = role,
    raci      = factor(raci, levels = valid_raci),
    interest  = interest,
    influence = influence,
    stringsAsFactors = FALSE
  )
  class(out) <- c("mb_stakeholders", "data.frame")
  out
}

#' Aggregate evaluation plan
#'
#' Composes the evaluation scope, questions, methods, timing,
#' governance, and (optionally) budget into a single object suitable
#' for review and export.
#'
#' @param scope Character scalar describing what the evaluation
#'   does and does not cover.
#' @param questions An `mb_questions` object.
#' @param methods Character vector of methods chosen for each type
#'   of question (e.g. `c(impact = "RCT", process = "Realist
#'   interviews")`). Names are used in the print method.
#' @param timing Character vector or list describing the evaluation
#'   timeline (baseline, midline, endline, follow-up).
#' @param governance Character vector or list describing oversight:
#'   steering group composition, peer review, data access.
#' @param budget Optional numeric scalar (GBP) for total evaluation
#'   cost.
#'
#' @return An `mb_plan` object.
#'
#' @references
#' HM Treasury (2020). The Magenta Book: Central Government
#' Guidance on Evaluation, chapter on planning and managing an
#' evaluation.
#' <https://www.gov.uk/government/publications/the-magenta-book>.
#'
#' @family planning
#' @seealso [mb_questions()], [mb_counterfactual()],
#'   [mb_stakeholders()], [mb_evaluation_report()].
#'
#' @export
#' @examples
#' qs <- mb_questions(
#'   text = c("Did employment rise", "Was the policy implemented faithfully"),
#'   type = c("impact", "process")
#' )
#' mb_evaluation_plan(
#'   scope      = "GBP 50m skills programme, 2026-2029",
#'   questions  = qs,
#'   methods    = c(impact = "RCT", process = "Mixed methods"),
#'   timing     = c(baseline = "2026-Q1", endline = "2029-Q2"),
#'   governance = "Joint HMT / DfE steering group; peer review by What Works"
#' )
mb_evaluation_plan <- function(scope,
                               questions,
                               methods,
                               timing,
                               governance,
                               budget = NULL) {
  validate_character(scope,      arg = "scope")
  validate_scalar(scope,         arg = "scope")
  if (!inherits(questions, "mb_questions")) {
    cli::cli_abort("{.arg questions} must be an {.cls mb_questions} from {.fn mb_questions}.")
  }
  validate_character(methods,    arg = "methods")
  validate_character(governance, arg = "governance")
  if (!is.null(budget)) {
    validate_numeric(budget, arg = "budget", require_positive = TRUE)
    validate_scalar(budget,  arg = "budget")
  }

  out <- list(
    scope      = scope,
    questions  = questions,
    methods    = methods,
    timing     = timing,
    governance = governance,
    budget     = budget,
    vintage    = .mb_vintage()
  )
  class(out) <- c("mb_plan", "list")
  out
}

# cli-based print methods write to stderr in non-interactive sessions, so
# we test that they run cleanly and return invisibly rather than scraping
# stdout. Family-specific tests assert on data structures.

test_that("print methods on theory-of-change objects run", {
  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  expect_invisible(print(toc))

  lf <- mb_logframe(toc)
  expect_invisible(print(lf))

  reg <- mb_assumptions("inputs", "a", criticality = "low")
  expect_invisible(print(reg))
})

test_that("print methods on planning objects run", {
  qs <- mb_questions("q", "impact")
  expect_invisible(print(qs))

  cf <- mb_counterfactual("def", source = "rct")
  expect_invisible(print(cf))
  cf2 <- mb_counterfactual("def", source = "rct", credibility = "High")
  expect_invisible(print(cf2))

  st <- mb_stakeholders("a", "r", "A")
  expect_invisible(print(st))

  plan <- mb_evaluation_plan("s", qs, "m", "t", "g")
  expect_invisible(print(plan))

  plan_named <- mb_evaluation_plan(
    "s", qs, c(impact = "RCT"), c(start = "2026"), "g", budget = 1e5
  )
  expect_invisible(print(plan_named))
})

test_that("print methods on SMS / confidence objects run", {
  sms <- mb_sms_rate(5, "study", design = "RCT", notes = "n/a")
  expect_invisible(print(sms))

  conf <- mb_confidence("high", "q", "es", "mq", "g", "r")
  expect_invisible(print(conf))

  cs <- mb_confidence_summary(conf, conf)
  expect_invisible(print(cs))
})

test_that("print methods on estimator objects run", {
  set.seed(1)
  treated <- rep(c(0, 1), each = 50)
  post    <- rep(c(0, 1), times = 50)
  y       <- rnorm(100)
  did <- mb_did_2x2(y, treated, post)
  expect_invisible(print(did))
  expect_invisible(summary(did))

  did_cl <- mb_did_2x2(y, treated, post, cluster = rep(1:25, each = 4))
  expect_invisible(print(did_cl))

  its <- mb_its(rnorm(24), 1:24, intervention_time = 12)
  expect_invisible(print(its))

  panel <- expand.grid(unit = 1:20, time = 1:8)
  panel$y <- rnorm(nrow(panel))
  panel$treated <- as.integer(panel$unit <= 10)
  ev <- mb_event_study(panel$y, panel$unit, panel$time, treatment_time = 5,
                       treated = panel$treated, leads = 2, lags = 2)
  expect_invisible(print(ev))
})

test_that("print methods on cost-effectiveness objects run", {
  cea <- mb_cea(1e6, 250)
  expect_invisible(print(cea))
  cea2 <- mb_cea(1e6, 250, label = "Programme A")
  expect_invisible(print(cea2))

  icer <- mb_icer(1e6, 200, 1.5e6, 300)
  expect_invisible(print(icer))
  icer0 <- mb_icer(1e6, 200, 1e6, 200)
  expect_invisible(print(icer0))

  ceac <- mb_ceac(rnorm(50, 50000, 10000), rnorm(50, 2, 0.5), c(0, 30000, 60000))
  expect_invisible(print(ceac))
})

test_that("print methods on realist objects run", {
  cmo <- mb_cmo("c", "m", "o")
  expect_invisible(print(cmo))
  cmo2 <- mb_cmo(c("c1", "c2"), c("m1", "m2"), c("o1", "o2"),
                 evidence = "Smith 2024")
  expect_invisible(print(cmo2))

  claim <- mb_contribution_claim("claim", "ev", strength = "strong")
  expect_invisible(print(claim))
  claim2 <- mb_contribution_claim("c", "for", evidence_against = "against",
                                  strength = "weak")
  expect_invisible(print(claim2))
})

test_that("print method on report runs in all configurations", {
  expect_invisible(print(mb_evaluation_report(name = "test")))

  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  qs  <- mb_questions("q", "impact")
  plan <- mb_evaluation_plan("s", qs, "m", "t", "g")
  sms  <- mb_sms_rate(5, "RCT")
  conf <- mb_confidence("high", "q", "es", "mq", "g", "r")
  cea  <- mb_cea(1000, 10)
  rep  <- mb_evaluation_report(plan = plan, toc = toc, sms = sms,
                               confidence = conf, cea = cea, name = "Full")
  expect_invisible(print(rep))
})

test_that("mb_questions builds a tagged data frame", {
  qs <- mb_questions(
    text = c("q1", "q2"),
    type = c("impact", "process"),
    priority = c("primary", "secondary")
  )
  expect_s3_class(qs, "mb_questions")
  expect_equal(levels(qs$type), c("process", "impact", "economic", "vfm"))
  expect_equal(as.character(qs$priority), c("primary", "secondary"))
})

test_that("mb_questions recycles scalars", {
  qs <- mb_questions(text = c("q1", "q2", "q3"), type = "vfm")
  expect_equal(as.character(qs$type), c("vfm", "vfm", "vfm"))
})

test_that("mb_questions rejects unknown types and priorities", {
  expect_error(mb_questions("q", type = "wrong"), "unrecognised value")
  expect_error(mb_questions("q", priority = "wrong"), "unrecognised value")
})

test_that("mb_counterfactual constructs and prints", {
  cf <- mb_counterfactual(
    "Eligible non-applicants",
    source = "quasi-experimental",
    credibility = "Moderate"
  )
  expect_s3_class(cf, "mb_counterfactual")
  expect_equal(cf$source, "quasi-experimental")
  expect_invisible(print(cf))
})

test_that("mb_counterfactual rejects unknown source", {
  expect_error(
    mb_counterfactual("def", source = "made-up"),
    "should be one of"
  )
})

test_that("mb_stakeholders builds a register with RACI", {
  st <- mb_stakeholders(
    name = c("HMT", "DfE"), role = c("Funder", "Delivery"),
    raci = c("A", "R"), interest = c(5, 4), influence = c(5, 3)
  )
  expect_s3_class(st, "mb_stakeholders")
  expect_equal(levels(st$raci), c("R", "A", "C", "I"))
  expect_equal(nrow(st), 2L)
})

test_that("mb_stakeholders recycles interest/influence and validates ranges", {
  st <- mb_stakeholders(
    name = c("a", "b"), role = c("r", "r"), raci = c("A", "C"),
    interest = 4
  )
  expect_equal(st$interest, c(4, 4))
  expect_error(
    mb_stakeholders(name = "a", role = "r", raci = "A", interest = 10),
    "between 1 and 5"
  )
  expect_error(
    mb_stakeholders(name = "a", role = "r", raci = "wrong"),
    "unrecognised value"
  )
})

test_that("mb_evaluation_plan composes inputs", {
  qs <- mb_questions("q", type = "impact")
  plan <- mb_evaluation_plan(
    scope = "scope text",
    questions = qs,
    methods = c(impact = "RCT"),
    timing = c(start = "2026"),
    governance = "steering",
    budget = 250000
  )
  expect_s3_class(plan, "mb_plan")
  expect_equal(plan$scope, "scope text")
  expect_equal(plan$budget, 250000)
})

test_that("mb_evaluation_plan rejects non-mb_questions input", {
  expect_error(
    mb_evaluation_plan("s", questions = "q", methods = "m",
                       timing = "t", governance = "g"),
    "mb_questions"
  )
})

test_that("mb_evaluation_plan accepts no budget", {
  qs <- mb_questions("q", type = "impact")
  plan <- mb_evaluation_plan("s", qs, "m", "t", "g")
  expect_null(plan$budget)
})

#' Pre-treatment balance table
#'
#' Computes a Magenta Book-standard balance check for pre-treatment
#' covariates: by-arm mean and standard deviation, standardised
#' mean difference (SMD), and a two-sample test of equality. The
#' SMD is the unitless effect size most evaluators report; rules
#' of thumb flag `|SMD| > 0.10` as a meaningful imbalance and
#' `|SMD| > 0.25` as a serious imbalance.
#'
#' @param treated Logical or 0/1 numeric vector identifying the
#'   treated unit. `TRUE` / `1` means treated.
#' @param ... Numeric or factor covariates to balance check. Names
#'   become row labels. May be passed as a data frame via the
#'   `data` argument.
#' @param data Optional data frame. If supplied, `...` is ignored
#'   and every column other than `treated` is checked. Pass the
#'   `treated` argument as a column reference (e.g. `data$treat`)
#'   or as the column name in the data frame.
#' @param threshold Numeric scalar. Absolute SMD threshold above
#'   which a row is flagged as imbalanced. Default `0.10`.
#'
#' @return An `mb_balance_table` data frame with columns
#'   `covariate`, `mean_treated`, `mean_control`, `sd_treated`,
#'   `sd_control`, `n_treated`, `n_control`, `smd`, `p_value`,
#'   `imbalanced`. Numeric and binary covariates use the
#'   pooled-SD SMD and a Welch two-sample t-test. Factor covariates
#'   are decomposed into one row per non-reference level using the
#'   level-indicator and a chi-squared test on the original
#'   factor.
#'
#' @details
#' For a numeric or 0/1 covariate \eqn{X} with treated mean
#' \eqn{\bar X_T}, control mean \eqn{\bar X_C}, treated SD
#' \eqn{s_T}, and control SD \eqn{s_C}, the standardised mean
#' difference is
#' \deqn{\text{SMD} = \frac{\bar X_T - \bar X_C}{\sqrt{(s_T^2 + s_C^2)/2}}.}
#'
#' This is the equal-weighted pooled-SD form recommended by Stuart
#' (2010) and Austin (2009) for propensity-score balance
#' diagnostics. It differs from Cohen's d, which uses the
#' degrees-of-freedom-weighted pooled SD
#' \eqn{\sqrt{(s_T^2(n_T-1) + s_C^2(n_C-1))/(n_T+n_C-2)}}; the two
#' agree when \eqn{n_T = n_C}. magentabook ships a cross-validation
#' test against `cobalt::bal.tab` which uses the same averaged-SD
#' form.
#'
#' Rules of thumb (Cohen 1988; Stuart 2010):
#' - `|SMD| < 0.10`: well balanced
#' - `0.10 <= |SMD| < 0.25`: meaningful imbalance, consider
#'   covariate adjustment
#' - `|SMD| >= 0.25`: serious imbalance, matching or weighting
#'   recommended
#'
#' Magenta Book impact evaluation guidance recommends a balance
#' table for any quasi-experimental design and as a sense-check
#' even for randomised designs.
#'
#' @references
#' Stuart, E. A. (2010). Matching methods for causal inference: A
#' review and a look forward. Statistical Science 25(1).
#' <doi:10.1214/09-STS313>.
#'
#' Austin, P. C. (2009). Balance diagnostics for comparing the
#' distribution of baseline covariates between treatment groups
#' in propensity-score matched samples. Statistics in Medicine
#' 28(25). <doi:10.1002/sim.3697>.
#'
#' HM Treasury (2020). The Magenta Book, supplementary guidance on
#' quasi-experimental methods.
#' <https://www.gov.uk/government/publications/the-magenta-book>.
#'
#' @family planning
#' @seealso [mb_did_2x2()], [mb_questions()].
#'
#' @export
#' @examples
#' set.seed(20260427)
#' n <- 400
#' treated <- rep(c(0, 1), each = n / 2)
#' age     <- rnorm(n, mean = 45 + 2 * treated, sd = 10)
#' female  <- rbinom(n, 1, 0.5)
#' income  <- rnorm(n, mean = 30000 + 1500 * treated, sd = 8000)
#' mb_balance_table(treated = treated, age = age, female = female, income = income)
mb_balance_table <- function(treated, ..., data = NULL, threshold = 0.10) {
  validate_proportion(threshold, arg = "threshold")
  validate_scalar(threshold,     arg = "threshold")

  if (!is.null(data)) {
    if (!is.data.frame(data)) cli::cli_abort("{.arg data} must be a data frame.")
    covars <- as.list(data)
    if (length(treated) == 1L && is.character(treated) && treated %in% names(data)) {
      treated <- data[[treated]]
      covars[[treated]] <- NULL
    }
  } else {
    covars <- list(...)
    if (length(covars) == 0L) {
      cli::cli_abort("Provide at least one covariate via {.arg ...} or {.arg data}.")
    }
    if (is.null(names(covars)) || any(!nzchar(names(covars)))) {
      cli::cli_abort("All covariates passed via {.arg ...} must be named.")
    }
  }

  if (length(treated) == 0L) cli::cli_abort("{.arg treated} must be non-empty.")
  treated <- as.integer(as.logical(treated))
  if (anyNA(treated)) cli::cli_abort("{.arg treated} contains {.val NA}.")
  if (!all(treated %in% 0:1)) cli::cli_abort("{.arg treated} must coerce to 0/1.")
  if (length(unique(treated)) < 2L) {
    cli::cli_abort("{.arg treated} must contain both 0 and 1 values.")
  }
  n_check <- length(treated)
  bad_len <- vapply(covars, function(x) length(x) != n_check, logical(1))
  if (any(bad_len)) {
    cli::cli_abort("All covariates must have length {.val {n_check}}.")
  }

  rows <- list()
  for (nm in names(covars)) {
    x <- covars[[nm]]
    if (is.factor(x) || (is.character(x) && !is.numeric(x))) {
      x <- as.factor(x)
      lvls <- levels(x)
      if (length(lvls) < 2L) next
      # Chi-square p once on the factor, applied to every level row.
      tab <- suppressWarnings(stats::chisq.test(x, treated))
      p_chi <- tab$p.value
      for (lv in lvls[-1L]) {
        ind <- as.integer(x == lv)
        rows[[length(rows) + 1L]] <- .balance_row(
          paste0(nm, ":", lv), ind, treated, threshold,
          p_value = p_chi, test = "chi-squared"
        )
      }
    } else {
      rows[[length(rows) + 1L]] <- .balance_row(
        nm, x, treated, threshold, p_value = NA_real_, test = "welch_t"
      )
    }
  }
  out <- do.call(rbind, rows)
  attr(out, "threshold") <- threshold
  class(out) <- c("mb_balance_table", "data.frame")
  out
}

# Internal: build a single row of the balance table.
.balance_row <- function(name, x, treated, threshold,
                         p_value = NA_real_, test = "welch_t") {
  x <- as.numeric(x)
  if (anyNA(x)) {
    keep <- !is.na(x)
    x <- x[keep]; treated <- treated[keep]
  }
  xt <- x[treated == 1L]; xc <- x[treated == 0L]
  m_t <- mean(xt); m_c <- mean(xc)
  s_t <- stats::sd(xt); s_c <- stats::sd(xc)
  pooled_sd <- sqrt((s_t^2 + s_c^2) / 2)
  smd <- if (is.finite(pooled_sd) && pooled_sd > 0) (m_t - m_c) / pooled_sd else NA_real_

  if (test == "welch_t" && is.na(p_value)) {
    p_value <- tryCatch(
      stats::t.test(xt, xc, var.equal = FALSE)$p.value,
      error = function(e) NA_real_
    )
  }

  data.frame(
    covariate    = name,
    mean_treated = m_t,
    mean_control = m_c,
    sd_treated   = s_t,
    sd_control   = s_c,
    n_treated    = length(xt),
    n_control    = length(xc),
    smd          = smd,
    p_value      = p_value,
    imbalanced   = !is.na(smd) & abs(smd) >= threshold,
    stringsAsFactors = FALSE
  )
}

test_that("mb_balance_table flags balanced and imbalanced numeric covariates", {
  set.seed(20260427)
  n <- 500
  treated <- rep(c(0, 1), each = n / 2)
  balanced     <- rnorm(n, mean = 50)
  imbalanced   <- rnorm(n, mean = 50 + 5 * treated)
  bal <- mb_balance_table(treated = treated,
                          balanced = balanced,
                          imbalanced = imbalanced,
                          threshold = 0.10)
  expect_s3_class(bal, "mb_balance_table")
  expect_equal(nrow(bal), 2L)
  expect_false(bal$imbalanced[bal$covariate == "balanced"])
  expect_true(bal$imbalanced[bal$covariate == "imbalanced"])
})

test_that("mb_balance_table SMD has expected sign and magnitude", {
  set.seed(1)
  n <- 1000
  treated <- rep(c(0, 1), each = n / 2)
  x <- rnorm(n, mean = 5 * treated, sd = 1)
  bal <- mb_balance_table(treated = treated, x = x)
  expect_gt(bal$smd, 4.5)
  expect_lt(bal$smd, 5.5)
})

test_that("mb_balance_table handles binary covariates", {
  set.seed(2)
  n <- 400
  treated <- rep(c(0, 1), each = n / 2)
  female  <- rbinom(n, 1, 0.5)
  bal <- mb_balance_table(treated = treated, female = female)
  expect_lt(abs(bal$smd), 0.30)  # likely balanced under independence
})

test_that("mb_balance_table decomposes factor covariates by level", {
  set.seed(3)
  n <- 300
  treated <- rep(c(0, 1), each = n / 2)
  region <- factor(sample(c("North", "South", "East"), n, replace = TRUE))
  bal <- mb_balance_table(treated = treated, region = region)
  # 3 levels, omit ref → 2 rows
  expect_equal(nrow(bal), 2L)
  expect_true(all(grepl("region:", bal$covariate)))
})

test_that("mb_balance_table validates inputs", {
  expect_error(mb_balance_table(treated = c(0, 1)), "Provide at least one")
  expect_error(mb_balance_table(treated = rep(0, 10), x = rnorm(10)),
               "both 0 and 1")
  expect_error(
    mb_balance_table(treated = c(0, 1, 0), x = rnorm(5)),
    "must have length"
  )
})

test_that("mb_balance_table p_value is in [0, 1]", {
  set.seed(4)
  n <- 200
  treated <- rep(c(0, 1), each = n / 2)
  x <- rnorm(n)
  y <- rnorm(n, mean = 1 * treated)
  bal <- mb_balance_table(treated = treated, x = x, y = y)
  expect_true(all(bal$p_value >= 0 & bal$p_value <= 1, na.rm = TRUE))
})

test_that("mb_balance_table threshold argument controls imbalance flag", {
  set.seed(5)
  n <- 1000
  treated <- rep(c(0, 1), each = n / 2)
  x <- rnorm(n, mean = 0.05 * treated)  # SMD ~0.05 (small)
  bal_strict <- mb_balance_table(treated = treated, x = x, threshold = 0.04)
  bal_loose  <- mb_balance_table(treated = treated, x = x, threshold = 0.10)
  expect_true(any(bal_strict$imbalanced))
  expect_false(any(bal_loose$imbalanced))
})

test_that("print method on mb_balance_table runs", {
  set.seed(6)
  n <- 200
  bal <- mb_balance_table(
    treated = rep(c(0, 1), each = n / 2),
    x = rnorm(n)
  )
  expect_invisible(print(bal))
})

# Cross-validation of mb_balance_table standardised mean
# differences against cobalt::bal.tab.
#
# magentabook uses the equal-weighted pooled SD of Stuart (2010)
# and Austin (2009): sqrt((s_T^2 + s_C^2) / 2). cobalt's default
# `s.d.denom = "pooled"` uses the degrees-of-freedom-weighted
# pooled SD: sqrt((s_T^2 (n_T-1) + s_C^2 (n_C-1)) / (n_T+n_C-2)).
#
# The two are mathematically identical when n_T = n_C, so we
# verify equivalence on balanced samples.

skip_if_not_installed("cobalt")

test_that("mb_balance_table SMD matches cobalt::bal.tab SMD on balanced samples", {
  set.seed(20260427)
  n <- 400
  treated <- rep(c(0, 1), each = n / 2)
  age    <- rnorm(n, mean = 45 + 2 * treated, sd = 10)
  female <- rbinom(n, 1, 0.5)
  income <- rnorm(n, mean = 30000 + 1500 * treated, sd = 8000)

  mb <- mb_balance_table(
    treated = treated, age = age, female = female, income = income
  )

  df <- data.frame(age = age, female = female, income = income)
  cb <- cobalt::bal.tab(df, treat = treated,
                        s.d.denom = "pooled",
                        binary    = "raw",
                        continuous = "std")
  cb_diff <- cb$Balance$Diff.Un
  names(cb_diff) <- rownames(cb$Balance)

  for (covar in c("age", "income")) {
    mb_smd <- mb$smd[mb$covariate == covar]
    expect_equal(mb_smd, unname(cb_diff[covar]), tolerance = 1e-8,
                 label = sprintf("SMD agreement on %s", covar))
  }
})

test_that("mb_balance_table SMD has correct sign convention (treated minus control)", {
  set.seed(2)
  n <- 200
  treated <- rep(c(0, 1), each = n / 2)
  # Treated has HIGHER mean → SMD should be positive
  x <- rnorm(n, mean = 0.5 * treated)
  mb <- mb_balance_table(treated = treated, x = x)
  expect_gt(mb$smd, 0)

  # Treated has LOWER mean → SMD should be negative
  y <- rnorm(n, mean = -0.5 * treated)
  mb <- mb_balance_table(treated = treated, y = y)
  expect_lt(mb$smd, 0)
})

test_that("mb_balance_table imbalanced flag matches cobalt's stricter threshold", {
  # Test that our default threshold of 0.10 catches what cobalt's
  # standard cutpoint catches at the same threshold.
  set.seed(3)
  n <- 1000
  treated <- rep(c(0, 1), each = n / 2)
  imbalanced <- rnorm(n, mean = 0.3 * treated)  # SMD ~ 0.3
  balanced   <- rnorm(n, mean = 0.02 * treated)  # SMD ~ 0.02

  mb <- mb_balance_table(treated = treated,
                         imbalanced = imbalanced,
                         balanced = balanced,
                         threshold = 0.10)

  expect_true(mb$imbalanced[mb$covariate == "imbalanced"])
  expect_false(mb$imbalanced[mb$covariate == "balanced"])
})

#' Structured Magenta Book confidence rating
#'
#' Records a single confidence rating against the bundled rubric:
#' high / medium / low, with explicit assessments of evidence
#' strength, methodological quality, and generalisability, and a
#' free-text rationale.
#'
#' @param rating Character scalar. One of `"high"`, `"medium"`,
#'   `"low"`.
#' @param question Character scalar. The evaluation question this
#'   rating refers to.
#' @param evidence_strength Character scalar. Plain-English
#'   description of the volume and quality of underlying studies.
#' @param methodological_quality Character scalar. Plain-English
#'   description of design rigour and identifying assumptions.
#' @param generalisability Character scalar. Plain-English
#'   description of how widely the findings travel across settings.
#' @param rationale Character scalar. Free-text justification for
#'   the chosen rating.
#'
#' @return An `mb_confidence` object: a list with the supplied
#'   fields plus the bundled-rubric `description` for the chosen
#'   rating, and `vintage`.
#'
#' @details
#' Magenta Book confidence ratings translate evidence into
#' decision-grade summaries for ministers and senior officials. The
#' bundled rubric (see [mb_schedule_table()] with table
#' `"confidence"`) is *not a direct quotation from the Magenta
#' Book*. It is a magentabook synthesis of cross-What-Works-Centre
#' confidence-rating traditions: Education Endowment Foundation
#' (5 padlocks), Early Intervention Foundation (Foundation
#' Standards), College of Policing (1-5 scale), and the Justice
#' Data Lab (red / amber / green). The three-level high / medium /
#' low structure is designed for HMG decision-grade reporting and
#' aligns with the value-for-money framing of the Magenta Book
#' (2020) supplementary guidance.
#'
#' @references
#' HM Treasury (2020). The Magenta Book: Central Government
#' Guidance on Evaluation. Supplementary guidance on value for
#' money.
#'
#' Education Endowment Foundation. Padlock evidence ratings.
#'
#' Early Intervention Foundation (2021). Foundation Standards of
#' Evidence.
#'
#' @family confidence
#' @seealso [mb_confidence_summary()], [mb_sms_rate()].
#'
#' @export
#' @examples
#' mb_confidence(
#'   rating                 = "medium",
#'   question               = "Did the policy raise employment",
#'   evidence_strength      = "One Level 4 DiD; one Level 3 matched cohort",
#'   methodological_quality = "Adequate; parallel trends plausible but limited pre-period",
#'   generalisability       = "Findings established in a single region",
#'   rationale              = "Effect direction consistent across two studies but limited replication"
#' )
mb_confidence <- function(rating = c("high", "medium", "low"),
                          question,
                          evidence_strength,
                          methodological_quality,
                          generalisability,
                          rationale) {
  rating <- match.arg(rating)
  validate_character(question,               arg = "question")
  validate_scalar(question,                  arg = "question")
  validate_character(evidence_strength,      arg = "evidence_strength")
  validate_scalar(evidence_strength,         arg = "evidence_strength")
  validate_character(methodological_quality, arg = "methodological_quality")
  validate_scalar(methodological_quality,    arg = "methodological_quality")
  validate_character(generalisability,       arg = "generalisability")
  validate_scalar(generalisability,          arg = "generalisability")
  validate_character(rationale,              arg = "rationale")
  validate_scalar(rationale,                 arg = "rationale")

  rubric <- .read_confidence_rubric()
  row <- rubric[rubric$rating == rating, , drop = FALSE]
  out <- list(
    rating                 = rating,
    label                  = row$label,
    question               = question,
    evidence_strength      = evidence_strength,
    methodological_quality = methodological_quality,
    generalisability       = generalisability,
    rationale              = rationale,
    description            = row$description,
    vintage                = .mb_vintage()
  )
  class(out) <- c("mb_confidence", "list")
  out
}

#' One-page confidence summary across multiple ratings
#'
#' Aggregates several `mb_confidence` ratings into a single summary
#' object with a confidence count and the underlying ratings as a
#' data frame.
#'
#' @param ... One or more `mb_confidence` objects, or a single
#'   list of them.
#'
#' @return An `mb_confidence_summary` object: a list with `n`
#'   (total ratings), `counts` (named integer vector by rating),
#'   `ratings` (data frame), and `vintage`.
#'
#' @family confidence
#' @seealso [mb_confidence()].
#'
#' @export
#' @examples
#' c1 <- mb_confidence(
#'   "high",   "Did employment rise",
#'   "Two Level 5 RCTs", "Strong; randomisation worked",
#'   "Tested in two regions", "Two RCTs both positive"
#' )
#' c2 <- mb_confidence(
#'   "medium", "Did wages rise",
#'   "One Level 4 DiD",  "Adequate; parallel trends plausible",
#'   "Single region",    "DiD effect positive but no replication"
#' )
#' mb_confidence_summary(c1, c2)
mb_confidence_summary <- function(...) {
  args <- list(...)
  if (length(args) == 1L && is.list(args[[1]]) && !inherits(args[[1]], "mb_confidence")) {
    args <- args[[1]]
  }
  if (length(args) == 0L) {
    cli::cli_abort("Provide at least one {.cls mb_confidence} object.")
  }
  ok <- vapply(args, inherits, logical(1), what = "mb_confidence")
  if (!all(ok)) {
    cli::cli_abort("All arguments must be {.cls mb_confidence} objects.")
  }

  ratings <- data.frame(
    question  = vapply(args, `[[`, character(1), "question"),
    rating    = vapply(args, `[[`, character(1), "rating"),
    rationale = vapply(args, `[[`, character(1), "rationale"),
    stringsAsFactors = FALSE
  )
  counts <- table(factor(ratings$rating, levels = c("high", "medium", "low")))
  out <- list(
    n       = nrow(ratings),
    counts  = as.integer(counts),
    levels  = c("high", "medium", "low"),
    ratings = ratings,
    vintage = .mb_vintage()
  )
  names(out$counts) <- out$levels
  class(out) <- c("mb_confidence_summary", "list")
  out
}

test_that("mb_confidence builds an mb_confidence object", {
  c1 <- mb_confidence(
    rating = "high",
    question = "q",
    evidence_strength = "es",
    methodological_quality = "mq",
    generalisability = "g",
    rationale = "r"
  )
  expect_s3_class(c1, "mb_confidence")
  expect_equal(c1$rating, "high")
  expect_equal(c1$label, "High confidence")
  expect_match(c1$description, "Decision-grade")
})

test_that("mb_confidence accepts all three ratings", {
  for (rt in c("high", "medium", "low")) {
    c1 <- mb_confidence(rt, "q", "es", "mq", "g", "r")
    expect_equal(c1$rating, rt)
  }
})

test_that("mb_confidence rejects unknown ratings", {
  expect_error(
    mb_confidence("wrong", "q", "es", "mq", "g", "r"),
    "should be one of"
  )
})

test_that("mb_confidence_summary tallies ratings", {
  c1 <- mb_confidence("high",   "q1", "es", "mq", "g", "r")
  c2 <- mb_confidence("medium", "q2", "es", "mq", "g", "r")
  c3 <- mb_confidence("medium", "q3", "es", "mq", "g", "r")
  s <- mb_confidence_summary(c1, c2, c3)
  expect_s3_class(s, "mb_confidence_summary")
  expect_equal(s$n, 3L)
  expect_equal(unname(s$counts), c(1L, 2L, 0L))
  expect_equal(nrow(s$ratings), 3L)
})

test_that("mb_confidence_summary accepts a list", {
  c1 <- mb_confidence("low", "q1", "es", "mq", "g", "r")
  c2 <- mb_confidence("low", "q2", "es", "mq", "g", "r")
  s_args <- mb_confidence_summary(c1, c2)
  s_list <- mb_confidence_summary(list(c1, c2))
  expect_equal(s_args$counts, s_list$counts)
})

test_that("mb_confidence_summary rejects non-mb_confidence input", {
  expect_error(mb_confidence_summary("not"), "mb_confidence")
  expect_error(mb_confidence_summary(), "Provide at least one")
})

#' Vintage of bundled rubric and reference tables
#'
#' Returns a data frame describing the source and last-updated date
#' of every CSV bundled in `inst/extdata/`. Critical for
#' reproducibility: every evaluation report can record the vintage
#' of the rubrics and reference values used.
#'
#' @return A data frame with columns `dataset`, `source`,
#'   `last_updated`, `notes`.
#'
#' @family lookups
#' @seealso [mb_schedule_table()].
#'
#' @export
#' @examples
#' mb_data_versions()
mb_data_versions <- function() {
  .read_data_versions()
}

#' Expose internal lookup tables
#'
#' Returns one of the bundled lookup tables: the Maryland SMS
#' rubric, the Magenta Book confidence rubric, the ICC reference
#' table, or the evaluation question taxonomy.
#'
#' @param table Character scalar. One of `"sms"`, `"confidence"`,
#'   `"icc"`, or `"questions"`.
#'
#' @return A data frame.
#'
#' @family lookups
#' @seealso [mb_data_versions()].
#'
#' @export
#' @examples
#' mb_schedule_table("sms")
#' mb_schedule_table("confidence")
#' mb_schedule_table("icc")
#' mb_schedule_table("questions")
mb_schedule_table <- function(table = c("sms", "confidence", "icc", "questions")) {
  table <- match.arg(table)
  switch(
    table,
    sms        = .read_sms_rubric(),
    confidence = .read_confidence_rubric(),
    icc        = .read_icc_reference(),
    questions  = .read_question_taxonomy()
  )
}

test_that("mb_data_versions returns the bundled vintage table", {
  vs <- mb_data_versions()
  expect_s3_class(vs, "data.frame")
  expect_true(all(c("dataset", "source", "last_updated", "notes") %in% names(vs)))
  expect_true(nrow(vs) >= 4)
})

test_that("mb_schedule_table returns each table by name", {
  expect_s3_class(mb_schedule_table("sms"),        "data.frame")
  expect_s3_class(mb_schedule_table("confidence"), "data.frame")
  expect_s3_class(mb_schedule_table("icc"),        "data.frame")
  expect_s3_class(mb_schedule_table("questions"),  "data.frame")
})

test_that("mb_schedule_table 'sms' table has 5 rows", {
  expect_equal(nrow(mb_schedule_table("sms")), 5L)
})

test_that("mb_schedule_table 'confidence' table has 3 ratings", {
  tbl <- mb_schedule_table("confidence")
  expect_equal(nrow(tbl), 3L)
  expect_equal(sort(tbl$rating), sort(c("high", "medium", "low")))
})

test_that("mb_schedule_table rejects unknown tables", {
  expect_error(mb_schedule_table("nonexistent"), "should be one of")
})

test_that("each entry in mb_data_versions has a non-empty source", {
  vs <- mb_data_versions()
  expect_true(all(nzchar(vs$source)))
  expect_true(all(nzchar(vs$last_updated)))
})

#' Context-mechanism-outcome (CMO) configuration
#'
#' Records one or more CMO configurations from a realist evaluation:
#' the *contexts* in which a *mechanism* fires to produce an
#' *outcome*, with optional supporting evidence.
#'
#' @param context Character vector. The contextual conditions
#'   needed for the mechanism to fire.
#' @param mechanism Character vector. The underlying generative
#'   mechanism (typically a change in reasoning or resources).
#' @param outcome Character vector. The observed outcome pattern.
#' @param evidence Character vector. Citation, quote, or other
#'   evidence supporting the configuration. Default `NA`.
#'
#' @return An `mb_cmo` data frame with columns `context`,
#'   `mechanism`, `outcome`, `evidence`.
#'
#' @details
#' Realist evaluation, developed by Pawson and Tilley (1997), seeks
#' to answer "what works for whom in what circumstances and why" by
#' surfacing CMO configurations rather than estimating average
#' treatment effects. The Magenta Book lists realist evaluation as
#' the principal theory-based approach for context-dependent
#' interventions.
#'
#' @references
#' Pawson, R., Tilley, N. (1997). Realistic Evaluation. SAGE.
#'
#' HM Treasury (2020). The Magenta Book, chapter on theory-based
#' evaluation.
#' <https://www.gov.uk/government/publications/the-magenta-book>.
#'
#' @family realist
#' @seealso [mb_contribution_claim()].
#'
#' @export
#' @examples
#' mb_cmo(
#'   context   = c("High trust GP-patient relationships",
#'                 "Low trust GP-patient relationships"),
#'   mechanism = c("Patients accept advice", "Patients ignore advice"),
#'   outcome   = c("Improved adherence", "No change in adherence"),
#'   evidence  = c("Smith et al. 2024 cohort study", "Smith et al. 2024")
#' )
mb_cmo <- function(context, mechanism, outcome, evidence = NA_character_) {
  validate_character(context,   arg = "context")
  validate_character(mechanism, arg = "mechanism")
  validate_character(outcome,   arg = "outcome")
  n <- length(context)
  if (length(mechanism) != n) cli::cli_abort("{.arg mechanism} must be length {.val {n}}.")
  if (length(outcome)   != n) cli::cli_abort("{.arg outcome} must be length {.val {n}}.")
  if (length(evidence)  == 1L) evidence <- rep(evidence, n)
  if (length(evidence)  != n) cli::cli_abort("{.arg evidence} must be length 1 or {.val {n}}.")

  out <- data.frame(
    context   = context,
    mechanism = mechanism,
    outcome   = outcome,
    evidence  = evidence,
    stringsAsFactors = FALSE
  )
  class(out) <- c("mb_cmo", "data.frame")
  out
}

#' Contribution-analysis claim
#'
#' Records a contribution claim with supporting and refuting
#' evidence and an overall strength rating. Used in
#' contribution-analysis-style theory-based evaluation, where causal
#' inference comes from triangulating multiple evidence streams
#' against a contribution story rather than from a counterfactual.
#'
#' @param claim Character scalar. The contribution claim being
#'   tested.
#' @param evidence_for Character vector. Evidence supporting the
#'   claim.
#' @param evidence_against Character vector. Evidence against the
#'   claim. Default `character(0)`.
#' @param strength Character scalar. One of `"weak"`, `"moderate"`,
#'   `"strong"`. Reflects the analyst's overall judgement after
#'   weighing evidence for and against.
#'
#' @return An `mb_contribution_claim` object.
#'
#' @references
#' Mayne, J. (2008). Contribution Analysis: An approach to exploring
#' cause and effect. ILAC Brief No. 16.
#'
#' HM Treasury (2020). The Magenta Book, chapter on theory-based
#' evaluation.
#'
#' @family realist
#' @seealso [mb_cmo()].
#'
#' @export
#' @examples
#' mb_contribution_claim(
#'   claim            = "The training programme contributed to higher employment",
#'   evidence_for     = c("Pre-post outcomes improved",
#'                        "Theory of change pathways visible in interviews"),
#'   evidence_against = "Macro labour market also improved",
#'   strength         = "moderate"
#' )
mb_contribution_claim <- function(claim,
                                  evidence_for,
                                  evidence_against = character(0),
                                  strength = c("weak", "moderate", "strong")) {
  validate_character(claim, arg = "claim"); validate_scalar(claim, "claim")
  validate_character(evidence_for, arg = "evidence_for")
  if (length(evidence_against)) validate_character(evidence_against, arg = "evidence_against")
  strength <- match.arg(strength)

  out <- list(
    claim            = claim,
    evidence_for     = evidence_for,
    evidence_against = evidence_against,
    strength         = strength,
    vintage          = .mb_vintage()
  )
  class(out) <- c("mb_contribution_claim", "list")
  out
}

test_that("mb_cmo builds a register", {
  cmo <- mb_cmo(
    context = c("c1", "c2"),
    mechanism = c("m1", "m2"),
    outcome = c("o1", "o2")
  )
  expect_s3_class(cmo, "mb_cmo")
  expect_equal(nrow(cmo), 2L)
  expect_true(all(is.na(cmo$evidence)))
})

test_that("mb_cmo recycles scalar evidence", {
  cmo <- mb_cmo(c("c1", "c2"), c("m1", "m2"), c("o1", "o2"),
                evidence = "Smith 2024")
  expect_equal(cmo$evidence, c("Smith 2024", "Smith 2024"))
})

test_that("mb_cmo errors on length mismatch", {
  expect_error(
    mb_cmo(context = c("c1", "c2"), mechanism = "m", outcome = c("o1", "o2")),
    "length"
  )
})

test_that("mb_contribution_claim builds a claim", {
  c <- mb_contribution_claim(
    claim = "Programme caused outcome",
    evidence_for = c("ev1", "ev2"),
    evidence_against = "ev3",
    strength = "moderate"
  )
  expect_s3_class(c, "mb_contribution_claim")
  expect_equal(c$strength, "moderate")
  expect_equal(length(c$evidence_for), 2L)
  expect_equal(length(c$evidence_against), 1L)
})

test_that("mb_contribution_claim accepts no evidence_against", {
  c <- mb_contribution_claim("c", "ev", strength = "weak")
  expect_equal(length(c$evidence_against), 0L)
})

test_that("mb_contribution_claim rejects unknown strength", {
  expect_error(
    mb_contribution_claim("c", "ev", strength = "wrong"),
    "should be one of"
  )
})

#' Cost per unit of outcome
#'
#' Computes a simple cost-effectiveness ratio: total cost divided
#' by total outcomes delivered. Use [mb_icer()] for two-option
#' comparisons.
#'
#' @param cost Numeric scalar or vector. Total cost (or per-period
#'   costs that will be summed).
#' @param effect Numeric scalar or vector. Total outcomes delivered
#'   (or per-period outcomes that will be summed).
#' @param label Optional character scalar. Name of the option.
#'
#' @return An `mb_cea` object.
#'
#' @family cost-effectiveness
#' @seealso [mb_icer()], [mb_ceac()], [mb_inb()].
#'
#' @export
#' @examples
#' mb_cea(cost = 1e6, effect = 250, label = "Workshop programme")
mb_cea <- function(cost, effect, label = NULL) {
  validate_numeric(cost,   arg = "cost")
  validate_numeric(effect, arg = "effect")
  if (!is.null(label)) {
    validate_character(label, arg = "label")
    validate_scalar(label,    arg = "label")
  }
  total_cost   <- sum(cost)
  total_effect <- sum(effect)
  if (total_effect == 0) {
    cli::cli_abort("Total {.arg effect} is zero; cost per unit is undefined.")
  }
  out <- list(
    label             = label,
    total_cost        = total_cost,
    total_effect      = total_effect,
    cost_per_unit     = total_cost / total_effect,
    vintage           = .mb_vintage()
  )
  class(out) <- c("mb_cea", "list")
  out
}

#' Incremental cost-effectiveness ratio with dominance handling
#'
#' Computes the ICER comparing option B to option A, with explicit
#' handling of the four dominance regions:
#' - **A dominates** B (B costs more, delivers less): no ICER.
#' - **B dominates** A (B costs less, delivers more): no ICER; B is
#'   the obvious choice.
#' - **B more costly, more effective**: standard ICER positive.
#' - **B less costly, less effective**: ICER negative — B saves
#'   money at the expense of effect.
#'
#' @param cost_a,effect_a Numeric scalars. Cost and effect of
#'   option A.
#' @param cost_b,effect_b Numeric scalars. Cost and effect of
#'   option B.
#' @param label_a,label_b Character scalars. Labels for the two
#'   options.
#'
#' @return An `mb_icer` object: a list with `delta_cost`,
#'   `delta_effect`, `icer`, `dominance` (one of `"a_dominates"`,
#'   `"b_dominates"`, `"b_more_costly_more_effective"`,
#'   `"b_less_costly_less_effective"`), and labels.
#'
#' @details
#' The ICER is the cost per additional unit of outcome from
#' switching from A to B:
#' \deqn{\text{ICER} = (C_B - C_A) / (E_B - E_A)}
#'
#' If `delta_effect` is zero, the ICER is reported as `Inf`
#' (when costs differ) or `NaN` (when costs are equal).
#'
#' @references
#' HM Treasury (2020). The Magenta Book, Annex A on cost-effectiveness.
#'
#' Drummond, M. F., Sculpher, M. J., Claxton, K., Stoddart, G. L.,
#' Torrance, G. W. (2015). Methods for the Economic Evaluation of
#' Health Care Programmes (4th ed.). Oxford University Press.
#'
#' @family cost-effectiveness
#' @seealso [mb_cea()], [mb_ceac()], [mb_inb()].
#'
#' @export
#' @examples
#' mb_icer(cost_a = 1e6, effect_a = 200, cost_b = 1.5e6, effect_b = 300,
#'         label_a = "Status quo", label_b = "Enhanced")
mb_icer <- function(cost_a, effect_a, cost_b, effect_b,
                    label_a = "A", label_b = "B") {
  validate_numeric(cost_a,   arg = "cost_a");   validate_scalar(cost_a,   "cost_a")
  validate_numeric(effect_a, arg = "effect_a"); validate_scalar(effect_a, "effect_a")
  validate_numeric(cost_b,   arg = "cost_b");   validate_scalar(cost_b,   "cost_b")
  validate_numeric(effect_b, arg = "effect_b"); validate_scalar(effect_b, "effect_b")
  validate_character(label_a, arg = "label_a"); validate_scalar(label_a, "label_a")
  validate_character(label_b, arg = "label_b"); validate_scalar(label_b, "label_b")

  delta_cost   <- cost_b - cost_a
  delta_effect <- effect_b - effect_a
  icer <- if (delta_effect == 0) {
    if (delta_cost == 0) NaN else Inf * sign(delta_cost)
  } else {
    delta_cost / delta_effect
  }
  dominance <- if (delta_cost <= 0 && delta_effect >= 0 &&
                   !(delta_cost == 0 && delta_effect == 0)) {
    "b_dominates"
  } else if (delta_cost >= 0 && delta_effect <= 0 &&
             !(delta_cost == 0 && delta_effect == 0)) {
    "a_dominates"
  } else if (delta_cost > 0 && delta_effect > 0) {
    "b_more_costly_more_effective"
  } else if (delta_cost < 0 && delta_effect < 0) {
    "b_less_costly_less_effective"
  } else {
    "no_difference"
  }

  out <- list(
    label_a       = label_a,
    label_b       = label_b,
    cost_a        = cost_a,
    cost_b        = cost_b,
    effect_a      = effect_a,
    effect_b      = effect_b,
    delta_cost    = delta_cost,
    delta_effect  = delta_effect,
    icer          = icer,
    dominance     = dominance,
    vintage       = .mb_vintage()
  )
  class(out) <- c("mb_icer", "list")
  out
}

#' Cost-effectiveness acceptability curve
#'
#' For a single A-vs-B comparison with sampled (`delta_cost`,
#' `delta_effect`) draws (e.g. from a probabilistic sensitivity
#' analysis), returns the probability that B is cost-effective at
#' each willingness-to-pay (WTP) value in `wtp_grid`.
#'
#' @param delta_cost Numeric vector. Sampled incremental costs of B
#'   relative to A.
#' @param delta_effect Numeric vector, same length as `delta_cost`.
#'   Sampled incremental effects.
#' @param wtp_grid Numeric vector of willingness-to-pay values
#'   (cost per unit of effect) at which to evaluate the curve.
#'
#' @return An `mb_ceac` object: a data-frame-like list with columns
#'   `wtp`, `prob_cost_effective`, plus `n_draws` and `vintage`.
#'
#' @details
#' At each WTP value `lambda`, B is cost-effective if the
#' incremental net benefit `lambda * delta_effect - delta_cost > 0`.
#' The CEAC is the proportion of draws for which this is true.
#'
#' @references
#' Fenwick, E., Claxton, K., Sculpher, M. (2001). Representing
#' uncertainty: the role of cost-effectiveness acceptability
#' curves. Health Economics 10(8). <doi:10.1002/hec.635>.
#'
#' @family cost-effectiveness
#' @seealso [mb_inb()], [mb_icer()].
#'
#' @export
#' @examples
#' set.seed(4)
#' delta_cost   <- rnorm(1000, mean = 50000, sd = 10000)
#' delta_effect <- rnorm(1000, mean = 2,     sd = 0.5)
#' mb_ceac(delta_cost, delta_effect, wtp_grid = seq(0, 100000, by = 10000))
mb_ceac <- function(delta_cost, delta_effect, wtp_grid) {
  validate_numeric(delta_cost,   arg = "delta_cost")
  validate_numeric(delta_effect, arg = "delta_effect")
  validate_numeric(wtp_grid,     arg = "wtp_grid", require_non_negative = TRUE)
  if (length(delta_cost) != length(delta_effect)) {
    cli::cli_abort("{.arg delta_cost} and {.arg delta_effect} must be the same length.")
  }
  prob <- vapply(wtp_grid, function(lambda) {
    inb <- lambda * delta_effect - delta_cost
    mean(inb > 0)
  }, numeric(1))
  out <- list(
    wtp                 = wtp_grid,
    prob_cost_effective = prob,
    n_draws             = length(delta_cost),
    vintage             = .mb_vintage()
  )
  class(out) <- c("mb_ceac", "list")
  out
}

#' Incremental net benefit
#'
#' Computes the incremental net benefit (INB) of B over A at a
#' single willingness-to-pay threshold. Equivalent to the ICER
#' framing on a monetary scale.
#'
#' @param delta_cost Numeric scalar. Incremental cost of B over A.
#' @param delta_effect Numeric scalar. Incremental effect of B
#'   over A.
#' @param wtp Numeric scalar. Willingness-to-pay per unit of effect
#'   (e.g. the NICE QALY threshold in a health context).
#'
#' @return Numeric scalar. INB in the units of `delta_cost`. INB > 0
#'   means B is cost-effective at the supplied WTP.
#'
#' @details
#' \deqn{\text{INB} = \lambda \cdot \Delta E - \Delta C}
#' Equivalent to ICER comparison: INB > 0 iff ICER < WTP (when
#' effect change is positive).
#'
#' @family cost-effectiveness
#' @seealso [mb_ceac()], [mb_icer()].
#'
#' @export
#' @examples
#' mb_inb(delta_cost = 50000, delta_effect = 2, wtp = 30000)
mb_inb <- function(delta_cost, delta_effect, wtp) {
  validate_numeric(delta_cost,   arg = "delta_cost");   validate_scalar(delta_cost,   "delta_cost")
  validate_numeric(delta_effect, arg = "delta_effect"); validate_scalar(delta_effect, "delta_effect")
  validate_numeric(wtp, arg = "wtp", require_non_negative = TRUE)
  validate_scalar(wtp, arg = "wtp")
  wtp * delta_effect - delta_cost
}

#' Quality-adjusted life years (QALYs) accumulator
#'
#' Sums utility-weighted years lived across persons, with optional
#' annual discounting.
#'
#' @param utility Numeric scalar or vector in `[0, 1]`. Utility
#'   weight per year. Length 1 or `years`.
#' @param persons Numeric scalar. Number of persons. Default `1`.
#' @param years Numeric scalar. Number of years. Default `1`.
#' @param discount_rate Optional numeric in `[0, 1)`. Annual
#'   discount rate. If supplied, returns the discounted QALY total.
#'   Default `NULL` (undiscounted).
#'
#' @return Numeric scalar. Total QALYs.
#'
#' @details
#' Without discounting:
#' \deqn{\text{QALY} = \text{persons} \cdot \sum_{t=0}^{T-1} u_t}
#'
#' With annual discount rate `r`:
#' \deqn{\text{QALY} = \text{persons} \cdot \sum_{t=0}^{T-1} \frac{u_t}{(1+r)^t}}
#'
#' Compatible with `greenbook::gb_qaly`: when `utility` is scalar and
#' `discount_rate` is `NULL`, this returns `persons * utility * years`.
#'
#' @references
#' Drummond, M. F. et al. (2015). Methods for the Economic
#' Evaluation of Health Care Programmes (4th ed.). OUP.
#'
#' NICE (2022). Guide to the methods of technology appraisal.
#'
#' @family cost-effectiveness
#' @seealso [mb_daly()], [mb_cea()].
#'
#' @export
#' @examples
#' mb_qaly(utility = 0.8, persons = 100, years = 5)
#' mb_qaly(utility = 0.8, persons = 100, years = 5, discount_rate = 0.035)
#' mb_qaly(utility = c(0.5, 0.7, 0.9), persons = 50)
mb_qaly <- function(utility, persons = 1, years = 1, discount_rate = NULL) {
  validate_numeric(utility, arg = "utility")
  if (any(utility < 0 | utility > 1)) {
    cli::cli_abort("{.arg utility} must be between 0 and 1.")
  }
  validate_numeric(persons, arg = "persons", require_positive = TRUE)
  validate_scalar(persons,  arg = "persons")
  validate_numeric(years,   arg = "years",   require_positive = TRUE)
  validate_scalar(years,    arg = "years")

  if (length(utility) == 1L) {
    if (is.null(discount_rate)) {
      return(persons * utility * years)
    }
    utility <- rep(utility, ceiling(years))
  }
  if (length(utility) != as.integer(ceiling(years))) {
    if (length(utility) > 1L && years != 1) {
      cli::cli_abort("Length of {.arg utility} ({.val {length(utility)}}) must equal {.arg years} ({.val {years}}) when both > 1.")
    }
    years <- length(utility)
  }
  if (is.null(discount_rate)) {
    return(persons * sum(utility))
  }
  validate_proportion(discount_rate, arg = "discount_rate")
  validate_scalar(discount_rate,     arg = "discount_rate")
  if (discount_rate >= 1) cli::cli_abort("{.arg discount_rate} must be less than 1.")
  weights <- 1 / (1 + discount_rate)^seq.int(0, length(utility) - 1L)
  persons * sum(utility * weights)
}

#' Disability-adjusted life years (DALYs) accumulator
#'
#' Sums years lived with disability (YLD) and years of life lost
#' (YLL) across persons. DALY is the global-health analogue of
#' QALY: lower is better.
#'
#' @param yld Numeric scalar or vector. Years lived with disability
#'   per person.
#' @param yll Numeric scalar or vector. Years of life lost per
#'   person (e.g. life expectancy minus age at death).
#' @param persons Numeric scalar. Number of persons. Default `1`.
#'
#' @return Numeric scalar. Total DALYs (YLD + YLL summed across
#'   persons).
#'
#' @details
#' \deqn{\text{DALY} = \text{persons} \cdot \sum (YLD + YLL)}
#'
#' This implementation follows the Global Burden of Disease
#' definition. Age-weighting and discounting are not applied by
#' default (the IHME GBD removed both in the 2010 update); add a
#' discount factor manually if your guidance still requires it.
#'
#' @references
#' Murray, C. J. L., Lopez, A. D. (1996). The Global Burden of
#' Disease. Harvard University Press.
#'
#' GBD 2019 Diseases and Injuries Collaborators (2020). Global
#' burden of 369 diseases and injuries in 204 countries and
#' territories, 1990-2019. The Lancet 396.
#' <doi:10.1016/S0140-6736(20)30925-9>.
#'
#' @family cost-effectiveness
#' @seealso [mb_qaly()].
#'
#' @export
#' @examples
#' mb_daly(yld = 2.5, yll = 8.0, persons = 100)
mb_daly <- function(yld, yll, persons = 1) {
  validate_numeric(yld,     arg = "yld",     require_non_negative = TRUE)
  validate_numeric(yll,     arg = "yll",     require_non_negative = TRUE)
  validate_numeric(persons, arg = "persons", require_positive = TRUE)
  validate_scalar(persons,  arg = "persons")
  persons * (sum(yld) + sum(yll))
}

# Cross-validation of mb_icer / mb_ceac / mb_inb against BCEA.
#
# BCEA is the canonical R implementation of Bayesian
# cost-effectiveness analysis. magentabook implements the same
# arithmetic (point ICER, CEAC, INB) as a deterministic /
# frequentist baseline. The two should agree numerically when
# magentabook is given the sample means from the same draws BCEA
# uses.

skip_if_not_installed("BCEA")

test_that("mb_icer point estimate agrees with BCEA ICER from sample means", {
  set.seed(20260427)
  n <- 1000
  effect_a <- rnorm(n, mean = 2.0, sd = 0.5)
  effect_b <- rnorm(n, mean = 2.5, sd = 0.5)
  cost_a   <- rnorm(n, mean = 1e6, sd = 1e5)
  cost_b   <- rnorm(n, mean = 1.5e6, sd = 1e5)

  e <- cbind(effect_a, effect_b)
  c_ <- cbind(cost_a, cost_b)
  res_bcea <- BCEA::bcea(e = e, c = c_, ref = 2,
                        interventions = c("A", "B"),
                        Kmax = 100000)

  # BCEA stores the mean-effect and mean-cost ICER in the comparison
  bcea_icer <- mean(cost_b - cost_a) / mean(effect_b - effect_a)

  mb <- mb_icer(
    cost_a   = mean(cost_a), effect_a = mean(effect_a),
    cost_b   = mean(cost_b), effect_b = mean(effect_b),
    label_a = "A", label_b = "B"
  )

  expect_equal(mb$icer, bcea_icer, tolerance = 1e-8)
})

test_that("mb_ceac probabilities agree with BCEA-style direct calculation", {
  set.seed(20260428)
  n <- 2000
  delta_cost   <- rnorm(n, mean = 50000, sd = 10000)
  delta_effect <- rnorm(n, mean = 2,     sd = 0.5)

  wtp_grid <- seq(0, 100000, by = 5000)
  mb <- mb_ceac(delta_cost, delta_effect, wtp_grid = wtp_grid)

  # BCEA computes CEAC as the probability that incremental net
  # benefit (k * delta_e - delta_c) is positive across draws.
  # This is exactly what mb_ceac does, so the two should match.
  bcea_style <- vapply(wtp_grid, function(k) {
    mean(k * delta_effect - delta_cost > 0)
  }, numeric(1))

  expect_equal(mb$prob_cost_effective, bcea_style, tolerance = 1e-12)
})

test_that("mb_inb sign agrees with BCEA decision at given WTP", {
  # When mb_inb returns positive, BCEA says B is preferred;
  # when negative, A is preferred. This validates the sign convention.
  set.seed(1)
  cases <- list(
    list(delta_cost = 50000, delta_effect = 2,    wtp = 30000, expect_positive = TRUE),
    list(delta_cost = 80000, delta_effect = 2,    wtp = 30000, expect_positive = FALSE),
    list(delta_cost = 50000, delta_effect = 2,    wtp = 24000, expect_positive = FALSE),
    list(delta_cost = -10000, delta_effect = 1,   wtp = 30000, expect_positive = TRUE)
  )
  for (cc in cases) {
    inb <- mb_inb(cc$delta_cost, cc$delta_effect, cc$wtp)
    if (cc$expect_positive) {
      expect_gt(inb, 0, label = sprintf("dc=%s, de=%s, k=%s", cc$delta_cost, cc$delta_effect, cc$wtp))
    } else {
      expect_lt(inb, 0, label = sprintf("dc=%s, de=%s, k=%s", cc$delta_cost, cc$delta_effect, cc$wtp))
    }
  }
})

test_that("mb_cea computes cost per unit", {
  res <- mb_cea(cost = 1e6, effect = 250)
  expect_s3_class(res, "mb_cea")
  expect_equal(res$cost_per_unit, 4000)
})

test_that("mb_cea sums vector inputs", {
  res <- mb_cea(cost = c(500, 500), effect = c(100, 150))
  expect_equal(res$total_cost, 1000)
  expect_equal(res$total_effect, 250)
  expect_equal(res$cost_per_unit, 4)
})

test_that("mb_cea errors on zero effect", {
  expect_error(mb_cea(cost = 1000, effect = 0), "undefined")
})

test_that("mb_icer computes ICER and tags dominance", {
  # B more costly, more effective
  r1 <- mb_icer(1e6, 200, 1.5e6, 300)
  expect_equal(r1$icer, (1.5e6 - 1e6) / (300 - 200))
  expect_equal(r1$dominance, "b_more_costly_more_effective")

  # B dominates: cheaper and more effective
  r2 <- mb_icer(1e6, 200, 0.5e6, 300)
  expect_equal(r2$dominance, "b_dominates")

  # A dominates: B more costly and less effective
  r3 <- mb_icer(1e6, 300, 1.5e6, 200)
  expect_equal(r3$dominance, "a_dominates")

  # B less costly, less effective
  r4 <- mb_icer(1e6, 300, 0.5e6, 200)
  expect_equal(r4$dominance, "b_less_costly_less_effective")
})

test_that("mb_icer handles zero effect-difference edge case", {
  # No difference in effect, positive cost diff
  r <- mb_icer(1e6, 100, 1.5e6, 100)
  expect_true(is.infinite(r$icer))
  # No diff in either: NaN
  r0 <- mb_icer(1e6, 100, 1e6, 100)
  expect_true(is.nan(r0$icer))
})

test_that("mb_ceac probabilities are non-decreasing in WTP for positive effect", {
  set.seed(4)
  delta_cost   <- rnorm(2000, mean = 50000, sd = 10000)
  delta_effect <- rnorm(2000, mean = 2,     sd = 0.5)
  res <- mb_ceac(delta_cost, delta_effect, wtp_grid = seq(0, 100000, by = 10000))
  expect_s3_class(res, "mb_ceac")
  expect_true(all(diff(res$prob_cost_effective) >= -0.05))
})

test_that("mb_ceac probabilities are in [0,1]", {
  delta_cost <- runif(100, 0, 1000)
  delta_effect <- runif(100, 0.1, 1)
  res <- mb_ceac(delta_cost, delta_effect, wtp_grid = c(0, 500, 1000, 5000))
  expect_true(all(res$prob_cost_effective >= 0))
  expect_true(all(res$prob_cost_effective <= 1))
})

test_that("mb_ceac validates equal lengths", {
  expect_error(
    mb_ceac(c(1, 2), c(1, 2, 3), wtp_grid = c(1, 2)),
    "same length"
  )
})

test_that("mb_inb returns positive when ICER below WTP", {
  inb <- mb_inb(delta_cost = 50000, delta_effect = 2, wtp = 30000)
  # ICER = 25000 < WTP = 30000 → INB > 0
  expect_gt(inb, 0)
})

test_that("mb_inb returns negative when ICER above WTP", {
  inb <- mb_inb(delta_cost = 80000, delta_effect = 2, wtp = 30000)
  # ICER = 40000 > WTP = 30000 → INB < 0
  expect_lt(inb, 0)
})

test_that("mb_qaly without discounting returns persons * utility * years", {
  expect_equal(mb_qaly(0.8, persons = 100, years = 5), 400)
})

test_that("mb_qaly with utility vector sums utilities", {
  expect_equal(mb_qaly(c(0.5, 0.7, 0.9), persons = 50), 50 * (0.5 + 0.7 + 0.9))
})

test_that("mb_qaly with discounting reduces total", {
  undisc <- mb_qaly(0.8, persons = 100, years = 5)
  disc   <- mb_qaly(0.8, persons = 100, years = 5, discount_rate = 0.035)
  expect_lt(disc, undisc)
})

test_that("mb_qaly enforces utility in [0, 1]", {
  expect_error(mb_qaly(1.2), "between 0 and 1")
  expect_error(mb_qaly(-0.1), "between 0 and 1")
})

test_that("mb_qaly enforces discount_rate < 1", {
  expect_error(
    mb_qaly(0.8, years = 2, discount_rate = 1),
    "less than 1"
  )
})

test_that("mb_daly sums YLD and YLL", {
  expect_equal(mb_daly(yld = 2.5, yll = 8.0, persons = 100), 1050)
})

test_that("mb_daly handles vector inputs", {
  expect_equal(mb_daly(yld = c(1, 2), yll = c(3, 4), persons = 1), 10)
})

test_that("mb_daly errors on negative inputs", {
  expect_error(mb_daly(-1, 1), "non-negative")
  expect_error(mb_daly(1, -1), "non-negative")
})

#' Interrupted time series via segmented regression
#'
#' Fits a single-group interrupted time series model:
#' \deqn{Y_t = \beta_0 + \beta_1 t + \beta_2 P_t + \beta_3 (t - t^*) P_t + \epsilon_t}
#' where `P_t` is 1 for `t >= t*` and `t*` is the intervention time.
#' `beta_2` is the immediate level change at the intervention;
#' `beta_3` is the change in slope.
#'
#' @param y Numeric vector of outcomes ordered by `time`.
#' @param time Numeric vector of time indices, same length as `y`.
#' @param intervention_time Numeric scalar. The first time point
#'   considered post-intervention.
#' @param lag Integer >= 0. Number of pre-intervention observations
#'   to drop near the intervention (transition period). Default `0`.
#' @param quiet Logical. If `FALSE` (default), the print method
#'   appends a one-line reminder that this is a single-group
#'   segmented regression and points to specialist tooling for
#'   autocorrelation correction. Set to `TRUE` to suppress.
#'
#' @return An `mb_its` object: a list with `coefficients` (named
#'   numeric), `se` (named numeric), `level_change`, `slope_change`,
#'   `intervention_time`, `n`, `n_pre`, `n_post`, and `vintage`.
#'
#' @details
#' Segmented regression assumes residuals are independent. For
#' autocorrelated series, fit a Newey-West, Prais-Winsten, or
#' ARIMA-error specification using \pkg{sandwich}, \pkg{nlme}, or
#' \pkg{forecast}. This function is the canonical baseline.
#'
#' @references
#' Bernal, J. L., Cummins, S., Gasparrini, A. (2017). Interrupted
#' time series regression for the evaluation of public health
#' interventions: a tutorial. International Journal of
#' Epidemiology 46(1). <doi:10.1093/ije/dyw098>.
#'
#' Wagner, A. K., Soumerai, S. B., Zhang, F., Ross-Degnan, D.
#' (2002). Segmented regression analysis of interrupted time series
#' studies in medication use research. Journal of Clinical Pharmacy
#' and Therapeutics 27. <doi:10.1046/j.1365-2710.2002.00430.x>.
#'
#' @family estimators
#' @seealso [mb_did_2x2()], [mb_event_study()].
#'
#' @export
#' @examples
#' set.seed(2)
#' time <- 1:48
#' y    <- 10 + 0.05 * time + ifelse(time >= 25, 2 + 0.1 * (time - 25), 0) + rnorm(48, sd = 0.5)
#' mb_its(y, time, intervention_time = 25)
mb_its <- function(y, time, intervention_time, lag = 0L, quiet = FALSE) {
  validate_numeric(y,    arg = "y")
  validate_numeric(time, arg = "time")
  if (length(y) != length(time)) {
    cli::cli_abort("{.arg y} and {.arg time} must be the same length.")
  }
  validate_numeric(intervention_time, arg = "intervention_time")
  validate_scalar(intervention_time,  arg = "intervention_time")
  validate_numeric(lag, arg = "lag", require_non_negative = TRUE)
  validate_scalar(lag,  arg = "lag")
  if (lag != as.integer(lag)) cli::cli_abort("{.arg lag} must be a non-negative integer.")

  P  <- as.numeric(time >= intervention_time)
  t_post <- pmax(time - intervention_time, 0) * P
  if (lag > 0L) {
    drop_idx <- which(time >= (intervention_time - lag) & time < intervention_time)
    if (length(drop_idx)) {
      y      <- y[-drop_idx]
      time   <- time[-drop_idx]
      P      <- P[-drop_idx]
      t_post <- t_post[-drop_idx]
    }
  }

  X <- cbind(1, time, P, t_post)
  colnames(X) <- c("(Intercept)", "time", "post", "post_time")
  XtX_inv <- tryCatch(
    solve(crossprod(X)),
    error = function(e) cli::cli_abort("Design matrix is singular; ensure both pre- and post-intervention observations are present.")
  )
  beta <- drop(XtX_inv %*% crossprod(X, y))
  resid <- y - drop(X %*% beta)
  n <- length(y); k <- ncol(X)
  sigma2 <- sum(resid^2) / (n - k)
  se     <- sqrt(diag(sigma2 * XtX_inv))
  names(beta) <- names(se) <- colnames(X)

  out <- list(
    coefficients      = beta,
    se                = se,
    level_change      = unname(beta["post"]),
    slope_change      = unname(beta["post_time"]),
    intervention_time = intervention_time,
    n                 = n,
    n_pre             = sum(P == 0),
    n_post            = sum(P == 1),
    quiet             = isTRUE(quiet),
    vintage           = .mb_vintage()
  )
  class(out) <- c("mb_its", "list")
  out
}

test_that("quiet argument propagates onto mb_did_2x2 / mb_its / mb_event_study", {
  set.seed(1)
  n <- 200
  treated <- rep(c(0, 1), each = n / 2)
  post    <- rep(c(0, 1), times = n / 2)
  y       <- rnorm(n)
  did_q  <- mb_did_2x2(y, treated, post, quiet = TRUE)
  did_nq <- mb_did_2x2(y, treated, post)
  expect_true(did_q$quiet)
  expect_false(did_nq$quiet)

  its_q <- mb_its(rnorm(24), 1:24, intervention_time = 12, quiet = TRUE)
  expect_true(its_q$quiet)

  panel <- expand.grid(unit = 1:20, time = 1:8)
  panel$y <- rnorm(nrow(panel))
  panel$treated <- as.integer(panel$unit <= 10)
  ev_q <- mb_event_study(panel$y, panel$unit, panel$time,
                         treatment_time = 5, treated = panel$treated,
                         leads = 2, lags = 2, quiet = TRUE)
  expect_true(ev_q$quiet)
})

test_that("mb_event_study cluster argument toggles cluster_robust flag", {
  set.seed(2)
  panel <- expand.grid(unit = 1:30, time = 1:8)
  panel$y <- rnorm(nrow(panel))
  panel$treated <- as.integer(panel$unit <= 15)
  ev_ols <- mb_event_study(panel$y, panel$unit, panel$time,
                           treatment_time = 5, treated = panel$treated,
                           leads = 2, lags = 2)
  ev_cr  <- mb_event_study(panel$y, panel$unit, panel$time,
                           treatment_time = 5, treated = panel$treated,
                           leads = 2, lags = 2,
                           cluster = panel$unit)
  expect_false(ev_ols$cluster_robust)
  expect_true(ev_cr$cluster_robust)
  # SEs should generally differ (not by exactly zero) once clustering is added
  expect_false(isTRUE(all.equal(ev_ols$se, ev_cr$se)))
})

test_that("mb_event_study cluster errors on length mismatch", {
  set.seed(3)
  panel <- expand.grid(unit = 1:30, time = 1:8)
  panel$y <- rnorm(nrow(panel))
  panel$treated <- as.integer(panel$unit <= 15)
  expect_error(
    mb_event_study(panel$y, panel$unit, panel$time,
                   treatment_time = 5, treated = panel$treated,
                   leads = 2, lags = 2,
                   cluster = c(1, 2, 3)),
    "must have length"
  )
})

test_that("mb_event_study cluster errors with single cluster", {
  set.seed(4)
  panel <- expand.grid(unit = 1:30, time = 1:8)
  panel$y <- rnorm(nrow(panel))
  panel$treated <- as.integer(panel$unit <= 15)
  expect_error(
    mb_event_study(panel$y, panel$unit, panel$time,
                   treatment_time = 5, treated = panel$treated,
                   leads = 2, lags = 2,
                   cluster = rep(1, nrow(panel))),
    "at least 2 clusters"
  )
})

test_that("mb_its recovers level and slope changes on noiseless data", {
  time <- 1:48
  base <- 10 + 0.05 * time
  step <- ifelse(time >= 25, 2 + 0.1 * (time - 25), 0)
  y    <- base + step
  res  <- mb_its(y, time, intervention_time = 25)
  expect_s3_class(res, "mb_its")
  expect_equal(res$level_change, 2, tolerance = 1e-8)
  expect_equal(res$slope_change, 0.1, tolerance = 1e-8)
})

test_that("mb_its returns coefficients with names", {
  time <- 1:24
  y <- rnorm(24)
  res <- mb_its(y, time, intervention_time = 12)
  expect_named(res$coefficients,
               c("(Intercept)", "time", "post", "post_time"))
  expect_named(res$se,
               c("(Intercept)", "time", "post", "post_time"))
})

test_that("mb_its respects lag parameter", {
  time <- 1:48
  y <- rnorm(48)
  res_no_lag <- mb_its(y, time, intervention_time = 25)
  res_lag    <- mb_its(y, time, intervention_time = 25, lag = 3L)
  expect_equal(res_no_lag$n - res_lag$n, 3L)
})

test_that("mb_its errors on degenerate design", {
  time <- 1:10
  expect_error(
    mb_its(rep(1, 10), time, intervention_time = 100),
    "singular"
  )
})

test_that("mb_its counts pre and post observations", {
  time <- 1:30
  y <- rnorm(30)
  res <- mb_its(y, time, intervention_time = 15)
  expect_equal(res$n_pre + res$n_post, 30L)
  expect_equal(res$n_post, 16L)
})

test_that("mb_its validates lag is non-negative integer", {
  time <- 1:24
  y <- rnorm(24)
  expect_error(mb_its(y, time, intervention_time = 12, lag = -1), "non-negative")
  expect_error(mb_its(y, time, intervention_time = 12, lag = 1.5), "non-negative integer")
})

#' Simple event-study coefficients
#'
#' Estimates a panel event-study with unit and time fixed effects
#' and event-time dummies. Treatment time is fixed across treated
#' units (no staggered adoption). Returns coefficients for `leads`
#' periods before and `lags` periods after treatment, with the
#' period immediately before treatment (`event_time = -1`) omitted
#' as the reference category.
#'
#' @param y Numeric vector of outcomes.
#' @param unit Vector identifying units (panel ID).
#' @param time Numeric vector of time indices.
#' @param treatment_time Numeric scalar. The first treated period.
#'   Units with `treated = 0` (never-treated) are pure controls.
#' @param treated Logical or 0/1 numeric vector indicating
#'   whether each observation belongs to a treated unit. The design
#'   requires at least some never-treated control units; without
#'   them the event-time dummies are collinear with the time fixed
#'   effects.
#' @param leads Integer >= 0. Number of pre-treatment periods to
#'   include. Default `3`.
#' @param lags Integer >= 0. Number of post-treatment periods.
#'   Default `3`.
#' @param cluster Optional vector identifying clusters for
#'   cluster-robust standard errors (CR1 with finite-sample
#'   correction `(G/(G-1)) * (N-1)/(N-K)`). Common choice: pass
#'   `unit` to cluster at the unit level. If `NULL` (default),
#'   conventional OLS SEs are returned.
#' @param quiet Logical. If `FALSE` (default), the print method
#'   appends a one-line reminder that this is a fixed-treatment-
#'   time event study and points to \pkg{fixest} (`sunab()`) or
#'   \pkg{did} for staggered adoption. Set to `TRUE` to suppress.
#'
#' @return An `mb_event_study` object: a list with `event_time`,
#'   `estimate`, `se`, plus `n`, `n_units`, `n_periods`,
#'   `treatment_time`, and `vintage`.
#'
#' @details
#' Implements the canonical two-way fixed-effects event study:
#' \deqn{Y_{it} = \alpha_i + \gamma_t + \sum_{k \neq -1} \beta_k \mathbf{1}\{t - t^* = k, D_i = 1\} + \epsilon_{it}}
#'
#' For staggered adoption (units treated at different times), this
#' specification is biased under treatment-effect heterogeneity. Use
#' the heterogeneity-robust estimators of Callaway & Sant'Anna
#' (2021) or de Chaisemartin & D'Haultfoeuille (2020), available in
#' the \pkg{did}, \pkg{didimputation}, or \pkg{fixest} packages
#' (`fixest::feols` with `sunab()`).
#'
#' Standard errors are conventional OLS; for clustered inference
#' use \pkg{sandwich} or \pkg{fixest}.
#'
#' @references
#' Callaway, B., Sant'Anna, P. H. C. (2021). Difference-in-Differences
#' with Multiple Time Periods. Journal of Econometrics 225(2).
#' <doi:10.1016/j.jeconom.2020.12.001>.
#'
#' de Chaisemartin, C., D'Haultfoeuille, X. (2020). Two-Way Fixed
#' Effects Estimators with Heterogeneous Treatment Effects. American
#' Economic Review 110(9). <doi:10.1257/aer.20181169>.
#'
#' @family estimators
#' @seealso [mb_did_2x2()], [mb_its()].
#'
#' @export
#' @examples
#' set.seed(3)
#' n_units <- 50; n_periods <- 10; treat_time <- 6
#' panel <- expand.grid(unit = 1:n_units, time = 1:n_periods)
#' panel$treated <- as.integer(panel$unit <= 25)
#' panel$post    <- as.integer(panel$time >= treat_time)
#' panel$y <- 0.1 * panel$time + 0.5 * (panel$treated * panel$post) +
#'            rnorm(nrow(panel))
#' mb_event_study(
#'   y = panel$y, unit = panel$unit, time = panel$time,
#'   treatment_time = treat_time, treated = panel$treated,
#'   leads = 3, lags = 3
#' )
mb_event_study <- function(y, unit, time, treatment_time,
                           treated, leads = 3L, lags = 3L,
                           cluster = NULL, quiet = FALSE) {
  validate_numeric(y,    arg = "y")
  validate_numeric(time, arg = "time")
  if (length(unit) != length(y) || length(time) != length(y)) {
    cli::cli_abort("{.arg y}, {.arg unit}, and {.arg time} must be the same length.")
  }
  validate_numeric(treatment_time, arg = "treatment_time")
  validate_scalar(treatment_time,  arg = "treatment_time")
  validate_numeric(leads, arg = "leads", require_non_negative = TRUE)
  validate_scalar(leads,  arg = "leads")
  validate_numeric(lags,  arg = "lags",  require_non_negative = TRUE)
  validate_scalar(lags,   arg = "lags")
  if (leads != as.integer(leads)) cli::cli_abort("{.arg leads} must be a non-negative integer.")
  if (lags  != as.integer(lags))  cli::cli_abort("{.arg lags} must be a non-negative integer.")

  if (length(treated) != length(y)) {
    cli::cli_abort("{.arg treated} must be the same length as {.arg y}.")
  }
  treated <- as.integer(as.logical(treated))
  if (sum(treated == 0L) == 0L) {
    cli::cli_abort(
      "Need at least one never-treated control unit; otherwise event-time dummies are collinear with time fixed effects."
    )
  }

  event_time <- time - treatment_time
  ks <- setdiff(seq(-leads, lags), -1L)
  D_event <- vapply(ks, function(k) as.integer(event_time == k & treated == 1L), integer(length(y)))
  colnames(D_event) <- sprintf("k_%s", ks)

  unit_f <- as.factor(unit)
  time_f <- as.factor(time)

  X_unit <- stats::model.matrix(~ unit_f - 1)[, -1, drop = FALSE]
  X_time <- stats::model.matrix(~ time_f - 1)[, -1, drop = FALSE]
  X <- cbind(1, X_unit, X_time, D_event)

  XtX_inv <- tryCatch(
    solve(crossprod(X)),
    error = function(e) cli::cli_abort("Design matrix is singular. Check unit/time coverage and event-time grid.")
  )
  beta <- drop(XtX_inv %*% crossprod(X, y))
  resid <- y - drop(X %*% beta)
  n <- length(y); k <- ncol(X)

  cluster_robust <- FALSE
  if (is.null(cluster)) {
    sigma2 <- sum(resid^2) / max(n - k, 1L)
    vcov_b <- sigma2 * XtX_inv
  } else {
    if (length(cluster) != n) {
      cli::cli_abort("{.arg cluster} must have length {.val {n}}.")
    }
    g <- as.factor(cluster)
    G <- nlevels(g)
    if (G < 2L) cli::cli_abort("Need at least 2 clusters for cluster-robust SEs.")
    meat <- matrix(0, nrow = k, ncol = k)
    for (lvl in levels(g)) {
      idx <- which(g == lvl)
      X_g <- X[idx, , drop = FALSE]
      u_g <- resid[idx]
      Xu  <- crossprod(X_g, u_g)
      meat <- meat + tcrossprod(Xu)
    }
    correction <- (G / (G - 1)) * ((n - 1) / (n - k))
    vcov_b <- correction * XtX_inv %*% meat %*% XtX_inv
    cluster_robust <- TRUE
  }

  event_idx <- (k - length(ks) + 1L):k
  estimate <- unname(beta[event_idx])
  se       <- sqrt(diag(vcov_b)[event_idx])

  out <- list(
    event_time     = ks,
    estimate       = estimate,
    se             = se,
    n              = n,
    n_units        = nlevels(unit_f),
    n_periods      = nlevels(time_f),
    treatment_time = treatment_time,
    cluster_robust = cluster_robust,
    quiet          = isTRUE(quiet),
    vintage        = .mb_vintage()
  )
  class(out) <- c("mb_event_study", "list")
  out
}

test_that("mb_event_study returns expected event-time grid", {
  set.seed(3)
  n_units <- 50; n_periods <- 10; tt <- 6
  panel <- expand.grid(unit = 1:n_units, time = 1:n_periods)
  panel$treated <- as.integer(panel$unit <= 25)
  panel$y <- 0.1 * panel$time +
             0.5 * (panel$treated * (panel$time >= tt)) + rnorm(nrow(panel))
  res <- mb_event_study(panel$y, panel$unit, panel$time,
                        treatment_time = tt, treated = panel$treated,
                        leads = 3, lags = 3)
  expect_s3_class(res, "mb_event_study")
  expect_equal(sort(res$event_time), c(-3, -2, 0, 1, 2, 3))
  expect_equal(length(res$estimate), length(res$event_time))
  expect_equal(res$n_units, 50L)
  expect_equal(res$n_periods, 10L)
})

test_that("mb_event_study post-treatment leads are positive in noiseless DGP", {
  n_units <- 50; n_periods <- 10; tt <- 6
  panel <- expand.grid(unit = 1:n_units, time = 1:n_periods)
  panel$treated <- as.integer(panel$unit <= 25)
  panel$y <- 0.5 * (panel$treated * (panel$time >= tt))
  res <- mb_event_study(panel$y, panel$unit, panel$time,
                        treatment_time = tt, treated = panel$treated,
                        leads = 3, lags = 3)
  post_idx <- which(res$event_time >= 0)
  expect_true(all(res$estimate[post_idx] > 0))
})

test_that("mb_event_study errors when no controls are present", {
  panel <- expand.grid(unit = 1:5, time = 1:6)
  panel$y <- rnorm(nrow(panel))
  panel$treated <- 1L
  expect_error(
    mb_event_study(panel$y, panel$unit, panel$time, treatment_time = 4,
                   treated = panel$treated),
    "never-treated"
  )
})

test_that("mb_event_study validates leads/lags as non-negative integers", {
  panel <- expand.grid(unit = 1:5, time = 1:6)
  panel$y <- rnorm(nrow(panel))
  panel$treated <- as.integer(panel$unit <= 3)
  expect_error(
    mb_event_study(panel$y, panel$unit, panel$time, treatment_time = 4,
                   treated = panel$treated, leads = -1),
    "non-negative"
  )
  expect_error(
    mb_event_study(panel$y, panel$unit, panel$time, treatment_time = 4,
                   treated = panel$treated, leads = 1.5),
    "non-negative integer"
  )
})

test_that("mb_event_study errors on length mismatch", {
  expect_error(
    mb_event_study(rnorm(10), 1:5, 1:5, treatment_time = 3, treated = c(1, 0)),
    "same length"
  )
})

#' Aggregate evaluation report
#'
#' Composes the components produced by other `magentabook`
#' functions into a single report object: theory of change,
#' evaluation plan, SMS ratings, confidence ratings,
#' cost-effectiveness analyses. Any component may be omitted.
#'
#' @param plan Optional `mb_plan` from [mb_evaluation_plan()].
#' @param toc Optional `mb_toc` from [mb_theory_of_change()].
#' @param sms Optional `mb_sms_rating` or list of them.
#' @param confidence Optional `mb_confidence`,
#'   `mb_confidence_summary`, or list of `mb_confidence`.
#' @param cea Optional `mb_cea`, `mb_icer`, or list of them.
#' @param name Optional character scalar naming the evaluation.
#'
#' @return An `mb_report` object.
#'
#' @family reporting
#' @seealso [mb_to_word()], [mb_to_excel()], [mb_to_latex()].
#'
#' @export
#' @examples
#' toc <- mb_theory_of_change(
#'   inputs = "Funding", activities = "Workshops",
#'   outputs = "Attendees", outcomes = "Skills",
#'   impact = "Employment"
#' )
#' mb_evaluation_report(toc = toc, name = "Skills uplift evaluation")
mb_evaluation_report <- function(plan       = NULL,
                                 toc        = NULL,
                                 sms        = NULL,
                                 confidence = NULL,
                                 cea        = NULL,
                                 name       = NULL) {
  if (!is.null(plan)       && !inherits(plan, "mb_plan")) cli::cli_abort("{.arg plan} must be an {.cls mb_plan}.")
  if (!is.null(toc)        && !inherits(toc,  "mb_toc"))  cli::cli_abort("{.arg toc} must be an {.cls mb_toc}.")
  if (!is.null(name)) { validate_character(name, arg = "name"); validate_scalar(name, arg = "name") }

  .normalise <- function(x, classes) {
    if (is.null(x)) return(NULL)
    if (any(vapply(classes, function(cl) inherits(x, cl), logical(1)))) return(list(x))
    if (is.list(x)) {
      ok <- vapply(x, function(el) any(vapply(classes, function(cl) inherits(el, cl), logical(1))), logical(1))
      if (!all(ok)) cli::cli_abort("All elements must be one of {.cls {classes}}.")
      return(x)
    }
    cli::cli_abort("Input must be one of {.cls {classes}} or a list of them.")
  }
  sms_list <- .normalise(sms,        c("mb_sms_rating"))
  cnf_list <- .normalise(confidence, c("mb_confidence", "mb_confidence_summary"))
  cea_list <- .normalise(cea,        c("mb_cea", "mb_icer"))

  out <- list(
    name       = name,
    plan       = plan,
    toc        = toc,
    sms        = sms_list,
    confidence = cnf_list,
    cea        = cea_list,
    vintage    = .mb_vintage()
  )
  class(out) <- c("mb_report", "list")
  out
}

#' Export an evaluation report to Word
#'
#' Writes a one- to two-page Word document summarising an
#' `mb_report`: name, theory of change, evaluation plan, SMS
#' ratings, confidence ratings, and cost-effectiveness.
#'
#' @param report An `mb_report` object.
#' @param file Output file path (must end in `.docx`).
#'
#' @return Invisibly, the file path.
#'
#' @details
#' Requires the \pkg{officer} and \pkg{flextable} packages (both in
#' Suggests).
#'
#' @family reporting
#' @seealso [mb_evaluation_report()], [mb_to_excel()],
#'   [mb_to_latex()].
#'
#' @export
#' @examples
#' \donttest{
#' if (requireNamespace("officer", quietly = TRUE) &&
#'     requireNamespace("flextable", quietly = TRUE)) {
#'   toc <- mb_theory_of_change(
#'     inputs = "Funding", activities = "Workshops",
#'     outputs = "Attendees", outcomes = "Skills",
#'     impact = "Employment"
#'   )
#'   rep <- mb_evaluation_report(toc = toc, name = "Skills uplift")
#'   tmp <- tempfile(fileext = ".docx")
#'   mb_to_word(rep, tmp)
#' }
#' }
mb_to_word <- function(report, file) {
  if (!requireNamespace("officer", quietly = TRUE) ||
      !requireNamespace("flextable", quietly = TRUE)) {
    cli::cli_abort(
      "Packages {.pkg officer} and {.pkg flextable} are required for {.fn mb_to_word}."
    )
  }
  if (!inherits(report, "mb_report")) {
    cli::cli_abort("{.arg report} must be an {.cls mb_report}.")
  }
  if (!grepl("\\.docx$", file)) {
    cli::cli_abort("{.arg file} must end in {.val .docx}.")
  }

  doc <- officer::read_docx()
  title <- report$name %||% "Magenta Book evaluation"
  doc <- officer::body_add_par(doc, title, style = "heading 1")

  if (!is.null(report$toc)) {
    doc <- officer::body_add_par(doc, "Theory of change", style = "heading 2")
    lf <- mb_logframe(report$toc)
    doc <- flextable::body_add_flextable(doc, flextable::flextable(as.data.frame(lf)))
  }
  if (!is.null(report$plan)) {
    doc <- officer::body_add_par(doc, "Evaluation plan", style = "heading 2")
    doc <- officer::body_add_par(doc, paste("Scope:", report$plan$scope))
    doc <- flextable::body_add_flextable(doc, flextable::flextable(as.data.frame(report$plan$questions)))
  }
  if (!is.null(report$sms)) {
    doc <- officer::body_add_par(doc, "Maryland SMS ratings", style = "heading 2")
    sms_df <- do.call(rbind, lapply(report$sms, function(s) {
      data.frame(level = s$level, label = s$label, study = s$study,
                 design = s$design %||% NA_character_,
                 stringsAsFactors = FALSE)
    }))
    doc <- flextable::body_add_flextable(doc, flextable::flextable(sms_df))
  }
  if (!is.null(report$confidence)) {
    doc <- officer::body_add_par(doc, "Confidence ratings", style = "heading 2")
    cnf_df <- do.call(rbind, lapply(report$confidence, function(c) {
      if (inherits(c, "mb_confidence_summary")) return(c$ratings)
      data.frame(question = c$question, rating = c$rating,
                 rationale = c$rationale, stringsAsFactors = FALSE)
    }))
    doc <- flextable::body_add_flextable(doc, flextable::flextable(cnf_df))
  }
  if (!is.null(report$cea)) {
    doc <- officer::body_add_par(doc, "Cost-effectiveness", style = "heading 2")
    cea_df <- do.call(rbind, lapply(report$cea, function(x) {
      if (inherits(x, "mb_cea")) {
        data.frame(type = "CEA", label = x$label %||% NA, cost = x$total_cost,
                   effect = x$total_effect, ratio = x$cost_per_unit,
                   stringsAsFactors = FALSE)
      } else {
        data.frame(type = "ICER", label = sprintf("%s vs %s", x$label_b, x$label_a),
                   cost = x$delta_cost, effect = x$delta_effect, ratio = x$icer,
                   stringsAsFactors = FALSE)
      }
    }))
    doc <- flextable::body_add_flextable(doc, flextable::flextable(cea_df))
  }

  print(doc, target = file)
  invisible(file)
}

#' Export an evaluation report to Excel
#'
#' Writes a multi-sheet workbook with one sheet per component:
#' summary, theory of change, plan, SMS ratings, confidence
#' ratings, cost-effectiveness, provenance.
#'
#' @inheritParams mb_to_word
#' @param file Output file path (must end in `.xlsx`).
#'
#' @return Invisibly, the file path.
#'
#' @details
#' Requires the \pkg{openxlsx} package (in Suggests).
#'
#' @family reporting
#' @seealso [mb_to_word()], [mb_to_latex()].
#'
#' @export
#' @examples
#' \donttest{
#' if (requireNamespace("openxlsx", quietly = TRUE)) {
#'   toc <- mb_theory_of_change(
#'     inputs = "Funding", activities = "Workshops",
#'     outputs = "Attendees", outcomes = "Skills",
#'     impact = "Employment"
#'   )
#'   rep <- mb_evaluation_report(toc = toc, name = "Skills uplift")
#'   tmp <- tempfile(fileext = ".xlsx")
#'   mb_to_excel(rep, tmp)
#' }
#' }
mb_to_excel <- function(report, file) {
  if (!requireNamespace("openxlsx", quietly = TRUE)) {
    cli::cli_abort(
      "Package {.pkg openxlsx} is required for {.fn mb_to_excel}. Install with {.code install.packages('openxlsx')}."
    )
  }
  if (!inherits(report, "mb_report")) {
    cli::cli_abort("{.arg report} must be an {.cls mb_report}.")
  }
  if (!grepl("\\.xlsx$", file)) {
    cli::cli_abort("{.arg file} must end in {.val .xlsx}.")
  }

  wb <- openxlsx::createWorkbook()
  openxlsx::addWorksheet(wb, "Summary")
  openxlsx::writeData(wb, "Summary", data.frame(
    component = c("name", "vintage", "has_toc", "has_plan", "n_sms", "n_confidence", "n_cea"),
    value     = c(
      report$name %||% "",
      as.character(report$vintage),
      !is.null(report$toc),
      !is.null(report$plan),
      length(report$sms %||% list()),
      length(report$confidence %||% list()),
      length(report$cea %||% list())
    ),
    stringsAsFactors = FALSE
  ))

  if (!is.null(report$toc)) {
    openxlsx::addWorksheet(wb, "Theory of change")
    openxlsx::writeData(wb, "Theory of change", as.data.frame(mb_logframe(report$toc)))
  }
  if (!is.null(report$plan)) {
    openxlsx::addWorksheet(wb, "Plan")
    openxlsx::writeData(wb, "Plan", as.data.frame(report$plan$questions))
  }
  if (!is.null(report$sms)) {
    openxlsx::addWorksheet(wb, "SMS")
    sms_df <- do.call(rbind, lapply(report$sms, function(s) {
      data.frame(level = s$level, label = s$label, study = s$study,
                 design = s$design %||% NA_character_,
                 notes = s$notes %||% NA_character_,
                 stringsAsFactors = FALSE)
    }))
    openxlsx::writeData(wb, "SMS", sms_df)
  }
  if (!is.null(report$confidence)) {
    openxlsx::addWorksheet(wb, "Confidence")
    cnf_df <- do.call(rbind, lapply(report$confidence, function(c) {
      if (inherits(c, "mb_confidence_summary")) return(c$ratings)
      data.frame(question = c$question, rating = c$rating,
                 rationale = c$rationale, stringsAsFactors = FALSE)
    }))
    openxlsx::writeData(wb, "Confidence", cnf_df)
  }
  if (!is.null(report$cea)) {
    openxlsx::addWorksheet(wb, "Cost-effectiveness")
    cea_df <- do.call(rbind, lapply(report$cea, function(x) {
      if (inherits(x, "mb_cea")) {
        data.frame(type = "CEA", label = x$label %||% NA_character_,
                   cost = x$total_cost, effect = x$total_effect,
                   ratio = x$cost_per_unit, stringsAsFactors = FALSE)
      } else {
        data.frame(type = "ICER",
                   label = sprintf("%s vs %s", x$label_b, x$label_a),
                   cost = x$delta_cost, effect = x$delta_effect,
                   ratio = x$icer, stringsAsFactors = FALSE)
      }
    }))
    openxlsx::writeData(wb, "Cost-effectiveness", cea_df)
  }

  openxlsx::addWorksheet(wb, "Provenance")
  openxlsx::writeData(wb, "Provenance", mb_data_versions())

  openxlsx::saveWorkbook(wb, file, overwrite = TRUE)
  invisible(file)
}

#' Render an evaluation report as a LaTeX table
#'
#' Returns a single LaTeX `tabular` summarising the report.
#' Multi-sheet Word/Excel exports are richer; LaTeX is intended for
#' insertion into a one-pager.
#'
#' @inheritParams mb_to_word
#' @param caption Optional table caption.
#' @param label Optional LaTeX label for cross-referencing.
#'
#' @return A character scalar containing a LaTeX `tabular`
#'   environment.
#'
#' @family reporting
#' @seealso [mb_to_word()], [mb_to_excel()].
#'
#' @export
#' @examples
#' toc <- mb_theory_of_change(
#'   inputs = "Funding", activities = "Workshops",
#'   outputs = "Attendees", outcomes = "Skills",
#'   impact = "Employment"
#' )
#' rep <- mb_evaluation_report(toc = toc, name = "Skills uplift")
#' cat(mb_to_latex(rep))
mb_to_latex <- function(report, caption = NULL, label = NULL) {
  if (!inherits(report, "mb_report")) {
    cli::cli_abort("{.arg report} must be an {.cls mb_report}.")
  }
  rows <- c(
    sprintf("Name & %s \\\\", report$name %||% "n/a"),
    sprintf("Vintage & magentabook %s \\\\", as.character(report$vintage)),
    sprintf("Has theory of change & %s \\\\", if (!is.null(report$toc)) "yes" else "no"),
    sprintf("Has plan & %s \\\\", if (!is.null(report$plan)) "yes" else "no"),
    sprintf("SMS ratings & %d \\\\", length(report$sms %||% list())),
    sprintf("Confidence ratings & %d \\\\", length(report$confidence %||% list())),
    sprintf("Cost-effectiveness items & %d \\\\", length(report$cea %||% list()))
  )
  body <- paste(rows, collapse = "\n")
  preamble <- "\\begin{tabular}{ll}\n\\hline\nComponent & Value \\\\\n\\hline\n"
  closing  <- "\n\\hline\n\\end{tabular}"
  out <- paste0(preamble, body, closing)
  if (!is.null(caption) || !is.null(label)) {
    cap <- if (!is.null(caption)) sprintf("\\caption{%s}\n", caption) else ""
    lab <- if (!is.null(label))   sprintf("\\label{%s}\n",   label)   else ""
    out <- sprintf("\\begin{table}[h]\n\\centering\n%s%s%s\n\\end{table}",
                   out, cap, lab)
  }
  out
}

test_that("mb_evaluation_report builds an empty report", {
  r <- mb_evaluation_report(name = "test")
  expect_s3_class(r, "mb_report")
  expect_equal(r$name, "test")
  expect_null(r$toc); expect_null(r$plan)
})

test_that("mb_evaluation_report composes components", {
  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  qs  <- mb_questions("q", "impact")
  plan <- mb_evaluation_plan("scope", qs, "RCT", "2026", "steering")
  sms  <- mb_sms_rate(5, "RCT")
  conf <- mb_confidence("high", "q", "es", "mq", "g", "r")
  cea  <- mb_cea(cost = 1000, effect = 10)
  rep  <- mb_evaluation_report(plan = plan, toc = toc, sms = sms,
                               confidence = conf, cea = cea, name = "test")
  expect_s3_class(rep, "mb_report")
  expect_equal(length(rep$sms), 1L)
  expect_equal(length(rep$confidence), 1L)
  expect_equal(length(rep$cea), 1L)
})

test_that("mb_evaluation_report accepts lists of components", {
  s1 <- mb_sms_rate(4, "study1"); s2 <- mb_sms_rate(5, "study2")
  rep <- mb_evaluation_report(sms = list(s1, s2))
  expect_equal(length(rep$sms), 2L)
})

test_that("mb_evaluation_report rejects wrong types", {
  expect_error(
    mb_evaluation_report(toc = "not a toc"),
    "mb_toc"
  )
  expect_error(
    mb_evaluation_report(sms = "not a rating"),
    "mb_sms_rating"
  )
})

test_that("mb_to_latex returns a tabular string", {
  rep <- mb_evaluation_report(name = "test")
  out <- mb_to_latex(rep)
  expect_type(out, "character")
  expect_match(out, "tabular")
  expect_match(out, "test")
})

test_that("mb_to_latex wraps in table when caption supplied", {
  rep <- mb_evaluation_report(name = "test")
  out <- mb_to_latex(rep, caption = "My eval", label = "tab:eval")
  expect_match(out, "begin\\{table\\}")
  expect_match(out, "caption\\{My eval\\}")
  expect_match(out, "label\\{tab:eval\\}")
})

test_that("mb_to_word and mb_to_excel error helpfully without optional packages", {
  # When the optional packages are installed, just check the file extension guard.
  rep <- mb_evaluation_report(name = "test")
  if (requireNamespace("officer", quietly = TRUE) &&
      requireNamespace("flextable", quietly = TRUE)) {
    expect_error(mb_to_word(rep, "out.txt"), "must end in")
  } else {
    expect_error(mb_to_word(rep, "out.docx"), "officer")
  }
  if (requireNamespace("openxlsx", quietly = TRUE)) {
    expect_error(mb_to_excel(rep, "out.txt"), "must end in")
  } else {
    expect_error(mb_to_excel(rep, "out.xlsx"), "openxlsx")
  }
})

test_that("mb_to_excel writes a workbook", {
  skip_if_not_installed("openxlsx")
  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  rep <- mb_evaluation_report(toc = toc, name = "x")
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)
  mb_to_excel(rep, tmp)
  expect_true(file.exists(tmp))
})

test_that("mb_to_word writes a docx", {
  skip_if_not_installed("officer")
  skip_if_not_installed("flextable")
  toc <- mb_theory_of_change("i", "a", "o", "u", "z")
  rep <- mb_evaluation_report(toc = toc, name = "x")
  tmp <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp), add = TRUE)
  mb_to_word(rep, tmp)
  expect_true(file.exists(tmp))
})

#' Canonical 2x2 difference-in-differences estimator
#'
#' Returns the simple two-period, two-group DiD estimate of an
#' average treatment effect on the treated, with optional
#' cluster-robust standard errors.
#'
#' @param y Numeric vector of outcomes.
#' @param treated Logical or 0/1 numeric vector. `TRUE` / `1` if
#'   the unit is in the treated group, regardless of period.
#' @param post Logical or 0/1 numeric vector. `TRUE` / `1` if the
#'   observation is in the post-treatment period.
#' @param cluster Optional vector identifying clusters for
#'   cluster-robust standard errors (CR1 with finite-sample
#'   correction). If `NULL`, conventional OLS SEs are returned.
#' @param alpha Numeric in `(0, 1)`. Significance level for the
#'   confidence interval. Default `0.05`.
#' @param quiet Logical. If `FALSE` (default), the print method
#'   appends a one-line reminder that this is a canonical 2x2 DiD
#'   and points to specialist tooling for staggered adoption or
#'   heterogeneous treatment effects. Set to `TRUE` to suppress.
#'
#' @return An `mb_did` object: a list with `estimate`, `se`,
#'   `t_stat`, `p_value`, `ci_low`, `ci_high`, group means,
#'   `cluster_robust`, `n`, `quiet`, and `vintage`.
#'
#' @details
#' Computes
#' \deqn{\hat{\tau} = (\bar{Y}_{T,1} - \bar{Y}_{T,0}) - (\bar{Y}_{C,1} - \bar{Y}_{C,0})}
#' which equals the coefficient on the `treated:post` interaction in
#' \eqn{Y = \beta_0 + \beta_1 T + \beta_2 P + \tau (T \times P) + \epsilon}.
#'
#' Cluster-robust SEs use the CR1 sandwich estimator with
#' finite-sample correction \eqn{(G/(G-1)) \cdot (N-1)/(N-K)}, where
#' \eqn{G} is the number of clusters, \eqn{N} the number of
#' observations, and \eqn{K} the number of regressors (4).
#'
#' For staggered adoption, heterogeneous treatment effects, or
#' production estimation, use \pkg{fixest}, \pkg{did}, or
#' \pkg{Synth}. This function is for the canonical 2x2 case only.
#'
#' @references
#' Card, D., Krueger, A. B. (1994). Minimum Wages and Employment:
#' A Case Study of the Fast-Food Industry in New Jersey and
#' Pennsylvania. American Economic Review 84(4).
#' <doi:10.1257/aer.84.4.772>.
#'
#' Cameron, A. C., Miller, D. L. (2015). A Practitioner's Guide to
#' Cluster-Robust Inference. Journal of Human Resources 50(2).
#' <doi:10.3368/jhr.50.2.317>.
#'
#' @family estimators
#' @seealso [mb_its()], [mb_event_study()].
#'
#' @export
#' @examples
#' set.seed(1)
#' n <- 400
#' treated <- rep(c(0, 1), each = n / 2)
#' post    <- rep(c(0, 1), times = n / 2)
#' y       <- 0.5 * treated + 0.2 * post + 0.4 * treated * post + rnorm(n)
#' mb_did_2x2(y, treated, post)
mb_did_2x2 <- function(y, treated, post, cluster = NULL, alpha = 0.05,
                       quiet = FALSE) {
  validate_numeric(y, arg = "y")
  if (length(treated) != length(y)) {
    cli::cli_abort("{.arg treated} must be the same length as {.arg y}.")
  }
  if (length(post) != length(y)) {
    cli::cli_abort("{.arg post} must be the same length as {.arg y}.")
  }
  validate_proportion(alpha, arg = "alpha", strict = TRUE)
  treated <- as.numeric(as.logical(treated))
  post    <- as.numeric(as.logical(post))
  if (anyNA(treated) || anyNA(post)) {
    cli::cli_abort("{.arg treated} and {.arg post} must be logical or 0/1 numeric without NA.")
  }
  if (!all(treated %in% 0:1)) cli::cli_abort("{.arg treated} must coerce to 0/1.")
  if (!all(post    %in% 0:1)) cli::cli_abort("{.arg post} must coerce to 0/1.")

  n <- length(y)
  X <- cbind(1, treated, post, treated * post)
  colnames(X) <- c("(Intercept)", "treated", "post", "treated:post")

  XtX_inv <- tryCatch(
    solve(crossprod(X)),
    error = function(e) cli::cli_abort("Design matrix is singular; check that all four cells are populated.")
  )
  beta <- drop(XtX_inv %*% crossprod(X, y))
  resid <- y - drop(X %*% beta)
  k <- ncol(X)

  if (is.null(cluster)) {
    sigma2  <- sum(resid^2) / (n - k)
    vcov_b  <- sigma2 * XtX_inv
    cluster_robust <- FALSE
  } else {
    if (length(cluster) != n) {
      cli::cli_abort("{.arg cluster} must have length {.val {n}}.")
    }
    g <- as.factor(cluster)
    G <- nlevels(g)
    if (G < 2L) cli::cli_abort("Need at least 2 clusters for cluster-robust SEs.")
    meat <- matrix(0, nrow = k, ncol = k)
    for (lvl in levels(g)) {
      idx <- which(g == lvl)
      X_g <- X[idx, , drop = FALSE]
      u_g <- resid[idx]
      Xu  <- crossprod(X_g, u_g)
      meat <- meat + tcrossprod(Xu)
    }
    correction <- (G / (G - 1)) * ((n - 1) / (n - k))
    vcov_b <- correction * XtX_inv %*% meat %*% XtX_inv
    cluster_robust <- TRUE
  }

  estimate <- unname(beta[4])
  se       <- sqrt(unname(vcov_b[4, 4]))
  t_stat   <- estimate / se
  df       <- if (cluster_robust) (length(unique(cluster)) - 1L) else (n - k)
  p_value  <- 2 * (1 - stats::pt(abs(t_stat), df = df))
  cv       <- stats::qt(1 - alpha / 2, df = df)
  ci_low   <- estimate - cv * se
  ci_high  <- estimate + cv * se

  group_means <- c(
    control_pre   = mean(y[treated == 0 & post == 0]),
    control_post  = mean(y[treated == 0 & post == 1]),
    treated_pre   = mean(y[treated == 1 & post == 0]),
    treated_post  = mean(y[treated == 1 & post == 1])
  )

  out <- list(
    estimate       = estimate,
    se             = se,
    t_stat         = t_stat,
    p_value        = p_value,
    ci_low         = ci_low,
    ci_high        = ci_high,
    df             = df,
    group_means    = group_means,
    cluster_robust = cluster_robust,
    n              = n,
    quiet          = isTRUE(quiet),
    vintage        = .mb_vintage()
  )
  class(out) <- c("mb_did", "list")
  out
}

# Cross-validation of mb_did_2x2 cluster-robust SEs against
# sandwich::vcovCL. Both should compute the CR1 estimator with
# Stata-style finite-sample correction (G/(G-1)) * (N-1)/(N-K).

skip_if_not_installed("sandwich")

test_that("mb_did_2x2 cluster-robust SE agrees with sandwich::vcovCL HC1", {
  set.seed(20260427)
  n <- 800
  treated <- rep(c(0, 1), each = n / 2)
  post    <- rep(c(0, 1), times = n / 2)
  cluster <- rep(seq_len(n / 8), each = 8)
  y <- 0.4 * treated * post + rnorm(n) +
       rep(rnorm(n / 8, sd = 0.3), each = 8)

  mb <- mb_did_2x2(y, treated, post, cluster = cluster)

  fit <- stats::lm(y ~ treated * post)
  vc  <- sandwich::vcovCL(fit, cluster = cluster, type = "HC1")
  sw_se <- sqrt(diag(vc)[["treated:post"]])

  expect_equal(mb$se, unname(sw_se), tolerance = 1e-6)
})

test_that("mb_did_2x2 cluster-robust SE agrees with sandwich for unbalanced clusters", {
  set.seed(2)
  n_clusters <- 40
  cluster_sizes <- sample(8:24, n_clusters, replace = TRUE)
  cluster <- rep(seq_len(n_clusters), cluster_sizes)
  n <- length(cluster)
  treated <- as.integer(cluster %in% sample(seq_len(n_clusters), n_clusters / 2))
  post    <- as.integer(stats::runif(n) > 0.5)
  y <- 0.5 * treated * post + rnorm(n) +
       rep(rnorm(n_clusters, sd = 0.4), cluster_sizes)

  mb <- mb_did_2x2(y, treated, post, cluster = cluster)

  fit <- stats::lm(y ~ treated * post)
  vc  <- sandwich::vcovCL(fit, cluster = cluster, type = "HC1")
  sw_se <- sqrt(diag(vc)[["treated:post"]])

  expect_equal(mb$se, unname(sw_se), tolerance = 1e-6)
})

test_that("mb_did_2x2 OLS SE agrees with conventional lm SE", {
  set.seed(3)
  n <- 200
  treated <- rep(c(0, 1), each = n / 2)
  post    <- rep(c(0, 1), times = n / 2)
  y       <- 0.3 * treated * post + rnorm(n)

  mb <- mb_did_2x2(y, treated, post)

  fit  <- stats::lm(y ~ treated * post)
  sumr <- summary(fit)
  ols_se <- sumr$coefficients["treated:post", "Std. Error"]

  expect_equal(mb$se, ols_se, tolerance = 1e-10)
})

test_that("mb_did_2x2 recovers a known effect on noiseless data", {
  # Construct deterministic data with a known DiD effect of 2.0:
  # Control pre = 1, Control post = 2 (time effect = 1)
  # Treated pre = 3, Treated post = 6 (time effect = 1, plus DiD = 2)
  treated <- rep(c(0, 1), each = 100)
  post    <- rep(c(0, 1), times = 100)
  y <- ifelse(treated == 0 & post == 0, 1,
        ifelse(treated == 0 & post == 1, 2,
         ifelse(treated == 1 & post == 0, 3, 6)))
  res <- mb_did_2x2(y, treated, post)
  expect_s3_class(res, "mb_did")
  expect_equal(res$estimate, 2)
  # On constant-within-cell data SE will be 0; CI collapses to point.
  expect_equal(res$ci_low, res$ci_high, tolerance = 1e-8)
})

test_that("mb_did_2x2 cluster-robust SE at least as large as OLS SE on iid data", {
  set.seed(20260427)
  n <- 800
  treated <- rep(c(0, 1), each = n / 2)
  post    <- rep(c(0, 1), times = n / 2)
  y       <- 0.4 * treated * post + rnorm(n)
  cluster <- rep(seq_len(n / 4), each = 4)
  ols <- mb_did_2x2(y, treated, post)
  cr  <- mb_did_2x2(y, treated, post, cluster = cluster)
  expect_true(cr$se >= ols$se * 0.5)
  expect_true(cr$cluster_robust)
  expect_false(ols$cluster_robust)
})

test_that("mb_did_2x2 estimate matches lm interaction coefficient", {
  set.seed(1)
  n <- 200
  treated <- rep(c(0, 1), each = n / 2)
  post    <- rep(c(0, 1), times = n / 2)
  y       <- 0.5 * treated + 0.2 * post + 0.4 * treated * post + rnorm(n)
  res     <- mb_did_2x2(y, treated, post)
  lm_est  <- coef(stats::lm(y ~ treated * post))[["treated:post"]]
  expect_equal(res$estimate, unname(lm_est))
})

test_that("mb_did_2x2 returns four group means", {
  set.seed(2)
  treated <- rep(c(0, 1), each = 50)
  post    <- rep(c(0, 1), times = 50)
  y       <- rnorm(100)
  res     <- mb_did_2x2(y, treated, post)
  expect_named(
    res$group_means,
    c("control_pre", "control_post", "treated_pre", "treated_post")
  )
})

test_that("mb_did_2x2 errors on length mismatch", {
  expect_error(
    mb_did_2x2(rnorm(10), c(0, 1), c(0, 1)),
    "same length"
  )
})

test_that("mb_did_2x2 errors on degenerate design", {
  expect_error(
    mb_did_2x2(rep(1, 4), c(0, 0, 1, 1), c(0, 0, 1, 1)),
    "singular"
  )
})

test_that("mb_did_2x2 cluster errors with single cluster", {
  treated <- rep(c(0, 1), each = 4)
  post    <- rep(c(0, 1), times = 4)
  y       <- rnorm(8)
  expect_error(
    mb_did_2x2(y, treated, post, cluster = rep(1, 8)),
    "at least 2 clusters"
  )
})

# S3 print and summary methods for magentabook result classes

#' @export
print.mb_toc <- function(x, ...) {
  cli::cli_h1("Theory of change{if (!is.null(x$name)) paste0(': ', x$name) else ''}")
  cli::cli_text("{.strong Inputs}: {paste(x$inputs, collapse = '; ')}")
  cli::cli_text("{.strong Activities}: {paste(x$activities, collapse = '; ')}")
  cli::cli_text("{.strong Outputs}: {paste(x$outputs, collapse = '; ')}")
  cli::cli_text("{.strong Outcomes}: {paste(x$outcomes, collapse = '; ')}")
  cli::cli_text("{.strong Impact}: {paste(x$impact, collapse = '; ')}")
  if (!is.null(x$assumptions)) {
    cli::cli_text("{.strong Assumptions}: {paste(x$assumptions, collapse = '; ')}")
  }
  if (!is.null(x$external_factors)) {
    cli::cli_text("{.strong External factors}: {paste(x$external_factors, collapse = '; ')}")
  }
  cli::cli_text("Vintage: magentabook {.val {as.character(x$vintage)}}")
  invisible(x)
}

#' @export
print.mb_logframe <- function(x, ...) {
  nm <- attr(x, "name")
  cli::cli_h1("Logframe{if (!is.null(nm)) paste0(': ', nm) else ''}")
  print.data.frame(x)
  invisible(x)
}

#' @export
print.mb_assumption_register <- function(x, ...) {
  cli::cli_h1("Assumption register ({nrow(x)} item{if (nrow(x) != 1L) 's' else ''})")
  print.data.frame(x)
  invisible(x)
}

#' @export
print.mb_questions <- function(x, ...) {
  cli::cli_h1("Evaluation questions ({nrow(x)} item{if (nrow(x) != 1L) 's' else ''})")
  print.data.frame(x)
  invisible(x)
}

#' @export
print.mb_counterfactual <- function(x, ...) {
  cli::cli_h1("Counterfactual")
  cli::cli_text("{.strong Definition}: {x$definition}")
  cli::cli_text("{.strong Source}: {x$source}")
  if (!is.na(x$credibility)) {
    cli::cli_text("{.strong Credibility}: {x$credibility}")
  }
  invisible(x)
}

#' @export
print.mb_stakeholders <- function(x, ...) {
  cli::cli_h1("Stakeholders ({nrow(x)} item{if (nrow(x) != 1L) 's' else ''})")
  print.data.frame(x)
  invisible(x)
}

#' @export
print.mb_plan <- function(x, ...) {
  cli::cli_h1("Evaluation plan")
  cli::cli_text("{.strong Scope}: {x$scope}")
  cli::cli_text("{.strong Questions}: {.val {nrow(x$questions)}} (primary: {.val {sum(x$questions$priority == 'primary')}})")
  if (length(x$methods)) {
    if (!is.null(names(x$methods))) {
      for (i in seq_along(x$methods)) {
        cli::cli_text("{.strong Method ({names(x$methods)[i]})}: {x$methods[[i]]}")
      }
    } else {
      cli::cli_text("{.strong Methods}: {paste(x$methods, collapse = '; ')}")
    }
  }
  if (length(x$timing)) {
    cli::cli_text("{.strong Timing}: {paste(unlist(x$timing), collapse = '; ')}")
  }
  if (length(x$governance)) {
    cli::cli_text("{.strong Governance}: {paste(x$governance, collapse = '; ')}")
  }
  if (!is.null(x$budget)) {
    budget_str <- .format_gbp(x$budget)
    cli::cli_text("{.strong Budget}: {.val {budget_str}}")
  }
  cli::cli_text("Vintage: magentabook {.val {as.character(x$vintage)}}")
  invisible(x)
}

#' @export
print.mb_sms_rating <- function(x, ...) {
  cli::cli_h1("Maryland SMS Level {x$level}: {x$label}")
  cli::cli_text("{.strong Study}: {x$study}")
  if (!is.null(x$design)) cli::cli_text("{.strong Design}: {x$design}")
  if (!is.null(x$notes))  cli::cli_text("{.strong Notes}: {x$notes}")
  cli::cli_text("{.strong Description}: {x$short_description}")
  cli::cli_text("{.strong Causal inference}: {x$causal_inference}")
  invisible(x)
}

#' @export
print.mb_confidence <- function(x, ...) {
  cli::cli_h1("{x$label}")
  cli::cli_text("{.strong Question}: {x$question}")
  cli::cli_text("{.strong Evidence strength}: {x$evidence_strength}")
  cli::cli_text("{.strong Methodological quality}: {x$methodological_quality}")
  cli::cli_text("{.strong Generalisability}: {x$generalisability}")
  cli::cli_text("{.strong Rationale}: {x$rationale}")
  cli::cli_text("{.strong Decision implication}: {x$description}")
  invisible(x)
}

#' @export
print.mb_confidence_summary <- function(x, ...) {
  cli::cli_h1("Confidence summary ({x$n} rating{if (x$n != 1L) 's' else ''})")
  for (lvl in x$levels) {
    cli::cli_text("{.strong {lvl}}: {.val {x$counts[[lvl]]}}")
  }
  cli::cli_h2("Ratings")
  print.data.frame(x$ratings)
  invisible(x)
}

#' @export
print.mb_did <- function(x, ...) {
  est_str  <- .format_es(x$estimate)
  se_str   <- .format_es(x$se)
  cil_str  <- .format_es(x$ci_low)
  cih_str  <- .format_es(x$ci_high)
  p_str    <- .format_es(x$p_value, 4)
  se_label <- if (x$cluster_robust) "cluster-robust" else "OLS"
  cli::cli_h1("Difference-in-differences (2x2)")
  cli::cli_text("Estimate: {.val {est_str}}")
  cli::cli_text("SE ({se_label}): {.val {se_str}}")
  cli::cli_text("95% CI: [{.val {cil_str}}, {.val {cih_str}}]")
  cli::cli_text("p-value: {.val {p_str}}")
  cli::cli_text("N = {.val {x$n}}, df = {.val {x$df}}")
  if (!isTRUE(x$quiet)) {
    cli::cli_alert_info(
      "Canonical 2x2 DiD. For staggered adoption or heterogeneous effects, use {.pkg fixest} or {.pkg did}."
    )
  }
  invisible(x)
}

#' @export
summary.mb_did <- function(object, ...) {
  cat("Difference-in-differences (2x2)\n")
  cat("--------------------------------\n")
  cat(sprintf("Estimate         : %.4f\n", object$estimate))
  cat(sprintf("SE (%s) : %.4f\n",
              if (object$cluster_robust) "cluster-robust" else "OLS           ",
              object$se))
  cat(sprintf("t                : %.3f\n", object$t_stat))
  cat(sprintf("p                : %.4f\n", object$p_value))
  cat(sprintf("95%% CI            : [%.4f, %.4f]\n", object$ci_low, object$ci_high))
  cat("\nGroup means:\n")
  print(object$group_means)
  invisible(object)
}

#' @export
print.mb_its <- function(x, ...) {
  level_str <- .format_es(x$level_change)
  slope_str <- .format_es(x$slope_change)
  level_se  <- .format_es(x$se['post'])
  slope_se  <- .format_es(x$se['post_time'])
  cli::cli_h1("Interrupted time series")
  cli::cli_text("Intervention at: {.val {x$intervention_time}}")
  cli::cli_text("Level change: {.val {level_str}} (SE {.val {level_se}})")
  cli::cli_text("Slope change: {.val {slope_str}} (SE {.val {slope_se}})")
  cli::cli_text("N = {.val {x$n}} (pre: {.val {x$n_pre}}, post: {.val {x$n_post}})")
  if (!isTRUE(x$quiet)) {
    cli::cli_alert_info(
      "Single-group segmented regression with OLS SEs. For autocorrelated series use {.pkg sandwich} (Newey-West) or {.pkg nlme} / {.pkg forecast} (ARIMA-error)."
    )
  }
  invisible(x)
}

#' @export
print.mb_event_study <- function(x, ...) {
  se_label <- if (isTRUE(x$cluster_robust)) "cluster-robust" else "OLS"
  cli::cli_h1("Event study")
  cli::cli_text("Treatment at: {.val {x$treatment_time}}")
  cli::cli_text("Units: {.val {x$n_units}}, periods: {.val {x$n_periods}}, N = {.val {x$n}}")
  cli::cli_text("SE: {se_label}")
  df <- data.frame(
    event_time = x$event_time,
    estimate   = x$estimate,
    se         = x$se
  )
  print(df, row.names = FALSE)
  if (!isTRUE(x$quiet)) {
    cli::cli_alert_info(
      "Fixed-treatment-time event study. For staggered adoption use {.pkg fixest} sunab() or the {.pkg did} package (Callaway-Sant'Anna)."
    )
  }
  invisible(x)
}

#' @export
print.mb_cea <- function(x, ...) {
  cost_str   <- .format_gbp(x$total_cost)
  effect_str <- .format_n(x$total_effect)
  unit_str   <- .format_gbp(x$cost_per_unit)
  title <- if (!is.null(x$label)) paste0("Cost-effectiveness: ", x$label) else "Cost-effectiveness"
  cli::cli_h1(title)
  cli::cli_text("Total cost: {.val {cost_str}}")
  cli::cli_text("Total effect: {.val {effect_str}}")
  cli::cli_text("Cost per unit: {.val {unit_str}}")
  invisible(x)
}

#' @export
print.mb_icer <- function(x, ...) {
  dc_str <- .format_gbp(x$delta_cost)
  de_str <- .format_n(x$delta_effect)
  cli::cli_h1("ICER: {x$label_b} vs {x$label_a}")
  cli::cli_text("Delta cost: {.val {dc_str}}")
  cli::cli_text("Delta effect: {.val {de_str}}")
  if (is.finite(x$icer)) {
    icer_str <- .format_gbp(x$icer)
    cli::cli_text("ICER: {.val {icer_str}} per unit")
  } else {
    cli::cli_text("ICER: {.val {x$icer}}")
  }
  cli::cli_text("Dominance: {.val {x$dominance}}")
  invisible(x)
}

#' @export
print.mb_ceac <- function(x, ...) {
  cli::cli_h1("Cost-effectiveness acceptability curve")
  cli::cli_text("Draws: {.val {x$n_draws}}; WTP grid: {.val {length(x$wtp)}} points")
  df <- data.frame(wtp = x$wtp, prob_cost_effective = x$prob_cost_effective)
  print(df, row.names = FALSE)
  invisible(x)
}

#' @export
print.mb_cmo <- function(x, ...) {
  cli::cli_h1("CMO configurations ({nrow(x)} item{if (nrow(x) != 1L) 's' else ''})")
  print.data.frame(x)
  invisible(x)
}

#' @export
print.mb_contribution_claim <- function(x, ...) {
  cli::cli_h1("Contribution claim ({x$strength})")
  cli::cli_text("{.strong Claim}: {x$claim}")
  cli::cli_h2("Evidence for ({.val {length(x$evidence_for)}})")
  for (e in x$evidence_for) cli::cli_bullets(c("v" = e))
  if (length(x$evidence_against)) {
    cli::cli_h2("Evidence against ({.val {length(x$evidence_against)}})")
    for (e in x$evidence_against) cli::cli_bullets(c("x" = e))
  }
  invisible(x)
}

#' @export
print.mb_balance_table <- function(x, ...) {
  thr <- attr(x, "threshold") %||% 0.10
  n_imb <- sum(x$imbalanced, na.rm = TRUE)
  cli::cli_h1("Pre-treatment balance ({nrow(x)} covariate{if (nrow(x) != 1L) 's' else ''})")
  cli::cli_text("|SMD| threshold: {.val {thr}}")
  cli::cli_text("Imbalanced rows: {.val {n_imb}} of {.val {nrow(x)}}")
  display <- x
  display$mean_treated <- round(display$mean_treated, 3)
  display$mean_control <- round(display$mean_control, 3)
  display$sd_treated   <- round(display$sd_treated, 3)
  display$sd_control   <- round(display$sd_control, 3)
  display$smd          <- round(display$smd, 3)
  display$p_value      <- round(display$p_value, 4)
  print(as.data.frame(display), row.names = FALSE)
  invisible(x)
}

#' @export
print.mb_report <- function(x, ...) {
  cli::cli_h1("Magenta Book evaluation report{if (!is.null(x$name)) paste0(': ', x$name) else ''}")
  cli::cli_text("Theory of change: {if (!is.null(x$toc)) 'present' else 'not set'}")
  cli::cli_text("Plan: {if (!is.null(x$plan)) 'present' else 'not set'}")
  cli::cli_text("SMS ratings: {.val {length(x$sms %||% list())}}")
  cli::cli_text("Confidence ratings: {.val {length(x$confidence %||% list())}}")
  cli::cli_text("Cost-effectiveness items: {.val {length(x$cea %||% list())}}")
  cli::cli_text("Vintage: magentabook {.val {as.character(x$vintage)}}")
  invisible(x)
}

---
title: "Designing a Magenta Book evaluation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Designing a Magenta Book evaluation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(magentabook)
```

This vignette walks through the four canonical Magenta Book stages for a worked
example: a hypothetical GBP 50m skills programme aimed at increasing employment
among long-term unemployed claimants. We move from theory of change to evaluation
plan to power calculation to confidence rating, all in one R session.

## Stage 1: theory of change

The theory of change links inputs through to long-run impact. `mb_theory_of_change()`
captures the five canonical Magenta Book levels plus assumptions and external
factors.

```{r}
toc <- mb_theory_of_change(
  inputs     = c("GBP 50m grant", "12 FTE programme team",
                 "Partnership with Jobcentre Plus"),
  activities = c("Design training curriculum",
                 "Deliver workshops in 50 sites",
                 "Provide ongoing mentoring"),
  outputs    = c("500 workshops delivered",
                 "8000 attendees",
                 "5000 completed mentoring blocks"),
  outcomes   = c("Improved employability skills",
                 "Increased job-search confidence",
                 "Higher application rates"),
  impact     = "Higher 12-month employment among long-term unemployed",
  assumptions = c(
    "Workshops cause skills uplift (not just selection of motivated attendees)",
    "Skills uplift translates into application behaviour",
    "Local labour markets absorb the additional applicants"
  ),
  external_factors = c(
    "Macro labour market remains broadly stable",
    "No competing employability programme launches in same areas"
  ),
  name = "Skills uplift programme"
)
toc
```

Pivoting to a logframe with indicators, means of verification, and risks:

```{r}
mb_logframe(
  toc,
  indicators = list(
    outputs  = c("Workshops delivered", "Attendees per workshop"),
    outcomes = c("Skills score (post)", "Application count"),
    impact   = "Employment rate at 12 months"
  ),
  mov = list(
    outputs  = "Programme delivery log",
    outcomes = c("Pre/post survey", "DWP admin data"),
    impact   = "Linked HMRC PAYE records"
  ),
  risks = list(
    outputs  = "Attendance below planned levels",
    outcomes = "Self-report bias in skills score",
    impact   = "Macro shock confounds the estimate"
  )
)
```

The high-criticality assumptions belong in a separate register:

```{r}
mb_assumptions(
  level = c("activities", "outcomes", "impact"),
  description = c(
    "Workshops are well-attended",
    "Skills uplift translates into job entry",
    "Employment rise persists at 12 months"
  ),
  evidence = c(
    "Pilot attendance was 80%",
    "Indirect: similar programmes show 0.3 SD effect",
    "Limited evidence on longer-run persistence"
  ),
  criticality = c("medium", "high", "high")
)
```

## Stage 2: evaluation plan

Tag the evaluation questions by Magenta Book type:

```{r}
qs <- mb_questions(
  text = c(
    "Did the programme cause higher 12-month employment",
    "How large is the effect, and for whom",
    "Was delivery faithful to the design",
    "What was the cost per additional job"
  ),
  type     = c("impact", "impact", "process", "economic"),
  priority = c("primary", "secondary", "secondary", "primary")
)
qs
```

Pin down the counterfactual:

```{r}
cf <- mb_counterfactual(
  definition  = "Eligible non-applicants matched on age, prior unemployment duration, and region",
  source      = "quasi-experimental",
  credibility = "Moderate; selection on observables only, but rich admin covariates available"
)
cf
```

Map stakeholders for governance:

```{r}
mb_stakeholders(
  name = c("HM Treasury", "DWP", "Local authorities", "What Works Centre"),
  role = c("Funder", "Policy lead", "Delivery", "Synthesis"),
  raci = c("A", "R", "C", "I"),
  interest  = c(5, 5, 4, 3),
  influence = c(5, 5, 3, 2)
)
```

Bundle into a plan:

```{r}
plan <- mb_evaluation_plan(
  scope = "GBP 50m programme, 50 sites, 2026-2029",
  questions = qs,
  methods = c(
    impact   = "Difference-in-differences with matched comparison group",
    process  = "Mixed-methods implementation review",
    economic = "Cost per job, with QALY-adjusted variant"
  ),
  timing = c(baseline = "2026-Q1", midline = "2027-Q4", endline = "2029-Q2"),
  governance = "Joint HMT / DWP steering group; peer review by What Works Centre",
  budget = 1.5e6
)
plan
```

## Stage 3: power and sample size

The Magenta Book stresses that an evaluation is only worth running if it can
detect effects of policy-relevant size. We size the study assuming a target
detectable effect of 5 percentage points on the employment rate, baseline
employment of 30 percent, and 80 percent power.

Naive (individual-level) sample size:

```{r}
mb_sample_size(
  type = "proportion", p1 = 0.30, p2 = 0.35,
  power = 0.8, alpha = 0.05
)
```

But the programme is delivered in clusters (sites), so we need to inflate by
the design effect. Jobcentre-level outcomes have an ICC around 0.04 (per the
bundled DWP reference values):

```{r}
mb_icc_reference("employment")
mb_cluster_design(individuals_per_cluster = 50, icc = 0.04, n_clusters = 25)
```

The design effect is a meaningful uplift; we would need roughly that multiple
of the naive N per arm. Alternatively, a stepped-wedge design could trade a
larger total N for staggered rollout that fits programme delivery:

```{r}
mb_stepped_wedge(
  steps = 5, clusters_per_step = 5,
  individuals_per_cluster = 50, icc = 0.04
)
```

What is the smallest effect we can detect with the planned design?

```{r}
mb_mde(
  n_per_group = 600, type = "proportion",
  baseline = 0.30, power = 0.8
)
```

## Stage 4: rate the evidence

Once the evaluation has run, score it on the Maryland SMS:

```{r}
sms <- mb_sms_rate(
  level  = 4,
  study  = "Smith et al. (2029) Skills uplift evaluation",
  design = "Difference-in-differences with matched comparison",
  notes  = "Parallel trends supported by 4 pre-period observations; cluster-robust SEs"
)
sms
```

Record a structured confidence rating:

```{r}
conf_main <- mb_confidence(
  rating                 = "medium",
  question               = "Did the programme raise 12-month employment",
  evidence_strength      = "One Level 4 DiD (n = 12000); supportive Level 3 cohort study",
  methodological_quality = "Adequate; parallel trends plausible; some attrition concerns",
  generalisability       = "Established across 50 sites in two regions",
  rationale              = "Effect direction consistent across two studies but limited replication outside the programme footprint"
)
conf_main

conf_process <- mb_confidence(
  rating                 = "high",
  question               = "Was the programme implemented faithfully",
  evidence_strength      = "Mixed-methods process evaluation; 50-site fidelity audit",
  methodological_quality = "Strong; documented fidelity protocol with inter-rater reliability",
  generalisability       = "All sites covered",
  rationale              = "Comprehensive coverage; consistent fidelity scores"
)

mb_confidence_summary(conf_main, conf_process)
```

## Bringing it together

A single `mb_report` object aggregates everything:

```{r}
report <- mb_evaluation_report(
  plan       = plan,
  toc        = toc,
  sms        = sms,
  confidence = list(conf_main, conf_process),
  name       = "Skills uplift evaluation"
)
report
```

Export to LaTeX for a one-pager:

```{r}
cat(mb_to_latex(report, caption = "Skills uplift evaluation summary"))
```

Word and Excel exports are available via `mb_to_word()` and `mb_to_excel()`
(both require optional packages: `officer` + `flextable`, and `openxlsx`
respectively).

## Reproducibility

Every result object stamps the package vintage. Bundled rubric and reference
tables expose their source via `mb_data_versions()`:

```{r}
mb_data_versions()
```

---
title: "Cost-effectiveness with magentabook and greenbook"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Cost-effectiveness with magentabook and greenbook}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(magentabook)
```

The Green Book covers *appraisal* (decide whether to fund a policy). The Magenta Book covers *evaluation* (learn whether the funded policy worked). The two packages compose cleanly: `greenbook` discounts and rebases cashflows; `magentabook` turns the resulting present values into cost-effectiveness ratios and net benefits.

This vignette compares two delivery options for a hypothetical GBP 1.5m health intervention: an enhanced model that costs more upfront but is expected to deliver more QALYs.

The vignette uses manual discount-factor arithmetic so it builds cleanly on every R installation. The `greenbook` integration is shown as `eval = FALSE` code blocks at the end; install `greenbook` and run those interactively to see the equivalent calls.

## Step 1: build the cashflows

```{r}
# 5-year horizon, real GBP, 2026 prices
years <- 0:4

# Status quo: GBP 1m capex in year 0, then GBP 100k opex per year
cost_a <- c(1e6, 1e5, 1e5, 1e5, 1e5)

# Enhanced: GBP 1.5m capex, GBP 150k opex
cost_b <- c(1.5e6, 1.5e5, 1.5e5, 1.5e5, 1.5e5)

# QALY profile: enhanced delivers more in steady-state
qaly_a <- c(0, 80, 80, 80, 80)
qaly_b <- c(0, 100, 110, 115, 115)
```

## Step 2: discount

Health-schedule kinked STPR uses 1.5 percent for the first 30 years. For this 5-year horizon a constant 1.5 percent annual discount factor is exact:

```{r}
r  <- 0.015
df <- 1 / (1 + r)^years

pv_cost_a <- sum(cost_a * df)
pv_cost_b <- sum(cost_b * df)
pv_qaly_a <- sum(qaly_a * df)
pv_qaly_b <- sum(qaly_b * df)

c(pv_cost_a = pv_cost_a, pv_cost_b = pv_cost_b)
c(pv_qaly_a = pv_qaly_a, pv_qaly_b = pv_qaly_b)
```

## Step 3: cost-effectiveness with magentabook

Plain cost per QALY for each option:

```{r}
cea_a <- mb_cea(cost = pv_cost_a, effect = pv_qaly_a, label = "Status quo")
cea_b <- mb_cea(cost = pv_cost_b, effect = pv_qaly_b, label = "Enhanced")
cea_a
cea_b
```

Incremental analysis (B vs A):

```{r}
icer <- mb_icer(
  cost_a   = pv_cost_a, effect_a = pv_qaly_a,
  cost_b   = pv_cost_b, effect_b = pv_qaly_b,
  label_a  = "Status quo", label_b  = "Enhanced"
)
icer
```

The dominance flag tells us which quadrant of the cost-effectiveness plane the enhanced option sits in.

## Step 4: net benefit at standard willingness-to-pay thresholds

NICE's reference WTP for a QALY is GBP 20k-30k. The cross-government Magenta Book equivalent is GBP 70k per QALY. Compute incremental net benefit at each:

```{r}
sapply(
  c(NICE_low = 20000, NICE_high = 30000, MB_central = 70000),
  function(wtp) mb_inb(icer$delta_cost, icer$delta_effect, wtp)
)
```

Positive INB means the option is cost-effective at that WTP.

## Step 5: probabilistic sensitivity

Real evaluations carry uncertainty in both costs and effects. Suppose a probabilistic sensitivity analysis (e.g. Monte-Carlo over an underlying trial's posterior) gives sampled draws of the incremental cost and incremental effect:

```{r}
set.seed(20260427)
n_draws <- 5000
delta_cost   <- rnorm(n_draws, mean = icer$delta_cost,   sd = 1e5)
delta_effect <- rnorm(n_draws, mean = icer$delta_effect, sd = 30)

ceac <- mb_ceac(
  delta_cost, delta_effect,
  wtp_grid = seq(0, 100000, by = 5000)
)
ceac
```

Each row is the probability that the enhanced option is cost-effective at the corresponding WTP. The CEAC is the standard cost-effectiveness uncertainty visualisation.

## Step 6: report

```{r}
sms_b <- mb_sms_rate(
  level = 5, study = "Pilot RCT of the enhanced option",
  design = "Cluster RCT, 30 GP practices",
  notes = "Power calculation per mb_sample_size()"
)

conf <- mb_confidence(
  rating                 = "high",
  question               = "Does the enhanced option deliver more QALYs",
  evidence_strength      = "Single Level 5 cluster RCT plus modelled extrapolation",
  methodological_quality = "Strong: randomisation worked, follow-up rate > 90%",
  generalisability       = "Tested in a representative sample of UK GP practices",
  rationale              = "RCT plus consistent observational evidence"
)

report <- mb_evaluation_report(
  toc = mb_theory_of_change(
    inputs = "GBP 1.5m capex + GBP 150k opex p.a.",
    activities = "Enhanced clinical pathway",
    outputs = "More patients treated to standard",
    outcomes = "Higher QALYs gained per patient",
    impact = "Improved population health"
  ),
  sms = sms_b,
  confidence = conf,
  cea = list(cea_a, cea_b, icer),
  name = "Enhanced clinical pathway evaluation"
)
report
```

## Composing with greenbook

The example above used direct discount-factor arithmetic so the vignette builds without optional dependencies. In production, install `greenbook` from CRAN and use its primitives for the appraisal-stage discounting. The pattern looks like:

```{r, eval = FALSE}
# Same cashflows, discounted via greenbook's kinked STPR

pv_cost_a <- abs(greenbook::gb_npv(-cost_a, schedule = "health"))
pv_cost_b <- abs(greenbook::gb_npv(-cost_b, schedule = "health"))
pv_qaly_a <- greenbook::gb_npv(qaly_a, schedule = "health")
pv_qaly_b <- greenbook::gb_npv(qaly_b, schedule = "health")

# Identical magentabook calls from here on
cea_a <- mb_cea(pv_cost_a, pv_qaly_a, label = "Status quo")
cea_b <- mb_cea(pv_cost_b, pv_qaly_b, label = "Enhanced")
icer  <- mb_icer(pv_cost_a, pv_qaly_a, pv_cost_b, pv_qaly_b)

# Long-horizon appraisals see the kink: STPR steps from 3.5 percent
# (or 1.5 percent on health) down through 1.0 percent at year 300.
# Manual discount factors don't capture that; greenbook does.
```

Why compose the two packages: by the time a Magenta Book evaluation is asked to compute a cost-effectiveness ratio, the cashflows are usually nominal and unaligned with the appraisal-stage price base. With `greenbook` loaded, the appraisal-stage discount factors and the evaluation-stage cost-effectiveness primitives draw from the same vintage-tagged parameter tables, and the entire chain is testable R code.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/realist.R
\name{mb_cmo}
\alias{mb_cmo}
\title{Context-mechanism-outcome (CMO) configuration}
\usage{
mb_cmo(context, mechanism, outcome, evidence = NA_character_)
}
\arguments{
\item{context}{Character vector. The contextual conditions
needed for the mechanism to fire.}

\item{mechanism}{Character vector. The underlying generative
mechanism (typically a change in reasoning or resources).}

\item{outcome}{Character vector. The observed outcome pattern.}

\item{evidence}{Character vector. Citation, quote, or other
evidence supporting the configuration. Default \code{NA}.}
}
\value{
An \code{mb_cmo} data frame with columns \code{context},
\code{mechanism}, \code{outcome}, \code{evidence}.
}
\description{
Records one or more CMO configurations from a realist evaluation:
the \emph{contexts} in which a \emph{mechanism} fires to produce an
\emph{outcome}, with optional supporting evidence.
}
\details{
Realist evaluation, developed by Pawson and Tilley (1997), seeks
to answer "what works for whom in what circumstances and why" by
surfacing CMO configurations rather than estimating average
treatment effects. The Magenta Book lists realist evaluation as
the principal theory-based approach for context-dependent
interventions.
}
\examples{
mb_cmo(
  context   = c("High trust GP-patient relationships",
                "Low trust GP-patient relationships"),
  mechanism = c("Patients accept advice", "Patients ignore advice"),
  outcome   = c("Improved adherence", "No change in adherence"),
  evidence  = c("Smith et al. 2024 cohort study", "Smith et al. 2024")
)
}
\references{
Pawson, R., Tilley, N. (1997). Realistic Evaluation. SAGE.

HM Treasury (2020). The Magenta Book, chapter on theory-based
evaluation.
\url{https://www.gov.uk/government/publications/the-magenta-book}.
}
\seealso{
\code{\link[=mb_contribution_claim]{mb_contribution_claim()}}.

Other realist: 
\code{\link{mb_contribution_claim}()}
}
\concept{realist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporting.R
\name{mb_to_excel}
\alias{mb_to_excel}
\title{Export an evaluation report to Excel}
\usage{
mb_to_excel(report, file)
}
\arguments{
\item{report}{An \code{mb_report} object.}

\item{file}{Output file path (must end in \code{.xlsx}).}
}
\value{
Invisibly, the file path.
}
\description{
Writes a multi-sheet workbook with one sheet per component:
summary, theory of change, plan, SMS ratings, confidence
ratings, cost-effectiveness, provenance.
}
\details{
Requires the \pkg{openxlsx} package (in Suggests).
}
\examples{
\donttest{
if (requireNamespace("openxlsx", quietly = TRUE)) {
  toc <- mb_theory_of_change(
    inputs = "Funding", activities = "Workshops",
    outputs = "Attendees", outcomes = "Skills",
    impact = "Employment"
  )
  rep <- mb_evaluation_report(toc = toc, name = "Skills uplift")
  tmp <- tempfile(fileext = ".xlsx")
  mb_to_excel(rep, tmp)
}
}
}
\seealso{
\code{\link[=mb_to_word]{mb_to_word()}}, \code{\link[=mb_to_latex]{mb_to_latex()}}.

Other reporting: 
\code{\link{mb_evaluation_report}()},
\code{\link{mb_to_latex}()},
\code{\link{mb_to_word}()}
}
\concept{reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/planning.R
\name{mb_evaluation_plan}
\alias{mb_evaluation_plan}
\title{Aggregate evaluation plan}
\usage{
mb_evaluation_plan(
  scope,
  questions,
  methods,
  timing,
  governance,
  budget = NULL
)
}
\arguments{
\item{scope}{Character scalar describing what the evaluation
does and does not cover.}

\item{questions}{An \code{mb_questions} object.}

\item{methods}{Character vector of methods chosen for each type
of question (e.g. \code{c(impact = "RCT", process = "Realist interviews")}). Names are used in the print method.}

\item{timing}{Character vector or list describing the evaluation
timeline (baseline, midline, endline, follow-up).}

\item{governance}{Character vector or list describing oversight:
steering group composition, peer review, data access.}

\item{budget}{Optional numeric scalar (GBP) for total evaluation
cost.}
}
\value{
An \code{mb_plan} object.
}
\description{
Composes the evaluation scope, questions, methods, timing,
governance, and (optionally) budget into a single object suitable
for review and export.
}
\examples{
qs <- mb_questions(
  text = c("Did employment rise", "Was the policy implemented faithfully"),
  type = c("impact", "process")
)
mb_evaluation_plan(
  scope      = "GBP 50m skills programme, 2026-2029",
  questions  = qs,
  methods    = c(impact = "RCT", process = "Mixed methods"),
  timing     = c(baseline = "2026-Q1", endline = "2029-Q2"),
  governance = "Joint HMT / DfE steering group; peer review by What Works"
)
}
\references{
HM Treasury (2020). The Magenta Book: Central Government
Guidance on Evaluation, chapter on planning and managing an
evaluation.
\url{https://www.gov.uk/government/publications/the-magenta-book}.
}
\seealso{
\code{\link[=mb_questions]{mb_questions()}}, \code{\link[=mb_counterfactual]{mb_counterfactual()}},
\code{\link[=mb_stakeholders]{mb_stakeholders()}}, \code{\link[=mb_evaluation_report]{mb_evaluation_report()}}.

Other planning: 
\code{\link{mb_balance_table}()},
\code{\link{mb_counterfactual}()},
\code{\link{mb_questions}()},
\code{\link{mb_stakeholders}()}
}
\concept{planning}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cea.R
\name{mb_inb}
\alias{mb_inb}
\title{Incremental net benefit}
\usage{
mb_inb(delta_cost, delta_effect, wtp)
}
\arguments{
\item{delta_cost}{Numeric scalar. Incremental cost of B over A.}

\item{delta_effect}{Numeric scalar. Incremental effect of B
over A.}

\item{wtp}{Numeric scalar. Willingness-to-pay per unit of effect
(e.g. the NICE QALY threshold in a health context).}
}
\value{
Numeric scalar. INB in the units of \code{delta_cost}. INB > 0
means B is cost-effective at the supplied WTP.
}
\description{
Computes the incremental net benefit (INB) of B over A at a
single willingness-to-pay threshold. Equivalent to the ICER
framing on a monetary scale.
}
\details{
\deqn{\text{INB} = \lambda \cdot \Delta E - \Delta C}
Equivalent to ICER comparison: INB > 0 iff ICER < WTP (when
effect change is positive).
}
\examples{
mb_inb(delta_cost = 50000, delta_effect = 2, wtp = 30000)
}
\seealso{
\code{\link[=mb_ceac]{mb_ceac()}}, \code{\link[=mb_icer]{mb_icer()}}.

Other cost-effectiveness: 
\code{\link{mb_cea}()},
\code{\link{mb_ceac}()},
\code{\link{mb_daly}()},
\code{\link{mb_icer}()},
\code{\link{mb_qaly}()}
}
\concept{cost-effectiveness}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cea.R
\name{mb_ceac}
\alias{mb_ceac}
\title{Cost-effectiveness acceptability curve}
\usage{
mb_ceac(delta_cost, delta_effect, wtp_grid)
}
\arguments{
\item{delta_cost}{Numeric vector. Sampled incremental costs of B
relative to A.}

\item{delta_effect}{Numeric vector, same length as \code{delta_cost}.
Sampled incremental effects.}

\item{wtp_grid}{Numeric vector of willingness-to-pay values
(cost per unit of effect) at which to evaluate the curve.}
}
\value{
An \code{mb_ceac} object: a data-frame-like list with columns
\code{wtp}, \code{prob_cost_effective}, plus \code{n_draws} and \code{vintage}.
}
\description{
For a single A-vs-B comparison with sampled (\code{delta_cost},
\code{delta_effect}) draws (e.g. from a probabilistic sensitivity
analysis), returns the probability that B is cost-effective at
each willingness-to-pay (WTP) value in \code{wtp_grid}.
}
\details{
At each WTP value \code{lambda}, B is cost-effective if the
incremental net benefit \code{lambda * delta_effect - delta_cost > 0}.
The CEAC is the proportion of draws for which this is true.
}
\examples{
set.seed(4)
delta_cost   <- rnorm(1000, mean = 50000, sd = 10000)
delta_effect <- rnorm(1000, mean = 2,     sd = 0.5)
mb_ceac(delta_cost, delta_effect, wtp_grid = seq(0, 100000, by = 10000))
}
\references{
Fenwick, E., Claxton, K., Sculpher, M. (2001). Representing
uncertainty: the role of cost-effectiveness acceptability
curves. Health Economics 10(8). \url{doi:10.1002/hec.635}.
}
\seealso{
\code{\link[=mb_inb]{mb_inb()}}, \code{\link[=mb_icer]{mb_icer()}}.

Other cost-effectiveness: 
\code{\link{mb_cea}()},
\code{\link{mb_daly}()},
\code{\link{mb_icer}()},
\code{\link{mb_inb}()},
\code{\link{mb_qaly}()}
}
\concept{cost-effectiveness}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cea.R
\name{mb_cea}
\alias{mb_cea}
\title{Cost per unit of outcome}
\usage{
mb_cea(cost, effect, label = NULL)
}
\arguments{
\item{cost}{Numeric scalar or vector. Total cost (or per-period
costs that will be summed).}

\item{effect}{Numeric scalar or vector. Total outcomes delivered
(or per-period outcomes that will be summed).}

\item{label}{Optional character scalar. Name of the option.}
}
\value{
An \code{mb_cea} object.
}
\description{
Computes a simple cost-effectiveness ratio: total cost divided
by total outcomes delivered. Use \code{\link[=mb_icer]{mb_icer()}} for two-option
comparisons.
}
\examples{
mb_cea(cost = 1e6, effect = 250, label = "Workshop programme")
}
\seealso{
\code{\link[=mb_icer]{mb_icer()}}, \code{\link[=mb_ceac]{mb_ceac()}}, \code{\link[=mb_inb]{mb_inb()}}.

Other cost-effectiveness: 
\code{\link{mb_ceac}()},
\code{\link{mb_daly}()},
\code{\link{mb_icer}()},
\code{\link{mb_inb}()},
\code{\link{mb_qaly}()}
}
\concept{cost-effectiveness}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporting.R
\name{mb_to_word}
\alias{mb_to_word}
\title{Export an evaluation report to Word}
\usage{
mb_to_word(report, file)
}
\arguments{
\item{report}{An \code{mb_report} object.}

\item{file}{Output file path (must end in \code{.docx}).}
}
\value{
Invisibly, the file path.
}
\description{
Writes a one- to two-page Word document summarising an
\code{mb_report}: name, theory of change, evaluation plan, SMS
ratings, confidence ratings, and cost-effectiveness.
}
\details{
Requires the \pkg{officer} and \pkg{flextable} packages (both in
Suggests).
}
\examples{
\donttest{
if (requireNamespace("officer", quietly = TRUE) &&
    requireNamespace("flextable", quietly = TRUE)) {
  toc <- mb_theory_of_change(
    inputs = "Funding", activities = "Workshops",
    outputs = "Attendees", outcomes = "Skills",
    impact = "Employment"
  )
  rep <- mb_evaluation_report(toc = toc, name = "Skills uplift")
  tmp <- tempfile(fileext = ".docx")
  mb_to_word(rep, tmp)
}
}
}
\seealso{
\code{\link[=mb_evaluation_report]{mb_evaluation_report()}}, \code{\link[=mb_to_excel]{mb_to_excel()}},
\code{\link[=mb_to_latex]{mb_to_latex()}}.

Other reporting: 
\code{\link{mb_evaluation_report}()},
\code{\link{mb_to_excel}()},
\code{\link{mb_to_latex}()}
}
\concept{reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cea.R
\name{mb_icer}
\alias{mb_icer}
\title{Incremental cost-effectiveness ratio with dominance handling}
\usage{
mb_icer(cost_a, effect_a, cost_b, effect_b, label_a = "A", label_b = "B")
}
\arguments{
\item{cost_a, effect_a}{Numeric scalars. Cost and effect of
option A.}

\item{cost_b, effect_b}{Numeric scalars. Cost and effect of
option B.}

\item{label_a, label_b}{Character scalars. Labels for the two
options.}
}
\value{
An \code{mb_icer} object: a list with \code{delta_cost},
\code{delta_effect}, \code{icer}, \code{dominance} (one of \code{"a_dominates"},
\code{"b_dominates"}, \code{"b_more_costly_more_effective"},
\code{"b_less_costly_less_effective"}), and labels.
}
\description{
Computes the ICER comparing option B to option A, with explicit
handling of the four dominance regions:
\itemize{
\item \strong{A dominates} B (B costs more, delivers less): no ICER.
\item \strong{B dominates} A (B costs less, delivers more): no ICER; B is
the obvious choice.
\item \strong{B more costly, more effective}: standard ICER positive.
\item \strong{B less costly, less effective}: ICER negative — B saves
money at the expense of effect.
}
}
\details{
The ICER is the cost per additional unit of outcome from
switching from A to B:
\deqn{\text{ICER} = (C_B - C_A) / (E_B - E_A)}

If \code{delta_effect} is zero, the ICER is reported as \code{Inf}
(when costs differ) or \code{NaN} (when costs are equal).
}
\examples{
mb_icer(cost_a = 1e6, effect_a = 200, cost_b = 1.5e6, effect_b = 300,
        label_a = "Status quo", label_b = "Enhanced")
}
\references{
HM Treasury (2020). The Magenta Book, Annex A on cost-effectiveness.

Drummond, M. F., Sculpher, M. J., Claxton, K., Stoddart, G. L.,
Torrance, G. W. (2015). Methods for the Economic Evaluation of
Health Care Programmes (4th ed.). Oxford University Press.
}
\seealso{
\code{\link[=mb_cea]{mb_cea()}}, \code{\link[=mb_ceac]{mb_ceac()}}, \code{\link[=mb_inb]{mb_inb()}}.

Other cost-effectiveness: 
\code{\link{mb_cea}()},
\code{\link{mb_ceac}()},
\code{\link{mb_daly}()},
\code{\link{mb_inb}()},
\code{\link{mb_qaly}()}
}
\concept{cost-effectiveness}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/its.R
\name{mb_its}
\alias{mb_its}
\title{Interrupted time series via segmented regression}
\usage{
mb_its(y, time, intervention_time, lag = 0L, quiet = FALSE)
}
\arguments{
\item{y}{Numeric vector of outcomes ordered by \code{time}.}

\item{time}{Numeric vector of time indices, same length as \code{y}.}

\item{intervention_time}{Numeric scalar. The first time point
considered post-intervention.}

\item{lag}{Integer >= 0. Number of pre-intervention observations
to drop near the intervention (transition period). Default \code{0}.}

\item{quiet}{Logical. If \code{FALSE} (default), the print method
appends a one-line reminder that this is a single-group
segmented regression and points to specialist tooling for
autocorrelation correction. Set to \code{TRUE} to suppress.}
}
\value{
An \code{mb_its} object: a list with \code{coefficients} (named
numeric), \code{se} (named numeric), \code{level_change}, \code{slope_change},
\code{intervention_time}, \code{n}, \code{n_pre}, \code{n_post}, and \code{vintage}.
}
\description{
Fits a single-group interrupted time series model:
\deqn{Y_t = \beta_0 + \beta_1 t + \beta_2 P_t + \beta_3 (t - t^*) P_t + \epsilon_t}
where \code{P_t} is 1 for \verb{t >= t*} and \verb{t*} is the intervention time.
\code{beta_2} is the immediate level change at the intervention;
\code{beta_3} is the change in slope.
}
\details{
Segmented regression assumes residuals are independent. For
autocorrelated series, fit a Newey-West, Prais-Winsten, or
ARIMA-error specification using \pkg{sandwich}, \pkg{nlme}, or
\pkg{forecast}. This function is the canonical baseline.
}
\examples{
set.seed(2)
time <- 1:48
y    <- 10 + 0.05 * time + ifelse(time >= 25, 2 + 0.1 * (time - 25), 0) + rnorm(48, sd = 0.5)
mb_its(y, time, intervention_time = 25)
}
\references{
Bernal, J. L., Cummins, S., Gasparrini, A. (2017). Interrupted
time series regression for the evaluation of public health
interventions: a tutorial. International Journal of
Epidemiology 46(1). \url{doi:10.1093/ije/dyw098}.

Wagner, A. K., Soumerai, S. B., Zhang, F., Ross-Degnan, D.
(2002). Segmented regression analysis of interrupted time series
studies in medication use research. Journal of Clinical Pharmacy
and Therapeutics 27. \url{doi:10.1046/j.1365-2710.2002.00430.x}.
}
\seealso{
\code{\link[=mb_did_2x2]{mb_did_2x2()}}, \code{\link[=mb_event_study]{mb_event_study()}}.

Other estimators: 
\code{\link{mb_did_2x2}()},
\code{\link{mb_event_study}()}
}
\concept{estimators}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporting.R
\name{mb_evaluation_report}
\alias{mb_evaluation_report}
\title{Aggregate evaluation report}
\usage{
mb_evaluation_report(
  plan = NULL,
  toc = NULL,
  sms = NULL,
  confidence = NULL,
  cea = NULL,
  name = NULL
)
}
\arguments{
\item{plan}{Optional \code{mb_plan} from \code{\link[=mb_evaluation_plan]{mb_evaluation_plan()}}.}

\item{toc}{Optional \code{mb_toc} from \code{\link[=mb_theory_of_change]{mb_theory_of_change()}}.}

\item{sms}{Optional \code{mb_sms_rating} or list of them.}

\item{confidence}{Optional \code{mb_confidence},
\code{mb_confidence_summary}, or list of \code{mb_confidence}.}

\item{cea}{Optional \code{mb_cea}, \code{mb_icer}, or list of them.}

\item{name}{Optional character scalar naming the evaluation.}
}
\value{
An \code{mb_report} object.
}
\description{
Composes the components produced by other \code{magentabook}
functions into a single report object: theory of change,
evaluation plan, SMS ratings, confidence ratings,
cost-effectiveness analyses. Any component may be omitted.
}
\examples{
toc <- mb_theory_of_change(
  inputs = "Funding", activities = "Workshops",
  outputs = "Attendees", outcomes = "Skills",
  impact = "Employment"
)
mb_evaluation_report(toc = toc, name = "Skills uplift evaluation")
}
\seealso{
\code{\link[=mb_to_word]{mb_to_word()}}, \code{\link[=mb_to_excel]{mb_to_excel()}}, \code{\link[=mb_to_latex]{mb_to_latex()}}.

Other reporting: 
\code{\link{mb_to_excel}()},
\code{\link{mb_to_latex}()},
\code{\link{mb_to_word}()}
}
\concept{reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cea.R
\name{mb_daly}
\alias{mb_daly}
\title{Disability-adjusted life years (DALYs) accumulator}
\usage{
mb_daly(yld, yll, persons = 1)
}
\arguments{
\item{yld}{Numeric scalar or vector. Years lived with disability
per person.}

\item{yll}{Numeric scalar or vector. Years of life lost per
person (e.g. life expectancy minus age at death).}

\item{persons}{Numeric scalar. Number of persons. Default \code{1}.}
}
\value{
Numeric scalar. Total DALYs (YLD + YLL summed across
persons).
}
\description{
Sums years lived with disability (YLD) and years of life lost
(YLL) across persons. DALY is the global-health analogue of
QALY: lower is better.
}
\details{
\deqn{\text{DALY} = \text{persons} \cdot \sum (YLD + YLL)}

This implementation follows the Global Burden of Disease
definition. Age-weighting and discounting are not applied by
default (the IHME GBD removed both in the 2010 update); add a
discount factor manually if your guidance still requires it.
}
\examples{
mb_daly(yld = 2.5, yll = 8.0, persons = 100)
}
\references{
Murray, C. J. L., Lopez, A. D. (1996). The Global Burden of
Disease. Harvard University Press.

GBD 2019 Diseases and Injuries Collaborators (2020). Global
burden of 369 diseases and injuries in 204 countries and
territories, 1990-2019. The Lancet 396.
\url{doi:10.1016/S0140-6736(20)30925-9}.
}
\seealso{
\code{\link[=mb_qaly]{mb_qaly()}}.

Other cost-effectiveness: 
\code{\link{mb_cea}()},
\code{\link{mb_ceac}()},
\code{\link{mb_icer}()},
\code{\link{mb_inb}()},
\code{\link{mb_qaly}()}
}
\concept{cost-effectiveness}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/event_study.R
\name{mb_event_study}
\alias{mb_event_study}
\title{Simple event-study coefficients}
\usage{
mb_event_study(
  y,
  unit,
  time,
  treatment_time,
  treated,
  leads = 3L,
  lags = 3L,
  cluster = NULL,
  quiet = FALSE
)
}
\arguments{
\item{y}{Numeric vector of outcomes.}

\item{unit}{Vector identifying units (panel ID).}

\item{time}{Numeric vector of time indices.}

\item{treatment_time}{Numeric scalar. The first treated period.
Units with \code{treated = 0} (never-treated) are pure controls.}

\item{treated}{Logical or 0/1 numeric vector indicating
whether each observation belongs to a treated unit. The design
requires at least some never-treated control units; without
them the event-time dummies are collinear with the time fixed
effects.}

\item{leads}{Integer >= 0. Number of pre-treatment periods to
include. Default \code{3}.}

\item{lags}{Integer >= 0. Number of post-treatment periods.
Default \code{3}.}

\item{cluster}{Optional vector identifying clusters for
cluster-robust standard errors (CR1 with finite-sample
correction \code{(G/(G-1)) * (N-1)/(N-K)}). Common choice: pass
\code{unit} to cluster at the unit level. If \code{NULL} (default),
conventional OLS SEs are returned.}

\item{quiet}{Logical. If \code{FALSE} (default), the print method
appends a one-line reminder that this is a fixed-treatment-
time event study and points to \pkg{fixest} (\code{sunab()}) or
\pkg{did} for staggered adoption. Set to \code{TRUE} to suppress.}
}
\value{
An \code{mb_event_study} object: a list with \code{event_time},
\code{estimate}, \code{se}, plus \code{n}, \code{n_units}, \code{n_periods},
\code{treatment_time}, and \code{vintage}.
}
\description{
Estimates a panel event-study with unit and time fixed effects
and event-time dummies. Treatment time is fixed across treated
units (no staggered adoption). Returns coefficients for \code{leads}
periods before and \code{lags} periods after treatment, with the
period immediately before treatment (\code{event_time = -1}) omitted
as the reference category.
}
\details{
Implements the canonical two-way fixed-effects event study:
\deqn{Y_{it} = \alpha_i + \gamma_t + \sum_{k \neq -1} \beta_k \mathbf{1}\{t - t^* = k, D_i = 1\} + \epsilon_{it}}

For staggered adoption (units treated at different times), this
specification is biased under treatment-effect heterogeneity. Use
the heterogeneity-robust estimators of Callaway & Sant'Anna
(2021) or de Chaisemartin & D'Haultfoeuille (2020), available in
the \pkg{did}, \pkg{didimputation}, or \pkg{fixest} packages
(\code{fixest::feols} with \code{sunab()}).

Standard errors are conventional OLS; for clustered inference
use \pkg{sandwich} or \pkg{fixest}.
}
\examples{
set.seed(3)
n_units <- 50; n_periods <- 10; treat_time <- 6
panel <- expand.grid(unit = 1:n_units, time = 1:n_periods)
panel$treated <- as.integer(panel$unit <= 25)
panel$post    <- as.integer(panel$time >= treat_time)
panel$y <- 0.1 * panel$time + 0.5 * (panel$treated * panel$post) +
           rnorm(nrow(panel))
mb_event_study(
  y = panel$y, unit = panel$unit, time = panel$time,
  treatment_time = treat_time, treated = panel$treated,
  leads = 3, lags = 3
)
}
\references{
Callaway, B., Sant'Anna, P. H. C. (2021). Difference-in-Differences
with Multiple Time Periods. Journal of Econometrics 225(2).
\url{doi:10.1016/j.jeconom.2020.12.001}.

de Chaisemartin, C., D'Haultfoeuille, X. (2020). Two-Way Fixed
Effects Estimators with Heterogeneous Treatment Effects. American
Economic Review 110(9). \url{doi:10.1257/aer.20181169}.
}
\seealso{
\code{\link[=mb_did_2x2]{mb_did_2x2()}}, \code{\link[=mb_its]{mb_its()}}.

Other estimators: 
\code{\link{mb_did_2x2}()},
\code{\link{mb_its}()}
}
\concept{estimators}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/planning.R
\name{mb_questions}
\alias{mb_questions}
\title{Tag and structure evaluation questions}
\usage{
mb_questions(text, type = "impact", priority = "primary")
}
\arguments{
\item{text}{Character vector of evaluation questions.}

\item{type}{Character vector. One of \code{"process"}, \code{"impact"},
\code{"economic"}, \code{"vfm"}. Length 1 or \code{length(text)}.}

\item{priority}{Character vector. \code{"primary"} or \code{"secondary"}.
Length 1 or \code{length(text)}.}
}
\value{
An \code{mb_questions} data frame with columns \code{text},
\code{type}, \code{priority}.
}
\description{
Stores a set of evaluation questions tagged by Magenta Book type
(process, impact, economic, value-for-money) and by priority
(primary or secondary). The Magenta Book canonical taxonomy is
bundled in \code{\link[=mb_schedule_table]{mb_schedule_table()}} under \code{"questions"}.
}
\examples{
mb_questions(
  text     = c("Did the policy cause employment to rise",
               "Was implementation faithful to the design"),
  type     = c("impact", "process"),
  priority = c("primary", "secondary")
)
}
\references{
HM Treasury (2020). The Magenta Book: Central Government
Guidance on Evaluation, chapters on process, impact, and
economic evaluation.
\url{https://www.gov.uk/government/publications/the-magenta-book}.
}
\seealso{
\code{\link[=mb_evaluation_plan]{mb_evaluation_plan()}}, \code{\link[=mb_schedule_table]{mb_schedule_table()}}.

Other planning: 
\code{\link{mb_balance_table}()},
\code{\link{mb_counterfactual}()},
\code{\link{mb_evaluation_plan}()},
\code{\link{mb_stakeholders}()}
}
\concept{planning}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cea.R
\name{mb_qaly}
\alias{mb_qaly}
\title{Quality-adjusted life years (QALYs) accumulator}
\usage{
mb_qaly(utility, persons = 1, years = 1, discount_rate = NULL)
}
\arguments{
\item{utility}{Numeric scalar or vector in \verb{[0, 1]}. Utility
weight per year. Length 1 or \code{years}.}

\item{persons}{Numeric scalar. Number of persons. Default \code{1}.}

\item{years}{Numeric scalar. Number of years. Default \code{1}.}

\item{discount_rate}{Optional numeric in \verb{[0, 1)}. Annual
discount rate. If supplied, returns the discounted QALY total.
Default \code{NULL} (undiscounted).}
}
\value{
Numeric scalar. Total QALYs.
}
\description{
Sums utility-weighted years lived across persons, with optional
annual discounting.
}
\details{
Without discounting:
\deqn{\text{QALY} = \text{persons} \cdot \sum_{t=0}^{T-1} u_t}

With annual discount rate \code{r}:
\deqn{\text{QALY} = \text{persons} \cdot \sum_{t=0}^{T-1} \frac{u_t}{(1+r)^t}}

Compatible with \code{greenbook::gb_qaly}: when \code{utility} is scalar and
\code{discount_rate} is \code{NULL}, this returns \code{persons * utility * years}.
}
\examples{
mb_qaly(utility = 0.8, persons = 100, years = 5)
mb_qaly(utility = 0.8, persons = 100, years = 5, discount_rate = 0.035)
mb_qaly(utility = c(0.5, 0.7, 0.9), persons = 50)
}
\references{
Drummond, M. F. et al. (2015). Methods for the Economic
Evaluation of Health Care Programmes (4th ed.). OUP.

NICE (2022). Guide to the methods of technology appraisal.
}
\seealso{
\code{\link[=mb_daly]{mb_daly()}}, \code{\link[=mb_cea]{mb_cea()}}.

Other cost-effectiveness: 
\code{\link{mb_cea}()},
\code{\link{mb_ceac}()},
\code{\link{mb_daly}()},
\code{\link{mb_icer}()},
\code{\link{mb_inb}()}
}
\concept{cost-effectiveness}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sms.R
\name{mb_sms_explain}
\alias{mb_sms_explain}
\title{Explain the Maryland SMS rubric}
\usage{
mb_sms_explain(level = NULL)
}
\arguments{
\item{level}{Optional integer in \code{1:5}. If supplied, prints the
rubric for that single level. If \code{NULL} (default), prints the
full rubric.}
}
\value{
Invisibly, the rubric data frame (filtered to \code{level} if
supplied). Called for the side-effect of printing.
}
\description{
Prints the bundled Maryland SMS rubric. Use this when scoring
studies, training reviewers, or presenting evidence ratings to
stakeholders.
}
\examples{
mb_sms_explain()
mb_sms_explain(4)
}
\seealso{
\code{\link[=mb_sms_rate]{mb_sms_rate()}}.

Other Maryland SMS: 
\code{\link{mb_sms_rate}()}
}
\concept{Maryland SMS}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confidence.R
\name{mb_confidence}
\alias{mb_confidence}
\title{Structured Magenta Book confidence rating}
\usage{
mb_confidence(
  rating = c("high", "medium", "low"),
  question,
  evidence_strength,
  methodological_quality,
  generalisability,
  rationale
)
}
\arguments{
\item{rating}{Character scalar. One of \code{"high"}, \code{"medium"},
\code{"low"}.}

\item{question}{Character scalar. The evaluation question this
rating refers to.}

\item{evidence_strength}{Character scalar. Plain-English
description of the volume and quality of underlying studies.}

\item{methodological_quality}{Character scalar. Plain-English
description of design rigour and identifying assumptions.}

\item{generalisability}{Character scalar. Plain-English
description of how widely the findings travel across settings.}

\item{rationale}{Character scalar. Free-text justification for
the chosen rating.}
}
\value{
An \code{mb_confidence} object: a list with the supplied
fields plus the bundled-rubric \code{description} for the chosen
rating, and \code{vintage}.
}
\description{
Records a single confidence rating against the bundled rubric:
high / medium / low, with explicit assessments of evidence
strength, methodological quality, and generalisability, and a
free-text rationale.
}
\details{
Magenta Book confidence ratings translate evidence into
decision-grade summaries for ministers and senior officials. The
bundled rubric (see \code{\link[=mb_schedule_table]{mb_schedule_table()}} with table
\code{"confidence"}) is \emph{not a direct quotation from the Magenta
Book}. It is a magentabook synthesis of cross-What-Works-Centre
confidence-rating traditions: Education Endowment Foundation
(5 padlocks), Early Intervention Foundation (Foundation
Standards), College of Policing (1-5 scale), and the Justice
Data Lab (red / amber / green). The three-level high / medium /
low structure is designed for HMG decision-grade reporting and
aligns with the value-for-money framing of the Magenta Book
(2020) supplementary guidance.
}
\examples{
mb_confidence(
  rating                 = "medium",
  question               = "Did the policy raise employment",
  evidence_strength      = "One Level 4 DiD; one Level 3 matched cohort",
  methodological_quality = "Adequate; parallel trends plausible but limited pre-period",
  generalisability       = "Findings established in a single region",
  rationale              = "Effect direction consistent across two studies but limited replication"
)
}
\references{
HM Treasury (2020). The Magenta Book: Central Government
Guidance on Evaluation. Supplementary guidance on value for
money.

Education Endowment Foundation. Padlock evidence ratings.

Early Intervention Foundation (2021). Foundation Standards of
Evidence.
}
\seealso{
\code{\link[=mb_confidence_summary]{mb_confidence_summary()}}, \code{\link[=mb_sms_rate]{mb_sms_rate()}}.

Other confidence: 
\code{\link{mb_confidence_summary}()}
}
\concept{confidence}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/planning.R
\name{mb_counterfactual}
\alias{mb_counterfactual}
\title{Define a counterfactual}
\usage{
mb_counterfactual(
  definition,
  source = c("rct", "quasi-experimental", "theory-based", "comparator", "historical"),
  credibility = NA_character_
)
}
\arguments{
\item{definition}{Character scalar describing the counterfactual:
what would have happened in the absence of the policy.}

\item{source}{Character scalar. Mechanism by which the
counterfactual is constructed. One of \code{"rct"},
\code{"quasi-experimental"}, \code{"theory-based"}, \code{"comparator"},
\code{"historical"}.}

\item{credibility}{Character scalar. Plain-English assessment of
how credible the counterfactual is.}
}
\value{
An \code{mb_counterfactual} object.
}
\description{
Records the comparison condition against which the policy effect
is to be measured. The Magenta Book stresses that no impact
evaluation is possible without an explicit counterfactual.
}
\examples{
mb_counterfactual(
  definition  = "Eligible non-applicants in the same year",
  source      = "quasi-experimental",
  credibility = "Moderate; selection on observables only"
)
}
\references{
HM Treasury (2020). The Magenta Book: Central Government
Guidance on Evaluation, supplementary guidance on
quasi-experimental and theory-based methods.
\url{https://www.gov.uk/government/publications/the-magenta-book}.
}
\seealso{
\code{\link[=mb_evaluation_plan]{mb_evaluation_plan()}}.

Other planning: 
\code{\link{mb_balance_table}()},
\code{\link{mb_evaluation_plan}()},
\code{\link{mb_questions}()},
\code{\link{mb_stakeholders}()}
}
\concept{planning}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power.R
\name{mb_sample_size}
\alias{mb_sample_size}
\title{Required sample size for a target power}
\usage{
mb_sample_size(
  effect_size = NULL,
  sd = 1,
  power = 0.8,
  alpha = 0.05,
  sides = 2L,
  type = c("mean", "proportion"),
  p1 = NULL,
  p2 = NULL
)
}
\arguments{
\item{effect_size}{Numeric. The standardised effect size:
Cohen's \code{d} for \code{type = "mean"}, or Cohen's \code{h} for
\code{type = "proportion"} (computed automatically if \code{p1} and \code{p2}
are supplied).}

\item{sd}{Numeric. Standard deviation, used only for \code{type = "mean"}. Default \code{1}, in which case \code{effect_size} is interpreted
in standard deviation units.}

\item{power}{Numeric in \verb{(0, 1)}. Target power. Default \code{0.8}.}

\item{alpha}{Numeric in \verb{(0, 1)}. Significance level. Default
\code{0.05}.}

\item{sides}{Integer. \code{2} (two-sided, default) or \code{1} (one-sided).}

\item{type}{Character. \code{"mean"} (default) or \code{"proportion"}.}

\item{p1, p2}{Optional numeric in \verb{(0, 1)}. If both supplied (and
\code{type = "proportion"}), the function computes Cohen's \code{h} and
ignores \code{effect_size}.}
}
\value{
Integer scalar. Sample size per arm (rounded up).
}
\description{
Given a target effect size, power, and significance level,
returns the required sample size per arm. Inverts \code{\link[=mb_power]{mb_power()}}.
}
\examples{
mb_sample_size(effect_size = 0.3, power = 0.8)
mb_sample_size(type = "proportion", p1 = 0.40, p2 = 0.50, power = 0.8)
}
\seealso{
\code{\link[=mb_power]{mb_power()}}, \code{\link[=mb_mde]{mb_mde()}}, \code{\link[=mb_cluster_design]{mb_cluster_design()}}.

Other power: 
\code{\link{mb_cluster_design}()},
\code{\link{mb_icc_reference}()},
\code{\link{mb_mde}()},
\code{\link{mb_power}()},
\code{\link{mb_stepped_wedge}()}
}
\concept{power}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/realist.R
\name{mb_contribution_claim}
\alias{mb_contribution_claim}
\title{Contribution-analysis claim}
\usage{
mb_contribution_claim(
  claim,
  evidence_for,
  evidence_against = character(0),
  strength = c("weak", "moderate", "strong")
)
}
\arguments{
\item{claim}{Character scalar. The contribution claim being
tested.}

\item{evidence_for}{Character vector. Evidence supporting the
claim.}

\item{evidence_against}{Character vector. Evidence against the
claim. Default \code{character(0)}.}

\item{strength}{Character scalar. One of \code{"weak"}, \code{"moderate"},
\code{"strong"}. Reflects the analyst's overall judgement after
weighing evidence for and against.}
}
\value{
An \code{mb_contribution_claim} object.
}
\description{
Records a contribution claim with supporting and refuting
evidence and an overall strength rating. Used in
contribution-analysis-style theory-based evaluation, where causal
inference comes from triangulating multiple evidence streams
against a contribution story rather than from a counterfactual.
}
\examples{
mb_contribution_claim(
  claim            = "The training programme contributed to higher employment",
  evidence_for     = c("Pre-post outcomes improved",
                       "Theory of change pathways visible in interviews"),
  evidence_against = "Macro labour market also improved",
  strength         = "moderate"
)
}
\references{
Mayne, J. (2008). Contribution Analysis: An approach to exploring
cause and effect. ILAC Brief No. 16.

HM Treasury (2020). The Magenta Book, chapter on theory-based
evaluation.
}
\seealso{
\code{\link[=mb_cmo]{mb_cmo()}}.

Other realist: 
\code{\link{mb_cmo}()}
}
\concept{realist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theory.R
\name{mb_logframe}
\alias{mb_logframe}
\title{Convert a theory of change into a logframe}
\usage{
mb_logframe(toc, indicators = NULL, mov = NULL, risks = NULL)
}
\arguments{
\item{toc}{An \code{mb_toc} object from \code{\link[=mb_theory_of_change]{mb_theory_of_change()}}.}

\item{indicators}{Optional named list. Names must be one of
\code{"inputs"}, \code{"activities"}, \code{"outputs"}, \code{"outcomes"}, or
\code{"impact"}. Each element is a character vector of indicators.}

\item{mov}{Optional named list, same convention. Means of
verification per level (data source, survey, administrative
record).}

\item{risks}{Optional named list, same convention. Risks per
level.}
}
\value{
An \code{mb_logframe} object: a data frame with columns
\code{level}, \code{description}, and (if supplied) \code{indicator}, \code{mov},
\code{risk}. Multiple items per level are concatenated with \code{"; "}.
}
\description{
Pivots an \code{mb_toc} into the canonical Magenta Book logframe
table: one row per level, with optional indicators, means of
verification, and risks columns.
}
\examples{
toc <- mb_theory_of_change(
  inputs = "Funding", activities = "Workshops",
  outputs = "Attendees", outcomes = "Skills",
  impact = "Employment"
)
mb_logframe(
  toc,
  indicators = list(outputs = "n attendees", outcomes = "skills score"),
  mov        = list(outputs = "attendance log", outcomes = "post-test")
)
}
\seealso{
\code{\link[=mb_theory_of_change]{mb_theory_of_change()}}.

Other theory of change: 
\code{\link{mb_assumptions}()},
\code{\link{mb_theory_of_change}()}
}
\concept{theory of change}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/planning.R
\name{mb_stakeholders}
\alias{mb_stakeholders}
\title{RACI-style stakeholder register}
\usage{
mb_stakeholders(name, role, raci, interest = NA_real_, influence = NA_real_)
}
\arguments{
\item{name}{Character vector of stakeholder names.}

\item{role}{Character vector of stakeholder roles.}

\item{raci}{Character vector. One of \code{"R"}, \code{"A"}, \code{"C"}, \code{"I"}.}

\item{interest}{Optional numeric vector in \verb{[1, 5]}. Higher
means more interest in the evaluation.}

\item{influence}{Optional numeric vector in \verb{[1, 5]}. Higher
means more influence over the evaluation.}
}
\value{
An \code{mb_stakeholders} data frame with columns \code{name},
\code{role}, \code{raci}, \code{interest}, \code{influence}.
}
\description{
Records who is Responsible, Accountable, Consulted, or Informed
for an evaluation, with optional interest and influence ratings
for use in a stakeholder map.
}
\examples{
mb_stakeholders(
  name      = c("HMT", "DfE", "What Works Centre"),
  role      = c("Funder", "Delivery", "Synthesis"),
  raci      = c("A", "R", "C"),
  interest  = c(5, 5, 4),
  influence = c(5, 4, 2)
)
}
\seealso{
\code{\link[=mb_evaluation_plan]{mb_evaluation_plan()}}.

Other planning: 
\code{\link{mb_balance_table}()},
\code{\link{mb_counterfactual}()},
\code{\link{mb_evaluation_plan}()},
\code{\link{mb_questions}()}
}
\concept{planning}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporting.R
\name{mb_to_latex}
\alias{mb_to_latex}
\title{Render an evaluation report as a LaTeX table}
\usage{
mb_to_latex(report, caption = NULL, label = NULL)
}
\arguments{
\item{report}{An \code{mb_report} object.}

\item{caption}{Optional table caption.}

\item{label}{Optional LaTeX label for cross-referencing.}
}
\value{
A character scalar containing a LaTeX \code{tabular}
environment.
}
\description{
Returns a single LaTeX \code{tabular} summarising the report.
Multi-sheet Word/Excel exports are richer; LaTeX is intended for
insertion into a one-pager.
}
\examples{
toc <- mb_theory_of_change(
  inputs = "Funding", activities = "Workshops",
  outputs = "Attendees", outcomes = "Skills",
  impact = "Employment"
)
rep <- mb_evaluation_report(toc = toc, name = "Skills uplift")
cat(mb_to_latex(rep))
}
\seealso{
\code{\link[=mb_to_word]{mb_to_word()}}, \code{\link[=mb_to_excel]{mb_to_excel()}}.

Other reporting: 
\code{\link{mb_evaluation_report}()},
\code{\link{mb_to_excel}()},
\code{\link{mb_to_word}()}
}
\concept{reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power.R
\name{mb_power}
\alias{mb_power}
\title{Power for a two-sample test}
\usage{
mb_power(
  n_per_group,
  effect_size = NULL,
  sd = 1,
  alpha = 0.05,
  sides = 2L,
  type = c("mean", "proportion"),
  p1 = NULL,
  p2 = NULL
)
}
\arguments{
\item{n_per_group}{Numeric. Sample size per arm.}

\item{effect_size}{Numeric. The standardised effect size:
Cohen's \code{d} for \code{type = "mean"}, or Cohen's \code{h} for
\code{type = "proportion"} (computed automatically if \code{p1} and \code{p2}
are supplied).}

\item{sd}{Numeric. Standard deviation, used only for \code{type = "mean"}. Default \code{1}, in which case \code{effect_size} is interpreted
in standard deviation units.}

\item{alpha}{Numeric in \verb{(0, 1)}. Significance level. Default
\code{0.05}.}

\item{sides}{Integer. \code{2} (two-sided, default) or \code{1} (one-sided).}

\item{type}{Character. \code{"mean"} (default) or \code{"proportion"}.}

\item{p1, p2}{Optional numeric in \verb{(0, 1)}. If both supplied (and
\code{type = "proportion"}), the function computes Cohen's \code{h} and
ignores \code{effect_size}.}
}
\value{
Numeric scalar in \verb{(0, 1)}: the power.
}
\description{
Computes statistical power for a two-sample test of equal-sized
arms, using the large-sample normal approximation. Supports tests
of two means (with a common standard deviation) or two
proportions (using Cohen's \code{h} arcsine effect size).
}
\details{
For two means, power is
\deqn{1 - \Phi(z_{1-\alpha/s} - d\sqrt{n/2}) + \Phi(-z_{1-\alpha/s} - d\sqrt{n/2})}
where \eqn{s} is \code{sides} and \eqn{d} is the standardised effect.
For two proportions, the effect uses the arcsine variance-stabilising
transform: \eqn{h = 2\arcsin\sqrt{p_1} - 2\arcsin\sqrt{p_2}}.

Approximation note: this implementation uses the large-sample
normal approximation. The standard alternative (used by
\pkg{pwr}::\code{\link[pwr]{pwr.t.test}}) uses the noncentral
t-distribution. For typical evaluation sample sizes
(\code{n_per_group >= 50}) the two agree to within 1-2 percentage
points of power; for \code{n_per_group < 30} the discrepancy is
larger and \pkg{pwr} should be preferred. magentabook ships
equivalence tests against \pkg{pwr} (see
\code{tests/testthat/test-pwr-equivalence.R}).
}
\examples{
mb_power(n_per_group = 200, effect_size = 0.3)
mb_power(n_per_group = 500, type = "proportion", p1 = 0.40, p2 = 0.50)
}
\references{
Cohen, J. (1988). Statistical Power Analysis for the
Behavioral Sciences (2nd ed.). Lawrence Erlbaum.

Champely, S. (2020). pwr: Basic Functions for Power Analysis.
R package version 1.3-0.
\url{https://CRAN.R-project.org/package=pwr}.

HM Treasury (2020). The Magenta Book: Central Government
Guidance on Evaluation. Chapter on impact evaluation, section
on power analysis.
\url{https://www.gov.uk/government/publications/the-magenta-book}.
}
\seealso{
\code{\link[=mb_mde]{mb_mde()}}, \code{\link[=mb_sample_size]{mb_sample_size()}}, \code{\link[=mb_cluster_design]{mb_cluster_design()}}.

Other power: 
\code{\link{mb_cluster_design}()},
\code{\link{mb_icc_reference}()},
\code{\link{mb_mde}()},
\code{\link{mb_sample_size}()},
\code{\link{mb_stepped_wedge}()}
}
\concept{power}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/balance.R
\name{mb_balance_table}
\alias{mb_balance_table}
\title{Pre-treatment balance table}
\usage{
mb_balance_table(treated, ..., data = NULL, threshold = 0.1)
}
\arguments{
\item{treated}{Logical or 0/1 numeric vector identifying the
treated unit. \code{TRUE} / \code{1} means treated.}

\item{...}{Numeric or factor covariates to balance check. Names
become row labels. May be passed as a data frame via the
\code{data} argument.}

\item{data}{Optional data frame. If supplied, \code{...} is ignored
and every column other than \code{treated} is checked. Pass the
\code{treated} argument as a column reference (e.g. \code{data$treat})
or as the column name in the data frame.}

\item{threshold}{Numeric scalar. Absolute SMD threshold above
which a row is flagged as imbalanced. Default \code{0.10}.}
}
\value{
An \code{mb_balance_table} data frame with columns
\code{covariate}, \code{mean_treated}, \code{mean_control}, \code{sd_treated},
\code{sd_control}, \code{n_treated}, \code{n_control}, \code{smd}, \code{p_value},
\code{imbalanced}. Numeric and binary covariates use the
pooled-SD SMD and a Welch two-sample t-test. Factor covariates
are decomposed into one row per non-reference level using the
level-indicator and a chi-squared test on the original
factor.
}
\description{
Computes a Magenta Book-standard balance check for pre-treatment
covariates: by-arm mean and standard deviation, standardised
mean difference (SMD), and a two-sample test of equality. The
SMD is the unitless effect size most evaluators report; rules
of thumb flag \verb{|SMD| > 0.10} as a meaningful imbalance and
\verb{|SMD| > 0.25} as a serious imbalance.
}
\details{
For a numeric or 0/1 covariate \eqn{X} with treated mean
\eqn{\bar X_T}, control mean \eqn{\bar X_C}, treated SD
\eqn{s_T}, and control SD \eqn{s_C}, the standardised mean
difference is
\deqn{\text{SMD} = \frac{\bar X_T - \bar X_C}{\sqrt{(s_T^2 + s_C^2)/2}}.}

This is the equal-weighted pooled-SD form recommended by Stuart
(2010) and Austin (2009) for propensity-score balance
diagnostics. It differs from Cohen's d, which uses the
degrees-of-freedom-weighted pooled SD
\eqn{\sqrt{(s_T^2(n_T-1) + s_C^2(n_C-1))/(n_T+n_C-2)}}; the two
agree when \eqn{n_T = n_C}. magentabook ships a cross-validation
test against \code{cobalt::bal.tab} which uses the same averaged-SD
form.

Rules of thumb (Cohen 1988; Stuart 2010):
\itemize{
\item \verb{|SMD| < 0.10}: well balanced
\item \verb{0.10 <= |SMD| < 0.25}: meaningful imbalance, consider
covariate adjustment
\item \verb{|SMD| >= 0.25}: serious imbalance, matching or weighting
recommended
}

Magenta Book impact evaluation guidance recommends a balance
table for any quasi-experimental design and as a sense-check
even for randomised designs.
}
\examples{
set.seed(20260427)
n <- 400
treated <- rep(c(0, 1), each = n / 2)
age     <- rnorm(n, mean = 45 + 2 * treated, sd = 10)
female  <- rbinom(n, 1, 0.5)
income  <- rnorm(n, mean = 30000 + 1500 * treated, sd = 8000)
mb_balance_table(treated = treated, age = age, female = female, income = income)
}
\references{
Stuart, E. A. (2010). Matching methods for causal inference: A
review and a look forward. Statistical Science 25(1).
\url{doi:10.1214/09-STS313}.

Austin, P. C. (2009). Balance diagnostics for comparing the
distribution of baseline covariates between treatment groups
in propensity-score matched samples. Statistics in Medicine
28(25). \url{doi:10.1002/sim.3697}.

HM Treasury (2020). The Magenta Book, supplementary guidance on
quasi-experimental methods.
\url{https://www.gov.uk/government/publications/the-magenta-book}.
}
\seealso{
\code{\link[=mb_did_2x2]{mb_did_2x2()}}, \code{\link[=mb_questions]{mb_questions()}}.

Other planning: 
\code{\link{mb_counterfactual}()},
\code{\link{mb_evaluation_plan}()},
\code{\link{mb_questions}()},
\code{\link{mb_stakeholders}()}
}
\concept{planning}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confidence.R
\name{mb_confidence_summary}
\alias{mb_confidence_summary}
\title{One-page confidence summary across multiple ratings}
\usage{
mb_confidence_summary(...)
}
\arguments{
\item{...}{One or more \code{mb_confidence} objects, or a single
list of them.}
}
\value{
An \code{mb_confidence_summary} object: a list with \code{n}
(total ratings), \code{counts} (named integer vector by rating),
\code{ratings} (data frame), and \code{vintage}.
}
\description{
Aggregates several \code{mb_confidence} ratings into a single summary
object with a confidence count and the underlying ratings as a
data frame.
}
\examples{
c1 <- mb_confidence(
  "high",   "Did employment rise",
  "Two Level 5 RCTs", "Strong; randomisation worked",
  "Tested in two regions", "Two RCTs both positive"
)
c2 <- mb_confidence(
  "medium", "Did wages rise",
  "One Level 4 DiD",  "Adequate; parallel trends plausible",
  "Single region",    "DiD effect positive but no replication"
)
mb_confidence_summary(c1, c2)
}
\seealso{
\code{\link[=mb_confidence]{mb_confidence()}}.

Other confidence: 
\code{\link{mb_confidence}()}
}
\concept{confidence}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theory.R
\name{mb_theory_of_change}
\alias{mb_theory_of_change}
\title{Build a Magenta Book theory of change}
\usage{
mb_theory_of_change(
  inputs,
  activities,
  outputs,
  outcomes,
  impact,
  assumptions = NULL,
  external_factors = NULL,
  name = NULL
)
}
\arguments{
\item{inputs}{Character vector of resources committed to the
policy: funding, staff, infrastructure, partnerships.}

\item{activities}{Character vector of what the policy does with
those inputs: design, delivery, communication, enforcement.}

\item{outputs}{Character vector of direct, countable products of
the activities: training sessions delivered, leaflets posted,
payments made.}

\item{outcomes}{Character vector of changes the outputs produce
in the target population, typically over months to a few years:
behaviour change, attitudes, take-up.}

\item{impact}{Character vector of long-term, ultimate goals the
outcomes contribute to: poverty reduction, decarbonisation,
improved health.}

\item{assumptions}{Optional character vector of assumptions that
must hold for each level to translate into the next.}

\item{external_factors}{Optional character vector of contextual
factors outside the policy's control that may affect outcomes.}

\item{name}{Optional character scalar naming the policy or
programme.}
}
\value{
An \code{mb_toc} object: a list with one element per level
plus optional \code{assumptions}, \code{external_factors}, \code{name}, and
\code{vintage}.
}
\description{
Constructs a five-level logic model in the form set out by the
HM Treasury Magenta Book: inputs → activities → outputs →
outcomes → impact, with optional assumptions and external
factors.
}
\details{
The Magenta Book theory of change is the foundation for every
subsequent evaluation step. It makes the implicit causal chain
explicit so that evaluation questions can be tied to specific
levels and indicators can be defined.
}
\examples{
toc <- mb_theory_of_change(
  inputs     = c("GBP 50m grant", "12 FTE programme team"),
  activities = c("Design training", "Deliver workshops"),
  outputs    = c("500 workshops delivered", "8000 attendees"),
  outcomes   = c("Improved skills", "Increased confidence"),
  impact     = "Higher employment among target group",
  assumptions = "Workshops cause skills uplift",
  external_factors = "Macro labour market remains stable",
  name = "Skills uplift programme"
)
toc
}
\references{
HM Treasury (2020). The Magenta Book: Central Government
Guidance on Evaluation, chapter on theory-based evaluation.
\url{https://www.gov.uk/government/publications/the-magenta-book}.
}
\seealso{
\code{\link[=mb_logframe]{mb_logframe()}}, \code{\link[=mb_assumptions]{mb_assumptions()}}.

Other theory of change: 
\code{\link{mb_assumptions}()},
\code{\link{mb_logframe}()}
}
\concept{theory of change}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sms.R
\name{mb_sms_rate}
\alias{mb_sms_rate}
\title{Score a study against the Maryland Scientific Methods Scale}
\usage{
mb_sms_rate(level, study, design = NULL, notes = NULL)
}
\arguments{
\item{level}{Integer in \code{1:5}. The Maryland SMS level.}

\item{study}{Character scalar. Reference for the study being
rated (citation, URL, internal ID).}

\item{design}{Optional character scalar. Brief description of
the design (e.g. \code{"Cluster RCT, 80 schools"}).}

\item{notes}{Optional character scalar. Additional notes on
methodological strengths and weaknesses.}
}
\value{
An \code{mb_sms_rating} object: a list capturing the level,
study, design, notes, the corresponding rubric row, and
\code{vintage}.
}
\description{
Records an evidence rating against the 1-5 Maryland SMS, the
What Works Network's standard for grading impact evidence.
}
\details{
The Maryland SMS, originally developed by Sherman et al. (1997)
for crime-prevention research, is the foundation for evidence
ratings used by the College of Policing What Works Centre,
the Education Endowment Foundation, the Early Intervention
Foundation, and others. The Magenta Book adopts SMS as its
default for grading impact evidence.

Level 1: cross-sectional or before-after with no comparison.
Level 2: before-after with a non-equivalent comparison group.
Level 3: well-matched comparison across multiple units.
Level 4: comparison adjusting for unobservables (DiD, RD, IV,
ITS, synthetic control).
Level 5: random assignment.

Provenance note: numeric levels 1-5 are direct from Sherman et
al. (1997). The word labels (Weakest / Weak / Moderate / Strong
/ Strongest) follow What Works UK / Education Endowment
Foundation convention and are not direct quotations from the
original report. The design-examples and typical-use columns of
the bundled rubric are magentabook synthesis, intended as a
practitioner reference rather than a verbatim reproduction.
}
\examples{
mb_sms_rate(
  level  = 5,
  study  = "Card & Krueger (1994) NJ minimum wage",
  design = "Difference-in-differences with PA comparison",
  notes  = "Large N, but contested measurement"
)
}
\references{
Sherman, L. W., Gottfredson, D. C., MacKenzie, D. L., Eck, J.,
Reuter, P., Bushway, S. (1997). Preventing Crime: What Works,
What Doesn't, What's Promising. Report to the US Congress.

HM Treasury (2020). The Magenta Book: Central Government
Guidance on Evaluation.
}
\seealso{
\code{\link[=mb_sms_explain]{mb_sms_explain()}}, \code{\link[=mb_confidence]{mb_confidence()}}.

Other Maryland SMS: 
\code{\link{mb_sms_explain}()}
}
\concept{Maryland SMS}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power.R
\name{mb_icc_reference}
\alias{mb_icc_reference}
\title{Reference intra-class correlation values}
\usage{
mb_icc_reference(domain = NULL)
}
\arguments{
\item{domain}{Optional character scalar. One of \code{"education"},
\code{"health"}, \code{"employment"}, \code{"local_government"},
\code{"criminal_justice"}, \code{"housing"}. If \code{NULL} (default),
returns the entire reference table.}
}
\value{
A data frame with columns \code{domain}, \code{outcome},
\code{unit_of_clustering}, \code{icc_low}, \code{icc_central}, \code{icc_high},
\code{value_source}, \code{source}, \code{notes}.
}
\description{
Returns bundled reference ICC values for common UK policy
domains and units of clustering. Use these for evaluation
planning when domain-specific baseline data are not available.
}
\details{
Values are \emph{reference} ICCs for planning purposes only.
Wherever feasible, evaluators should compute domain-specific
ICCs from baseline data before finalising sample size
calculations.

Each row carries a \code{value_source} flag:
\itemize{
\item \code{"table_quote"}: direct extraction of a specific row or value
from a published table (cited table number in the \code{source}
field).
\item \code{"central_estimate"}: researcher synthesis of a plausible
central value within the published range, used as a
practitioner default in the absence of domain-specific
baseline data.
}

At v0.1.0 every bundled row is \code{central_estimate}. Future
versions will upgrade individual rows to \code{table_quote} as exact
table-level citations are added. Treat the bundled values as a
planning prior; verify against your own baseline ICC before
relying on them in a published power calculation.
}
\examples{
mb_icc_reference()
mb_icc_reference("education")
}
\references{
Hedges, L. V., Hedberg, E. C. (2007). Educational Evaluation
and Policy Analysis 29(1).
\url{doi:10.3102/0162373707299706}.

Adams, G., Gulliford, M. C., Ukoumunne, O. C., Eldridge, S.,
Chinn, S., Campbell, M. J. (2004). Patterns of intra-cluster
correlation from primary care research. Statistics in
Medicine 23. \url{doi:10.1002/sim.1764}.

Campbell, M. K., Mollison, J., Grimshaw, J. M. (2000). Cluster
trials in implementation research: estimation of intracluster
correlation coefficients and sample size. BMJ 321.
\url{doi:10.1136/bmj.321.7263.778}.
}
\seealso{
\code{\link[=mb_cluster_design]{mb_cluster_design()}}, \code{\link[=mb_stepped_wedge]{mb_stepped_wedge()}}.

Other power: 
\code{\link{mb_cluster_design}()},
\code{\link{mb_mde}()},
\code{\link{mb_power}()},
\code{\link{mb_sample_size}()},
\code{\link{mb_stepped_wedge}()}
}
\concept{power}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lookups.R
\name{mb_schedule_table}
\alias{mb_schedule_table}
\title{Expose internal lookup tables}
\usage{
mb_schedule_table(table = c("sms", "confidence", "icc", "questions"))
}
\arguments{
\item{table}{Character scalar. One of \code{"sms"}, \code{"confidence"},
\code{"icc"}, or \code{"questions"}.}
}
\value{
A data frame.
}
\description{
Returns one of the bundled lookup tables: the Maryland SMS
rubric, the Magenta Book confidence rubric, the ICC reference
table, or the evaluation question taxonomy.
}
\examples{
mb_schedule_table("sms")
mb_schedule_table("confidence")
mb_schedule_table("icc")
mb_schedule_table("questions")
}
\seealso{
\code{\link[=mb_data_versions]{mb_data_versions()}}.

Other lookups: 
\code{\link{mb_data_versions}()}
}
\concept{lookups}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magentabook-package.R
\docType{package}
\name{magentabook-package}
\alias{magentabook}
\alias{magentabook-package}
\title{magentabook: HM Treasury Magenta Book Policy Evaluation Primitives}
\description{
Implements the framework set out in HM Treasury's Magenta Book
(2020): theory of change, evaluation planning, power analysis,
Maryland SMS ratings, structured confidence ratings, light-weight
difference-in-differences and interrupted-time-series estimators,
and cost-effectiveness analysis. Designed as the evaluation
companion to the appraisal package \code{greenbook}.
}
\section{Function families}{

\itemize{
\item \strong{Theory of change}: \code{\link[=mb_theory_of_change]{mb_theory_of_change()}}, \code{\link[=mb_logframe]{mb_logframe()}},
\code{\link[=mb_assumptions]{mb_assumptions()}}.
\item \strong{Planning}: \code{\link[=mb_evaluation_plan]{mb_evaluation_plan()}}, \code{\link[=mb_questions]{mb_questions()}},
\code{\link[=mb_counterfactual]{mb_counterfactual()}}, \code{\link[=mb_stakeholders]{mb_stakeholders()}}.
\item \strong{Power and design}: \code{\link[=mb_power]{mb_power()}}, \code{\link[=mb_mde]{mb_mde()}},
\code{\link[=mb_sample_size]{mb_sample_size()}}, \code{\link[=mb_cluster_design]{mb_cluster_design()}},
\code{\link[=mb_stepped_wedge]{mb_stepped_wedge()}}, \code{\link[=mb_icc_reference]{mb_icc_reference()}}.
\item \strong{Maryland SMS}: \code{\link[=mb_sms_rate]{mb_sms_rate()}}, \code{\link[=mb_sms_explain]{mb_sms_explain()}}.
\item \strong{Confidence}: \code{\link[=mb_confidence]{mb_confidence()}}, \code{\link[=mb_confidence_summary]{mb_confidence_summary()}}.
\item \strong{Estimators}: \code{\link[=mb_did_2x2]{mb_did_2x2()}}, \code{\link[=mb_its]{mb_its()}}, \code{\link[=mb_event_study]{mb_event_study()}}.
\item \strong{Cost-effectiveness}: \code{\link[=mb_cea]{mb_cea()}}, \code{\link[=mb_icer]{mb_icer()}}, \code{\link[=mb_ceac]{mb_ceac()}},
\code{\link[=mb_inb]{mb_inb()}}, \code{\link[=mb_qaly]{mb_qaly()}}, \code{\link[=mb_daly]{mb_daly()}}.
\item \strong{Realist}: \code{\link[=mb_cmo]{mb_cmo()}}, \code{\link[=mb_contribution_claim]{mb_contribution_claim()}}.
\item \strong{Reporting}: \code{\link[=mb_evaluation_report]{mb_evaluation_report()}}, \code{\link[=mb_to_word]{mb_to_word()}},
\code{\link[=mb_to_excel]{mb_to_excel()}}, \code{\link[=mb_to_latex]{mb_to_latex()}}.
\item \strong{Lookups}: \code{\link[=mb_data_versions]{mb_data_versions()}}, \code{\link[=mb_schedule_table]{mb_schedule_table()}}.
}
}

\section{Reproducibility}{

Bundled rubric and reference tables in \verb{inst/extdata/} carry
vintage metadata accessible via \code{\link[=mb_data_versions]{mb_data_versions()}}. Every result
object records the package version it was produced under.
}

\references{
HM Treasury (2020). The Magenta Book: Central Government Guidance
on Evaluation. London: HMSO.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/charlescoverdale/magentabook}
  \item Report bugs at \url{https://github.com/charlescoverdale/magentabook/issues}
}

}
\author{
\strong{Maintainer}: Charles Coverdale \email{charlesfcoverdale@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theory.R
\name{mb_assumptions}
\alias{mb_assumptions}
\title{Build a structured assumption register}
\usage{
mb_assumptions(
  level,
  description,
  evidence = NA_character_,
  criticality = "medium"
)
}
\arguments{
\item{level}{Character vector. The theory-of-change level the
assumption sits at. One of \code{"inputs"}, \code{"activities"},
\code{"outputs"}, \code{"outcomes"}, \code{"impact"}.}

\item{description}{Character vector. Plain-English statement of
the assumption.}

\item{evidence}{Optional character vector. Source or rationale
for believing the assumption holds. Defaults to \code{NA}.}

\item{criticality}{Character vector. One of \code{"low"}, \code{"medium"},
\code{"high"}. Failure of high-criticality assumptions invalidates
the causal chain.}
}
\value{
An \code{mb_assumption_register} data frame with columns
\code{level}, \code{description}, \code{evidence}, \code{criticality}.
}
\description{
Captures one or more assumptions from a theory of change in a
tidy register, with the level they sit at, the supporting
evidence (or its absence), and a criticality rating.
}
\examples{
mb_assumptions(
  level       = c("activities", "outcomes"),
  description = c("Workshops are well-attended",
                  "Skills uplift translates into job entry"),
  evidence    = c("Pilot attendance 80\%",
                  "Indirect: similar programmes show 0.3 SD effect"),
  criticality = c("medium", "high")
)
}
\seealso{
\code{\link[=mb_theory_of_change]{mb_theory_of_change()}}, \code{\link[=mb_logframe]{mb_logframe()}}.

Other theory of change: 
\code{\link{mb_logframe}()},
\code{\link{mb_theory_of_change}()}
}
\concept{theory of change}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power.R
\name{mb_stepped_wedge}
\alias{mb_stepped_wedge}
\title{Stepped-wedge design effect}
\usage{
mb_stepped_wedge(steps, clusters_per_step, individuals_per_cluster, icc)
}
\arguments{
\item{steps}{Integer. Number of measurement periods (also called
\code{T}). Includes the baseline.}

\item{clusters_per_step}{Numeric. Number of clusters that
crossover at each step.}

\item{individuals_per_cluster}{Numeric. Individuals measured per
cluster per period.}

\item{icc}{Numeric in \verb{[0, 1]}. Intra-class correlation
coefficient.}
}
\value{
A list with elements \code{deff_cluster} (the within-period
cluster design effect), \code{correction_factor} (the stepped-wedge
correction relative to a parallel cluster RCT), \code{deff_sw} (the
product), and \code{n_total} (total observations across the trial).
}
\description{
Computes the design effect for a stepped-wedge cluster
randomised trial relative to an individually-randomised parallel
design with the same total observations.
}
\details{
Implements the closed-form approximation from Hemming et al.
(2015) BMJ Box 2:

Within-cluster design effect (cluster RCT vs individual RCT
with same total observations):
\deqn{\text{DEFF}_c = 1 + (mT - 1)\rho}

Stepped-wedge correction relative to a parallel cluster RCT:
\deqn{\text{CF} = \frac{3(1-\rho)}{2T(1 - 1/T^2)}}

Combined: \code{DEFF_sw = DEFF_c * CF}. This is a multiplier on the
variance of the treatment effect compared with an
individually-randomised design with the same total observations.

Approximation note: this is the closed-form approximation. The
exact Hussey-Hughes (2007) variance, which \code{swCRTdesign::swPwr}
computes from the design matrix, can differ by 20-40 percent for
typical UK evaluation designs. magentabook ships a
cross-validation test (\code{tests/testthat/test-swcrt-equivalence.R})
that documents the magnitude of this approximation gap on a
grid of designs. For production sample-size work, especially
where rho is high or the number of steps is small, prefer
\code{swCRTdesign::swPwr} or \code{clusterPower::cps.sw.binary} over this
function. Use \code{mb_stepped_wedge} for quick comparative
exploration; use the specialist packages for the number you
commit to in a published evaluation plan.

Both forms assume a balanced design: equal cluster size,
equal-period intervals, complete data, no time-by-treatment
interaction, and one outcome measurement per cluster-period.
For non-standard designs use the specialist packages above.
}
\examples{
mb_stepped_wedge(
  steps = 5,
  clusters_per_step = 4,
  individuals_per_cluster = 20,
  icc = 0.05
)
}
\references{
Hussey, M. A., Hughes, J. P. (2007). Design and analysis of
stepped wedge cluster randomized trials. Contemporary Clinical
Trials 28(2). \url{doi:10.1016/j.cct.2006.05.007}.

Woertman, W., de Hoop, E., Moerbeek, M., Zuidema, S. U.,
Gerritsen, D. L., Teerenstra, S. (2013). Stepped wedge designs
could reduce the required sample size in cluster randomized
trials. Journal of Clinical Epidemiology 66(7).
\url{doi:10.1016/j.jclinepi.2012.12.003}.

Hemming, K., Haines, T. P., Chilton, P. J., Girling, A. J.,
Lilford, R. J. (2015). The stepped wedge cluster randomised
trial: rationale, design, analysis, and reporting. BMJ 350.
\url{doi:10.1136/bmj.h391}.
}
\seealso{
\code{\link[=mb_cluster_design]{mb_cluster_design()}}, \code{\link[=mb_icc_reference]{mb_icc_reference()}}.

Other power: 
\code{\link{mb_cluster_design}()},
\code{\link{mb_icc_reference}()},
\code{\link{mb_mde}()},
\code{\link{mb_power}()},
\code{\link{mb_sample_size}()}
}
\concept{power}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power.R
\name{mb_cluster_design}
\alias{mb_cluster_design}
\title{Cluster-RCT design effect}
\usage{
mb_cluster_design(individuals_per_cluster, icc, n_clusters = NULL)
}
\arguments{
\item{individuals_per_cluster}{Numeric. Number of individuals
sampled per cluster (\code{m}).}

\item{icc}{Numeric in \verb{[0, 1]}. Intra-class correlation
coefficient. Use \code{\link[=mb_icc_reference]{mb_icc_reference()}} for plausible values.}

\item{n_clusters}{Optional numeric. Number of clusters per arm.
If supplied, returns effective sample size per arm in addition
to the design effect.}
}
\value{
A list with elements \code{deff} and (if \code{n_clusters}
supplied) \code{n_total_per_arm} and \code{n_effective_per_arm}.
}
\description{
Computes the design effect (DEFF) for a parallel cluster
randomised trial: how much the variance of the treatment effect
inflates relative to an individually-randomised design with the
same total sample size, due to within-cluster correlation.
}
\details{
\deqn{\text{DEFF} = 1 + (m - 1) \, \rho}
where \code{m} is the cluster size and \code{rho} is the ICC. The
effective sample size for power is \code{n_total / DEFF}.

Standard reference values for \code{rho} across UK policy domains
are bundled and accessible via \code{\link[=mb_icc_reference]{mb_icc_reference()}}.
}
\examples{
mb_cluster_design(individuals_per_cluster = 30, icc = 0.05)
mb_cluster_design(individuals_per_cluster = 30, icc = 0.05, n_clusters = 20)
}
\references{
Donner, A., Klar, N. (2000). Design and Analysis of Cluster
Randomization Trials in Health Research. Arnold.

Hedges, L. V., Hedberg, E. C. (2007). Intraclass Correlation
Values for Planning Group-Randomized Trials in Education.
Educational Evaluation and Policy Analysis 29(1).
\url{doi:10.3102/0162373707299706}.
}
\seealso{
\code{\link[=mb_icc_reference]{mb_icc_reference()}}, \code{\link[=mb_stepped_wedge]{mb_stepped_wedge()}},
\code{\link[=mb_sample_size]{mb_sample_size()}}.

Other power: 
\code{\link{mb_icc_reference}()},
\code{\link{mb_mde}()},
\code{\link{mb_power}()},
\code{\link{mb_sample_size}()},
\code{\link{mb_stepped_wedge}()}
}
\concept{power}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/did.R
\name{mb_did_2x2}
\alias{mb_did_2x2}
\title{Canonical 2x2 difference-in-differences estimator}
\usage{
mb_did_2x2(y, treated, post, cluster = NULL, alpha = 0.05, quiet = FALSE)
}
\arguments{
\item{y}{Numeric vector of outcomes.}

\item{treated}{Logical or 0/1 numeric vector. \code{TRUE} / \code{1} if
the unit is in the treated group, regardless of period.}

\item{post}{Logical or 0/1 numeric vector. \code{TRUE} / \code{1} if the
observation is in the post-treatment period.}

\item{cluster}{Optional vector identifying clusters for
cluster-robust standard errors (CR1 with finite-sample
correction). If \code{NULL}, conventional OLS SEs are returned.}

\item{alpha}{Numeric in \verb{(0, 1)}. Significance level for the
confidence interval. Default \code{0.05}.}

\item{quiet}{Logical. If \code{FALSE} (default), the print method
appends a one-line reminder that this is a canonical 2x2 DiD
and points to specialist tooling for staggered adoption or
heterogeneous treatment effects. Set to \code{TRUE} to suppress.}
}
\value{
An \code{mb_did} object: a list with \code{estimate}, \code{se},
\code{t_stat}, \code{p_value}, \code{ci_low}, \code{ci_high}, group means,
\code{cluster_robust}, \code{n}, \code{quiet}, and \code{vintage}.
}
\description{
Returns the simple two-period, two-group DiD estimate of an
average treatment effect on the treated, with optional
cluster-robust standard errors.
}
\details{
Computes
\deqn{\hat{\tau} = (\bar{Y}_{T,1} - \bar{Y}_{T,0}) - (\bar{Y}_{C,1} - \bar{Y}_{C,0})}
which equals the coefficient on the \code{treated:post} interaction in
\eqn{Y = \beta_0 + \beta_1 T + \beta_2 P + \tau (T \times P) + \epsilon}.

Cluster-robust SEs use the CR1 sandwich estimator with
finite-sample correction \eqn{(G/(G-1)) \cdot (N-1)/(N-K)}, where
\eqn{G} is the number of clusters, \eqn{N} the number of
observations, and \eqn{K} the number of regressors (4).

For staggered adoption, heterogeneous treatment effects, or
production estimation, use \pkg{fixest}, \pkg{did}, or
\pkg{Synth}. This function is for the canonical 2x2 case only.
}
\examples{
set.seed(1)
n <- 400
treated <- rep(c(0, 1), each = n / 2)
post    <- rep(c(0, 1), times = n / 2)
y       <- 0.5 * treated + 0.2 * post + 0.4 * treated * post + rnorm(n)
mb_did_2x2(y, treated, post)
}
\references{
Card, D., Krueger, A. B. (1994). Minimum Wages and Employment:
A Case Study of the Fast-Food Industry in New Jersey and
Pennsylvania. American Economic Review 84(4).
\url{doi:10.1257/aer.84.4.772}.

Cameron, A. C., Miller, D. L. (2015). A Practitioner's Guide to
Cluster-Robust Inference. Journal of Human Resources 50(2).
\url{doi:10.3368/jhr.50.2.317}.
}
\seealso{
\code{\link[=mb_its]{mb_its()}}, \code{\link[=mb_event_study]{mb_event_study()}}.

Other estimators: 
\code{\link{mb_event_study}()},
\code{\link{mb_its}()}
}
\concept{estimators}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power.R
\name{mb_mde}
\alias{mb_mde}
\title{Minimum detectable effect (MDE)}
\usage{
mb_mde(
  n_per_group,
  sd = 1,
  power = 0.8,
  alpha = 0.05,
  sides = 2L,
  type = c("mean", "proportion"),
  baseline = NULL
)
}
\arguments{
\item{n_per_group}{Numeric. Sample size per arm.}

\item{sd}{Numeric. Standard deviation, used only for \code{type = "mean"}. Default \code{1}, in which case \code{effect_size} is interpreted
in standard deviation units.}

\item{power}{Numeric in \verb{(0, 1)}. Target power. Default \code{0.8}.}

\item{alpha}{Numeric in \verb{(0, 1)}. Significance level. Default
\code{0.05}.}

\item{sides}{Integer. \code{2} (two-sided, default) or \code{1} (one-sided).}

\item{type}{Character. \code{"mean"} (default) or \code{"proportion"}.}

\item{baseline}{Optional numeric in \verb{(0, 1)}. For
\code{type = "proportion"}, the baseline proportion \code{p1} against
which the MDE is calculated. The MDE is then returned in
absolute proportion-point units.}
}
\value{
Numeric scalar. The minimum detectable effect in the
units implied by \code{type}: standard deviation units (\code{type = "mean"}, with \code{sd = 1}) or absolute proportion-point difference
(\code{type = "proportion"} with \code{baseline} supplied), or Cohen's \code{h}
(\code{type = "proportion"} without \code{baseline}).
}
\description{
Inverts \code{\link[=mb_power]{mb_power()}}: given a sample size, target power, and
significance level, returns the smallest effect size the design
can reliably detect.
}
\examples{
mb_mde(n_per_group = 200)
mb_mde(n_per_group = 500, type = "proportion", baseline = 0.4)
}
\seealso{
\code{\link[=mb_power]{mb_power()}}, \code{\link[=mb_sample_size]{mb_sample_size()}}.

Other power: 
\code{\link{mb_cluster_design}()},
\code{\link{mb_icc_reference}()},
\code{\link{mb_power}()},
\code{\link{mb_sample_size}()},
\code{\link{mb_stepped_wedge}()}
}
\concept{power}
